@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CustomService.cpp,v 1.4 2006/09/10 22:40:36 vkurland Exp $
+  $Id: CustomService.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -29,6 +29,7 @@
 
 #include <fwbuilder/libfwbuilder-config.h>
 
+#include <fwbuilder/FWObjectDatabase.h>
 #include <fwbuilder/CustomService.h>
 #include <fwbuilder/XMLTools.h>
 
@@ -68,7 +69,7 @@ void CustomService::fromXML(xmlNodePtr root) throw(FWException)
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("id")));
     if(n)
     {
-        setId(n);
+        setId(FWObjectDatabase::registerStringId(n));
         FREEXMLBUFF(n);
     }
 
@@ -101,6 +102,9 @@ xmlNodePtr CustomService::toXML(xmlNodePtr parent) throw(FWException)
     xmlNodePtr opt;
 
     xmlNodePtr me = FWObject::toXML(parent);
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
 
     map<string, string>::const_iterator i;
     for(i=codes.begin(); i!=codes.end(); ++i)  
