@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWObject.cpp,v 1.15 2007/07/07 05:36:22 vkurland Exp $
+  $Id: FWObject.cpp 194 2008-09-25 13:36:52Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -33,12 +33,19 @@
 #include <fwbuilder/FWObjectReference.h>
 #include <fwbuilder/Library.h>
 
+#include <fwbuilder/FWOptions.h>
+#include <fwbuilder/RuleSet.h>
+#include <fwbuilder/RuleElement.h>
+#include <fwbuilder/Rule.h>
+#include <fwbuilder/Host.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 // #include <unistd.h>
 #include <fcntl.h>
 
 #include <iostream>
+#include <sstream>
 #include <algorithm>
 #include <functional>
 #include <stack>
@@ -68,7 +75,7 @@ void FWObject::fromXML(xmlNodePtr root) throw(FWException)
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("id")));
     if(n)
     {
-        setId(n);
+        setId(FWObjectDatabase::registerStringId(n));
         FREEXMLBUFF(n);
     }
 
@@ -82,7 +89,7 @@ void FWObject::fromXML(xmlNodePtr root) throw(FWException)
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("ro")));
     if(n)
     {
-        setStr("ro",n);
+        ro = (cxx_strcasecmp(n, "1")==0 || cxx_strcasecmp(n , "true")==0);
         FREEXMLBUFF(n);
     }
 
@@ -119,9 +126,25 @@ xmlNodePtr FWObject::toXML(xmlNodePtr xml_parent_node) throw(FWException)
     return toXML(xml_parent_node, true);
 }
 
-xmlNodePtr FWObject::toXML(xmlNodePtr parent, bool process_children) throw(FWException)
+xmlNodePtr FWObject::toXML(xmlNodePtr parent, bool process_children)
+    throw(FWException)
 {
-    xmlNodePtr me = xmlNewChild(parent, NULL, xml_name.empty()?STRTOXMLCAST(getTypeName()):STRTOXMLCAST(xml_name), NULL);
+    string s_id = FWObjectDatabase::getStringId(getId());
+
+    xmlNodePtr me = xmlNewChild(
+        parent,
+        NULL,
+        xml_name.empty() ? STRTOXMLCAST(getTypeName()) : STRTOXMLCAST(xml_name),
+        NULL);
+
+    if (id!=-1)
+    {
+        xmlAttrPtr pr = xmlNewProp(
+            me, 
+            TOXMLCAST("id"),
+            STRTOXMLCAST(s_id));
+        xmlAddID(NULL, parent->doc, STRTOXMLCAST(s_id), pr);
+    }
 
     for(map<string, string>::const_iterator i=data.begin(); i!=data.end(); ++i) 
     {
@@ -129,16 +152,8 @@ xmlNodePtr FWObject::toXML(xmlNodePtr parent, bool process_children) throw(FWExc
         const string &value = (*i).second;
 
         if (name[0]=='.') continue;
-        xmlAttrPtr pr = xmlNewProp(me, 
-                                   STRTOXMLCAST(name),
-                                   STRTOXMLCAST(value));
-        if(name=="id")
-        {
-            xmlAddID(NULL, parent->doc, STRTOXMLCAST(value), pr);
-        } else if(name=="ref") 
-        {
-            xmlAddRef(NULL, parent->doc, STRTOXMLCAST(value), pr);
-        }
+
+        xmlNewProp(me, STRTOXMLCAST(name), STRTOXMLCAST(value));
     }
 
     if(process_children)
@@ -155,29 +170,51 @@ FWObject::FWObject()
     ref_counter = 0;
     parent      = NULL;
     dbroot      = NULL;
+    name        = "";
+    comment     = "";
+    id          = -1;
+    ro          = false;
 
-    setName("New object");
-    
-    // When object created we assign it unique Id
+    // When object is created we assign it unique Id
     setId(FWObjectDatabase::generateUniqueId());
 
     setDirty(false);
 }
 
+FWObject::FWObject(bool new_id)
+{
+    init        = false;
+    ref_counter = 0;
+    parent      = NULL;
+    dbroot      = NULL;
+    name        = "";
+    comment     = "";
+    id          = -1;
+    ro          = false;
+
+    // When object created we assign it unique Id
+    if (new_id)
+        setId(FWObjectDatabase::generateUniqueId());
+
+    setDirty(false);
+}
+
 FWObject::FWObject(const FWObject &c) : list<FWObject*>(c)
 {
     init = false;
-    *this=c;
+    *this = c;
 }
 
-FWObject::FWObject(const FWObject *root,bool prepopulate)
+FWObject::FWObject(const FWObject *root, bool )
 {
     init        = false;
     ref_counter = 0    ;
     parent      = NULL ;
     dbroot      = (FWObject*)root;
-    
-    setName("New object");
+    name        = "";
+    comment     = "";
+    id          = -1;
+    ro          = false;
     
     // When object created we assign it unique Id
     setId(FWObjectDatabase::generateUniqueId());
@@ -230,6 +267,24 @@ FWObject* FWObject::findObjectByName(const string &type,
     return NULL; // not found
 }
 
+FWObject* FWObject::findObjectByAttribute(const std::string &attr,
+                                          const std::string &val)
+    throw(FWException)
+{
+    if (getStr(attr)==val) return this;
+
+    list<FWObject*>::iterator j;
+    for(j=begin(); j!=end(); ++j)     
+    {
+        FWObject *o=*j;
+
+        o=o->findObjectByAttribute(attr, val);
+        if(o) return o;
+    }
+
+    return NULL; // not found
+}
+
 
 bool FWObject::cmp(const FWObject *obj) throw(FWException)
 {
@@ -286,20 +341,13 @@ FWObject& FWObject::operator=(const FWObject &x)  throw(FWException)
     return duplicate(&x, false);
 }
 
-FWObject& FWObject::duplicate(const FWObject *x, bool preserve_id) throw(FWException)
+FWObject& FWObject::duplicate(const FWObject *x, bool preserve_id)
+    throw(FWException)
 {
     checkReadOnly();
-    bool xro=x->getBool("ro"); 
-#if 0
-    cerr << "--------------------------------" << endl;
-    cerr << "this: " << endl;
-    dump(true,true);
-    cerr << endl;
-    cerr << "x: " << endl;
-    ((FWObject*)x)->dump(true,true);
-    cerr << endl;
-#endif
-    shallowDuplicate(x,preserve_id);
+    bool xro = x->ro;
+
+    shallowDuplicate(x, preserve_id);
 
     if (xro) setReadOnly(false);
 
@@ -317,7 +365,8 @@ FWObject& FWObject::duplicate(const FWObject *x, bool preserve_id) throw(FWExcep
     return *this;
 }
 
-FWObject* FWObject::addCopyOf(const FWObject *x, bool preserve_id) throw(FWException)
+FWObject* FWObject::addCopyOf(const FWObject *x, bool preserve_id)
+    throw(FWException)
 {
     if (x==NULL) return NULL;
     FWObject *o1;
@@ -325,12 +374,14 @@ FWObject* FWObject::addCopyOf(const FWObject *x, bool preserve_id) throw(FWExcep
     if (root==NULL) root = x->getRoot();
     // do not prepopulte children for objects that do that automatically
     // in their constructor
-    o1 = root->create(x->getTypeName(),"",false);
+    o1 = root->create(x->getTypeName(), -1, false);
     if(!o1)
         throw FWException(string("Error creating object with type: ")+
                           x->getTypeName());
     add(o1);
+
     o1->duplicate(x, preserve_id);
+
     return o1;
 }
 
@@ -342,42 +393,50 @@ FWObject* FWObject::addCopyOf(const FWObject *x, bool preserve_id) throw(FWExcep
  * 07/06/2007 -- changed that. Now copy has the same value of "ro"
  * attribute.  Clear it in the caller if neccessary.
  */
-FWObject& FWObject::shallowDuplicate(const FWObject *x, bool preserve_id) throw(FWException)
+FWObject& FWObject::shallowDuplicate(const FWObject *x, bool preserve_id)
+    throw(FWException)
 {
     checkReadOnly();
 
-    string id = getId();
+    int old_id = getId();
 
+    id = x->id;
+    name = x->name;
+    comment = x->comment;
+    ro = x->ro;
     data = x->data;
-    bool ro_status = getBool("ro");
-    if (ro_status) setReadOnly(false);
+
+    setReadOnly(false);
 
     //if (getBool("ro")) setReadOnly(false);
     if (!preserve_id)
     {
         ref_counter = 0           ;
         xml_name    = x->xml_name ;
-    } else if(id!="")   // some objects do not have ID per DTD (e.g. Src, Dst, etc.)
-	setId(id);
+    } else
+    {
+        // some objects do not have ID per DTD (e.g. Src, Dst, etc.)
+        // Those will return -1 from getId()
+        if (id > -1)
+            setId(old_id);
+    }
 
     if (dbroot==NULL) setRoot(x->getRoot());
     if (dbroot!=NULL) FWObjectDatabase::cast(dbroot)->addToIndex(this);
 
-    // do setReadOnly only if needed; this way objects that
-    // do not have this attribute won't get it accidentally.
-    if (ro_status) setReadOnly(true);
+    setReadOnly(x->ro);
     setDirty(true);
     return *this;
 }
 
 const string &FWObject::getName() const 
 { 
-    return getStr("name"); 
+    return name;
 }
 
 void FWObject::setName(const string &n)   
 {
-    setStr("name",n);
+    name = n;
     setDirty(true);
 }
 
@@ -396,12 +455,6 @@ FWObject*  FWObject::getLibrary() const
 FWObjectDatabase* FWObject::getRoot() const
 {
     return FWObjectDatabase::cast(dbroot);
-#if 0
-    const FWObject *p=this;
-
-    while(p->getParent()!=NULL) p=p->getParent();
-    return FWObjectDatabase::cast((FWObject*)p);
-#endif
 }
 
 string FWObject::getPath(bool relative) const
@@ -423,26 +476,26 @@ string FWObject::getPath(bool relative) const
 
 const string& FWObject::getComment() const
 { 
-    return getStr("comment"); 
+    return comment;
 }
 
 void FWObject::setComment(const string &c)
 {
-    setStr("comment",c);
+    comment = c;
     setDirty(true);
 }
 
-const string &FWObject::getId() const
+int FWObject::getId() const
 { 
-    return getStr("id");
+    return id;
 }
 
 /*
  * need to update index because ID of the object changes
  */
-void FWObject::setId(const string &c)
+void FWObject::setId(int c)
 {
-    setStr("id",c);
+    id = c;
     setDirty(true);
     if (dbroot!=NULL)
         FWObjectDatabase::cast(dbroot)->addToIndex(this);
@@ -455,6 +508,12 @@ bool FWObject::exists(const string &name) const
 
 const string &FWObject::getStr(const string &name) const
 {
+    map<string,string>::const_iterator i=data.find(name);
+    if (i==data.end())
+        return NOT_FOUND;
+    else
+        return (*i).second;
+#if 0
     if(exists(name)) 
     {
         map<string,string>::const_iterator i=data.find(name);
@@ -463,6 +522,7 @@ const string &FWObject::getStr(const string &name) const
     {
         return NOT_FOUND;
     }
+#endif
 }
 
 void FWObject::remStr(const string &name)
@@ -483,9 +543,8 @@ void FWObject::remStr(const string &name)
 void FWObject::setStr(const string &name, const string &val)
 {
     if (name[0]!='.') checkReadOnly();
-
-    data[name]=val;
-    setDirty(true);
+    data[name] = val;
+    if (name[0]!='.') setDirty(true);
 }
 
 int FWObject::getInt(const string &name) const
@@ -499,18 +558,21 @@ int FWObject::getInt(const string &name) const
 
 void FWObject::setInt(const string &name, int val)
 {
-    char str[128];
-    sprintf(str,"%d",val);
-    setStr(name, str);
-    setDirty(true);
+    ostringstream str;
+    str << val;
+    setStr(name, str.str());
+    // attribute with name that starts with "." is considered "hidden"
+    // or "internal". Such attribute is not saved to the data file and
+    // should not trigger "dirty" flag.
+    if (name[0]!='.') setDirty(true);
 }
 
 bool FWObject::getBool(const string &name) const
 {
     if(exists(name)) 
     {
-		return(getStr(name)=="1" || 
-               cxx_strcasecmp(getStr(name).c_str() , "true")==0);
+        string val = getStr(name);
+        return (val=="1" || cxx_strcasecmp(val.c_str() , "true")==0);
     } else
         return false;
 }
@@ -518,7 +580,7 @@ bool FWObject::getBool(const string &name) const
 void FWObject::setBool(const string &name, bool val)
 {
     setStr(name, (val)?"True":"False");
-    setDirty(true);
+    if (name[0]!='.') setDirty(true);
 }
 
 void FWObject::setBool(const string &name, const string &val)
@@ -540,20 +602,22 @@ void FWObject::Hide()
 
 
 
-void FWObject::dump(bool recursive,bool brief,int offset)
+void FWObject::dump(bool recursive,bool brief,int offset) 
 {
     dump(cerr,recursive,brief,offset);
 }
 
-void FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset)
+void FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset) 
 {
     FWObject *o;
     string    n;
 
-    if (brief) {
+    if (brief)
+    {
 	f << string(offset,' ');
 	f << " Obj=" << this;
-	f << " ID="  << getId();
+	f << " ID="  << getId()
+          << " (" << FWObjectDatabase::getStringId(getId()) << ")";
 	f << " Name=" << getName();
 	f << " Type=" << getTypeName();
         if (this!=getRoot())
@@ -561,24 +625,24 @@ void FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset)
 	f << " Root=" << getRoot();
         f << " ref_counter=" << ref_counter;
 
-	if (FWReference::cast(this)!=0) {
-	    f << " Ref=" << FWReference::cast(this)->getPointer();
+	if (FWReference::constcast(this)!=0)
 	    f << " RefID=" << FWReference::cast(this)->getPointerId();
-	}
 
 	f << endl;
 
 	if (recursive) {
-	    list<FWObject*>::iterator m;
+	    list<FWObject*>::const_iterator m;
 	    for (m=begin(); m!=end(); ++m) {
 		if (  (o=(*m))!=NULL)  o->dump(f,recursive,brief,offset+2);
 	    }
 	}
-    } else {
-
+    } else
+    {
 	f << string(offset,' ') << string(16,'-') << endl;
 	f << string(offset,' ') << "Obj:    " << this << endl;
-	f << string(offset,' ') << "ID:     " << getId() << endl;
+	f << string(offset,' ') << "ID:     " << getId()
+          << " (" << FWObjectDatabase::getStringId(getId()) << ")"
+          << endl;
 	f << string(offset,' ') << "Name:   " << getName() << endl;
 	f << string(offset,' ') << "Ref.ctr:" << ref_counter << endl;
 	f << string(offset,' ') << "Type:   " << getTypeName() << endl;
@@ -589,16 +653,17 @@ void FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset)
 	  << "  name=" << n << endl;
 	f << string(offset,' ') << "Root:   " << getRoot() << endl;
 
-	map<string, string>::iterator d;
-	for (d=data.begin(); d!=data.end(); ++d) {
-	    if((*d).first=="name") 
-                continue;
+	map<string, string>::const_iterator d;
+	for (d=data.begin(); d!=data.end(); ++d)
+        {
 	    f << string(offset,' ');
 	    f << (*d).first << ": " << (*d).second << endl;
 	}
-	if (recursive) {
-	    list<FWObject*>::iterator m;
-	    for (m=begin(); m!=end(); ++m) {
+	if (recursive)
+        {
+	    list<FWObject*>::const_iterator m;
+	    for (m=begin(); m!=end(); ++m)
+            {
 		if (  (o=(*m))!=NULL)  o->dump(f,recursive,brief,offset+2);
 	    }
 	}
@@ -623,14 +688,14 @@ void FWObject::_adopt(FWObject *obj)
     obj->setRoot(getRoot());
 }
 
-void FWObject::addAt(const string& where_id, FWObject *obj)
+void FWObject::addAt(int where_id, FWObject *obj)
 {
     FWObject *p=getRoot()->findInIndex( where_id );
     assert (p!=NULL);
     p->add(obj);
 }
 
-void FWObject::add(FWObject *obj,bool validate)
+void FWObject::add(FWObject *obj, bool validate)
 {
     checkReadOnly();
 
@@ -729,12 +794,12 @@ void FWObject::swapObjects(FWObject *o1, FWObject *o2)
 void FWObject::_moveToDeletedObjects(FWObject *obj)
 {
     FWObjectDatabase *root = getRoot();
-    FWObject *dobj = root->findInIndex( root->getDeletedObjectsId() );
+    FWObject *dobj = root->findInIndex( FWObjectDatabase::DELETED_OBJECTS_ID );
             
     if (dobj==NULL)
     {
         dobj=root->create(Library::TYPENAME);
-        dobj->setId(root->getDeletedObjectsId());
+        dobj->setId(FWObjectDatabase::DELETED_OBJECTS_ID);
         dobj->setName("Deleted Objects");
         dobj->setReadOnly(false);
         root->add(dobj);
@@ -761,7 +826,7 @@ void FWObject::remove(FWObject *obj, bool delete_if_last)
 
         if (delete_if_last &&
             obj->ref_counter==0 &&
-            getId()!=FWObjectDatabase::getDeletedObjectsId())
+            getId()!=FWObjectDatabase::DELETED_OBJECTS_ID)
             _moveToDeletedObjects(obj);
     }
 }
@@ -779,7 +844,7 @@ void FWObject::removeAllInstances(FWObject *rm)
     checkReadOnly();
 
     bool deletedObject =
-        (rm->getParent()->getId()==FWObjectDatabase::getDeletedObjectsId());
+        (rm->getParent()->getId()==FWObjectDatabase::DELETED_OBJECTS_ID);
     removeAllReferences(rm);
     _removeAll(rm);
 
@@ -791,7 +856,7 @@ void FWObject::removeAllInstances(FWObject *rm)
 
 void FWObject::findAllReferences(const FWObject *obj, std::set<FWReference*> &res)
 {
-    string obj_id=obj->getId();
+    int obj_id = obj->getId();
     for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
     {
         FWObject *o=*m;
@@ -816,7 +881,7 @@ set<FWReference*> FWObject::findAllReferences(const FWObject *obj)
 
 void FWObject::removeRef(FWObject *obj)
 {
-    string obj_id=obj->getId();
+    int  obj_id=obj->getId();
     for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
     {
         FWObject *o=*m;
@@ -963,7 +1028,7 @@ bool FWObject::isChildOf(FWObject *obj)
     return (p==obj);
 }
 
-FWObject* FWObject::getById  (const string &id, bool recursive)
+FWObject* FWObject::getById  (int id, bool recursive)
 {
     if(id==getId())  return this;
     
@@ -971,7 +1036,7 @@ FWObject* FWObject::getById  (const string &id, bool recursive)
     for(j=begin(); j!=end(); ++j)     
     {
         FWObject *o=*j;
-        string oid=o->getId();
+        int oid = o->getId();
         if(id==oid) return o;
 
         if(recursive && (o=o->getById(id, true))!=NULL ) return o;
@@ -979,7 +1044,6 @@ FWObject* FWObject::getById  (const string &id, bool recursive)
     return NULL; // not found
 }
 
-
 FWObject* FWObject::getFirstByType(const string &type_name) const
 {
     const_iterator i=find_if(begin(),end(), FWObjectTypeNameEQPredicate(type_name));
@@ -1011,8 +1075,8 @@ void  FWObject::setDirty(bool f)
 {
     FWObjectDatabase *dbroot = getRoot();
     if (dbroot==NULL) return;
-    if (dbroot==this) dirty=f;
-    else              dbroot->setDirty(f);
+    if (dbroot==this) dirty = f;
+    else              dbroot->dirty = f;
 }
 
 bool FWObject::isDirty()
@@ -1028,16 +1092,15 @@ bool FWObject::isDirty()
  */
 void FWObject::setReadOnly(bool f)
 {
-    data["ro"]= (f)?"True":"False" ;
+    ro = f;
     FWObjectDatabase *dbroot = getRoot();
     if (dbroot)
     {
-        bool ri=dbroot->init;
-        dbroot->init=true;
+        bool ri = dbroot->init;
+        dbroot->init = true;
         setDirty(true);
-        dbroot->init=ri;
+        dbroot->init = ri;
     }
-//    setBool("ro",f);
 }
 
 /*
@@ -1056,26 +1119,27 @@ bool FWObject::isReadOnly()
     FWObject *p=this;
     while (p)
     {
-        if (p->getBool("ro")) return true;
-        p=p->getParent();
+        if (p->ro) return true;
+        p = p->getParent();
     }
     return false;
 }
 
 void FWObject::checkReadOnly() throw(FWException)
 {
-    if (isReadOnly())
-        throw FWException(
-            string("Attempt to modify read-only object ")+getName());
+    if (isReadOnly()) throw FWException(
+        string("Attempt to modify read-only object ")+getName());
 }
 
-
-FWObjectTypedChildIterator::FWObjectTypedChildIterator(const FWObjectTypedChildIterator &o):
-    type_name(o.type_name),real_iterator(o.real_iterator), _begin(o._begin), _end(o._end)
+FWObjectTypedChildIterator::FWObjectTypedChildIterator(
+    const FWObjectTypedChildIterator &o) :
+    type_name(o.type_name),
+    real_iterator(o.real_iterator), _begin(o._begin), _end(o._end)
 {
 }
 
-FWObjectTypedChildIterator::FWObjectTypedChildIterator(const FWObject *o, const std::string &_type_name)
+FWObjectTypedChildIterator::FWObjectTypedChildIterator(
+    const FWObject *o, const std::string &_type_name)
 {
     type_name     = _type_name ;
     _end          = o->end()   ;
@@ -1086,12 +1150,14 @@ FWObjectTypedChildIterator::FWObjectTypedChildIterator(const FWObject *o, const
     _begin = real_iterator;
 }
 
-bool FWObjectTypedChildIterator::operator==(const FWObject::const_iterator& __x) const 
+bool FWObjectTypedChildIterator::operator==(
+    const FWObject::const_iterator& __x) const 
 { 
     return real_iterator == __x; 
 }
 
-bool FWObjectTypedChildIterator::operator!=(const FWObject::const_iterator& __x) const 
+bool FWObjectTypedChildIterator::operator!=(
+    const FWObject::const_iterator& __x) const 
 { 
     return real_iterator != __x; 
 }
@@ -1119,12 +1185,13 @@ FWObjectTypedChildIterator& FWObjectTypedChildIterator::operator--()
     do
     {
         real_iterator--;
-    } while(real_iterator!=_begin && (real_iterator==_end || (*real_iterator)->getTypeName()!=type_name));
+    } while(real_iterator!=_begin &&
+           (real_iterator==_end || (*real_iterator)->getTypeName()!=type_name));
     return *this;
 }
 
 /* this is a postfix operator */
-FWObject::tree_iterator FWObject::tree_iterator::operator++(int n)
+FWObject::tree_iterator FWObject::tree_iterator::operator++(int )
 {
     FWObject::tree_iterator __tmp = *this;
     ++*this;
@@ -1217,3 +1284,70 @@ FWObject::tree_iterator FWObject::tree_end()
     return FWObject::tree_iterator( (FWObject*)(-1) );
 }
 
+/*
+ * find all references to object with id "old_id" in objects in rs
+ * (recursively) and replace them with references to object with id
+ * "new_id" Use this to find all references to old firewall with
+ * references to the new one when copying policy of the old one into
+ * the new one.
+ */
+int FWObject::replaceRef(int old_id, int new_id)
+{
+    int ref_replacement_counter = 0;
+    _replaceRef(this, old_id, new_id, ref_replacement_counter);
+    return ref_replacement_counter;
+}
+
+void FWObject::_replaceRef(FWObject *rs, int old_id, int new_id,
+                           int &counter)
+{
+    FWReference *ref = FWReference::cast(rs);
+    if (ref==NULL)
+    {
+        for (FWObject::iterator j1=rs->begin(); j1!=rs->end(); ++j1)
+            _replaceRef( *j1, old_id, new_id, counter);
+    } else
+    { 
+        if (ref->getPointerId()==old_id)
+        {
+            ref->setPointerId(new_id);
+            counter++;
+        }
+    }
+}
+
+void FWObject::findDependencies(list<FWObject*> &deps)
+{
+    int loop_id = time(NULL);
+    _findDependencies_internal(this, deps, loop_id);
+}
+
+void FWObject::_findDependencies_internal(FWObject *obj,
+                                          list<FWObject*> &deps,
+                                          int anti_loop_id)
+{
+    if (obj==NULL) return;
+    if (FWOptions::cast(obj)) return;
+    if (FWReference::cast(obj)!=NULL)
+    {
+        _findDependencies_internal(FWReference::cast(obj)->getPointer(),
+                                   deps, anti_loop_id);
+    } else
+    {
+        if (obj->getInt(".anti_loop")==anti_loop_id) return;
+        obj->setInt(".anti_loop", anti_loop_id);
+
+        if (!RuleElement::cast(obj) && !Rule::cast(obj) && !RuleSet::cast(obj))
+            deps.push_back(obj);
+
+        if (Group::cast(obj) || RuleSet::cast(obj) || Host::cast(obj))
+        {
+            for (FWObject::iterator j1=obj->begin(); j1!=obj->end(); ++j1)
+            {
+                FWObject *o = *j1;
+                _findDependencies_internal(o, deps, anti_loop_id);
+            }
+        }
+    }
+}
+
