@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWObject.h,v 1.3 2006/03/05 04:15:42 vkurland Exp $
+  $Id: FWObject.h 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -38,6 +38,7 @@
 #include <fstream>
 #include <cstdlib>
 
+#include <fwbuilder/libfwbuilder-config.h>
 #include <fwbuilder/FWException.h>
 
 namespace libfwbuilder
@@ -66,6 +67,7 @@ class FWObjectTypedChildIterator;
  */
 class FWObject : public std::list<FWObject*>
 {
+    friend class libfwbuilder::FWObjectDatabase;
 
 private:
 
@@ -89,7 +91,11 @@ private:
      *
      * dbroot is assigned by method FWObjectDatabase::create 
      */
-    FWObject      *dbroot;
+    FWObject *dbroot;
+    int id;
+    bool ro;
+    std::string name;
+    std::string comment;
 
     static std::string NOT_FOUND;
 
@@ -101,10 +107,24 @@ protected:
     
     std::map<std::string, std::string> data;
 
-    void      _removeAll(FWObject *rm);
-    void      _removeAllRef(FWObject *rm);
-    void      _adopt(FWObject *obj);   // increments reference
-    void      _moveToDeletedObjects(FWObject *obj);
+    void clearRefCounter() { ref_counter=0; }
+
+    /**
+     * _getRO() simply returns value of private member ro. Note the
+     * difference between _getRO and isReadOnly() - the latter checks
+     * if any parent of this object is read-only in addition to
+     * checking if this object is read-only itself.
+     */
+    bool _getRO() { return ro; }
+
+    void _removeAll(FWObject *rm);
+    void _removeAllRef(FWObject *rm);
+    void _adopt(FWObject *obj);   // increments reference
+    void _moveToDeletedObjects(FWObject *obj);
+    void _replaceRef(FWObject *rs, int oldfw_id, int newfw_id, int &counter);
+    void _findDependencies_internal(FWObject *obj,
+                                    std::list<FWObject*> &deps,
+                                    int anti_loop_id);
     
     /**
      * deletes all objects in the tree regardless of their usage counter
@@ -125,11 +145,14 @@ protected:
     
     FWObject();
 
+    // special constructor used to create FWObjectDatabase objects
+    FWObject(bool new_id);
+    
     /**
      *  Parameter 'root' is a pointer at the root object of the tree,
      *  we are going to add this newly created object to.
      */
-    FWObject(const FWObject *root);
+    FWObject(const FWObject *root,bool prepopulate);
 
 public:
 
@@ -159,8 +182,12 @@ public:
 
     FWObject(const FWObject &copy);
 
-    virtual void       fromXML    (xmlNodePtr xml_parent_node) throw(FWException);
-    virtual xmlNodePtr toXML      (xmlNodePtr xml_parent_node) throw(FWException);
+    int  getId() const;
+    void setId(int i);
+    bool haveId() { return (id != -1); }
+
+    virtual void fromXML(xmlNodePtr xml_parent_node) throw(FWException);
+    virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
     xmlNodePtr toXML(xmlNodePtr xml_parent_node, bool process_children) throw(FWException);
 
     /**
@@ -235,9 +262,6 @@ public:
     const std::string &getComment() const;
     void          setComment(const std::string& c);
 
-    const std::string  &getId() const;
-    void           setId(const std::string& c);
-
     /**
      * convenience method: returns the name of the library this object belongs to.
      * In fwbuilder2 library is an element directly under FWObjectDatabase; all
@@ -269,7 +293,7 @@ public:
      */
     std::string getPath(bool relative=false) const;
 
-            void addAt(const std::string& where_id, FWObject *obj);
+    void addAt(int where_id, FWObject *obj);
     virtual void add(FWObject *obj,bool validate=true);
     virtual void insert_before(FWObject *o1,FWObject *obj);
     virtual void insert_after(FWObject *o1,FWObject *obj);
@@ -355,7 +379,7 @@ public:
      * and itslef. If it is true, method proceeds to all chidren of all
      * objects starting from current one.
      */
-    virtual FWObject* getById  (const std::string &id, bool recursive=false);
+    virtual FWObject* getById  (int id, bool recursive=false);
 
 
     /**
@@ -383,6 +407,26 @@ public:
      */
     FWObject* findObjectByName(const std::string &type,
                                const std::string &name) throw(FWException);
+
+    /**
+     * finds a child object of a given type with an attribute attr
+     */
+    FWObject* findObjectByAttribute(const std::string &attr,
+                                    const std::string &val) throw(FWException);
+
+    /**
+     * if this object has any references as its children, replace IDs these
+     * references point to.
+     */
+    int replaceRef(int oldfw_id, int newfw_id);
+
+    /**
+     * recursively find all FWReference objects that are children of
+     * this and generate list of pointers to the objects these
+     * references point to.
+     */
+    void findDependencies(std::list<FWObject*> &deps);
+
     
     virtual void    setDirty(bool f);
     virtual bool    isDirty();
