@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: FWObjectDatabase.cpp,v 1.20 2006/10/15 21:43:55 vkurland Exp $
+  $Id: FWObjectDatabase.cpp 193 2008-09-24 13:20:49Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -30,6 +30,7 @@
 #include <time.h>
 #include <assert.h>
 #include <stdlib.h>
+#include <cstring>
 #include <signal.h>
 
 #include <fwbuilder/libfwbuilder-config.h>
@@ -41,6 +42,7 @@
 #include <fwbuilder/Library.h>
 #include <fwbuilder/Interval.h>
 #include <fwbuilder/ICMPService.h>
+#include <fwbuilder/ICMP6Service.h>
 #include <fwbuilder/IPService.h>
 #include <fwbuilder/TCPService.h>
 #include <fwbuilder/UDPService.h>
@@ -52,6 +54,7 @@
 #include <fwbuilder/Host.h>
 #include <fwbuilder/Interface.h>
 #include <fwbuilder/IPv4.h>
+#include <fwbuilder/IPv6.h>
 #include <fwbuilder/physAddress.h>
 #include <fwbuilder/DNSName.h>
 #include <fwbuilder/AddressTable.h>
@@ -69,22 +72,39 @@
 #include <fwbuilder/ServiceGroup.h>
 #include <fwbuilder/IntervalGroup.h>
 #include <fwbuilder/Network.h>
+#include <fwbuilder/NetworkIPv6.h>
 #include <fwbuilder/AddressRange.h>
 #include <fwbuilder/Management.h>
 #include <fwbuilder/XMLTools.h>
 #include <fwbuilder/TagService.h>
+#include <fwbuilder/UserService.h>
 
 #include <iostream>
+#include <sstream>
 
 using namespace std;
 using namespace libfwbuilder;
 
-IDcounter::IDcounter()
-{
-    cntr=time(NULL);
-}
+// each program invocation tracks its own set of object ids (as int)
+// we just increment this counter to get new id. We also keep a dictionary
+// that maps integer ids to strings. This dictionary is populated when
+// objects are loaded from xml file and then used to write string ids
+// when objects are written back to file. Internally we operate with
+// integer ids all the time, string ids are only used in xml file.
+// "System" objects use ids < 1000.
 
-IDcounter          IDc;
+int id_seed = 1000;
+#ifdef _WIN32
+static int cached_pid = _getpid();
+#else
+static int cached_pid = getpid();
+#endif
+
+// these two dictionaries must be static to ensure uniqueness of integer
+// ids across multiple FWObjectDatabase objects
+
+map<int, string>       id_dict;
+map<string, int>       id_dict_reverse;
 
 const char*        FWObjectDatabase::TYPENAME  = {"FWObjectDatabase"};
 //long               FWObjectDatabase::IDcounter = 0;
@@ -92,10 +112,11 @@ const char*        FWObjectDatabase::TYPENAME  = {"FWObjectDatabase"};
 const string FWObjectDatabase::DTD_FILE_NAME  = "fwbuilder.dtd"    ;
 
 
-FWObjectDatabase::FWObjectDatabase():FWObject(), data_file()
+FWObjectDatabase::FWObjectDatabase() : FWObject(false), data_file()
 {
     setRoot(this);
     index_hits = index_misses = 0;
+    init_id_dict();
 
 //    if(db==NULL) db=this;
 
@@ -104,14 +125,15 @@ FWObjectDatabase::FWObjectDatabase():FWObject(), data_file()
 
     setName(TYPENAME);
 
-    setId( getRootId() );
+    setId( ROOT_ID );
     setDirty(false);
 }
 
-FWObjectDatabase::FWObjectDatabase(FWObjectDatabase& d):FWObject()
+FWObjectDatabase::FWObjectDatabase(FWObjectDatabase& d) : FWObject(false)
 {
     setRoot(this);
     index_hits = index_misses = 0;
+    init_id_dict();
 
     data_file = d.data_file;
 
@@ -119,9 +141,9 @@ FWObjectDatabase::FWObjectDatabase(FWObjectDatabase& d):FWObject()
 
     searchId =0;
 
-    init=true;
+    init = true;
     *this = d;  // copies entire tree
-    setId( getRootId() );
+    setId( ROOT_ID );
 
 // I do not understand why do I need to reindex the whole database
 // after operator=. It calls FWobject::duplicate, which in turn
@@ -131,7 +153,60 @@ FWObjectDatabase::FWObjectDatabase(FWObjectDatabase& d):FWObject()
     addToIndexRecursive( this );  // reindex
 
     setDirty(false);
-    init=false;
+    init = false;
+}
+
+void FWObjectDatabase::init_id_dict()
+{
+    if (id_dict.size()==0)
+    {
+        id_dict[ROOT_ID] = "root";
+        id_dict[ANY_ADDRESS_ID] = "sysid0";
+        id_dict[ANY_SERVICE_ID] = "sysid1";
+        id_dict[ANY_INTERVAL_ID] = "sysid2";
+        id_dict[STANDARD_LIB_ID] = "syslib000";
+        id_dict[TEMPLATE_LIB_ID] = "syslib100";
+        id_dict[DELETED_OBJECTS_ID] = "sysid99";
+
+        for (map<int,string>::iterator i=id_dict.begin(); i!=id_dict.end(); ++i)
+            id_dict_reverse[i->second] = i->first;
+    }
+}
+
+int FWObjectDatabase::registerStringId(const std::string &s_id)
+{
+    int i_id = -1;
+
+    if (id_dict_reverse.count(s_id) > 0)
+        return id_dict_reverse[s_id];
+
+    i_id = ++id_seed;
+    id_dict[i_id] = s_id;
+    id_dict_reverse[s_id] = i_id;
+    return i_id;
+}
+
+int FWObjectDatabase::getIntId(const std::string &s_id)
+{
+    if (id_dict_reverse.count(s_id) > 0) return id_dict_reverse[s_id];
+    return -1;
+}
+
+string FWObjectDatabase::getStringId(int i_id)
+{
+    if (id_dict.count(i_id) > 0) return id_dict[i_id];
+
+    // TODO: Use proper GUID algorithm here
+    char id_buf[64];
+    sprintf(id_buf, "id%dX%d", i_id, cached_pid);
+    id_dict[i_id] = string(id_buf);
+    id_dict_reverse[string(id_buf)] = i_id;
+    return id_dict[i_id];
+}
+
+int FWObjectDatabase::generateUniqueId()
+{
+    return ++id_seed;
 }
 
 void FWObjectDatabase::setFileName(const string &filename)
@@ -165,20 +240,22 @@ void FWObjectDatabase::load(const string &f,
     if(f=="") 
         return;
     
-    xmlDocPtr doc=XMLTools::loadFile(f, FWObjectDatabase::TYPENAME,
-                                     FWObjectDatabase::DTD_FILE_NAME,
-                                     upgrade, template_dir);
+    xmlDocPtr doc = XMLTools::loadFile(f, FWObjectDatabase::TYPENAME,
+                                       FWObjectDatabase::DTD_FILE_NAME,
+                                       upgrade, template_dir);
     
     xmlNodePtr root = xmlDocGetRootElement(doc);
     
-    if(!root || !root->name || strcmp(FROMXMLCAST(root->name), FWObjectDatabase::TYPENAME)!=SAME)
+    if(!root || !root->name || strcmp(FROMXMLCAST(root->name),
+                                      FWObjectDatabase::TYPENAME)!=SAME)
     {
 	xmlFreeDoc(doc);
-        throw FWException("Data file have invalid structure: "+f);
+        throw FWException("Data file has invalid structure: "+f);
     }
     
-    try {
-        init=true;
+    try
+    {
+        init = true;
 
 //        clearChildren();
         deleteChildren();
@@ -189,10 +266,10 @@ void FWObjectDatabase::load(const string &f,
         setFileName(f);
     } catch (FWException &ex)
     {
-        init=false;
+        init = false;
         throw(ex);
     }
-    init=false;
+    init = false;
 }
 
 void FWObjectDatabase::saveXML(xmlDocPtr doc) throw(FWException)
@@ -259,9 +336,11 @@ void FWObjectDatabase::fromXML(xmlNodePtr root) throw(FWException)
     
     const char *n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("lastModified")));
     if(n!=NULL)
-    {
+    {        
         int i=0;
-        sscanf(n,"%d",&i);
+        istringstream str(n);
+        str >> i;
+        //sscanf(n,"%d",&i);
         lastModified=i;
         FREEXMLBUFF(n);
     }
@@ -279,20 +358,22 @@ xmlNodePtr FWObjectDatabase::toXML(xmlNodePtr parent) throw(FWException)
 
     if (lastModified!=0)
     {
-        char lmbuf[32];
-        sprintf(lmbuf,"%ld",lastModified);
+        ostringstream str;
+        str << lastModified;
+        //char lmbuf[32];
+        //sprintf(lmbuf,"%ld",lastModified);
         xmlNewProp(parent, 
                    TOXMLCAST("lastModified"),
-                   TOXMLCAST(lmbuf));
+                   TOXMLCAST(str.str().c_str()));
     }
 
-    string rootid = getId();
+    int rootid = getId();
 
     xmlAttrPtr pr = xmlNewProp(parent, 
 			       TOXMLCAST("id") , 
-			       STRTOXMLCAST(rootid));
+			       STRTOXMLCAST(id_dict[rootid]));
 
-    xmlAddID(NULL, parent->doc, STRTOXMLCAST(rootid), pr);
+    xmlAddID(NULL, parent->doc, STRTOXMLCAST(id_dict[rootid]), pr);
 
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
@@ -302,29 +383,6 @@ xmlNodePtr FWObjectDatabase::toXML(xmlNodePtr parent) throw(FWException)
     return parent;
 }
 
-string FWObjectDatabase::generateUniqueId()
-{
-    char res[20];
-
-    //TODO: mutex could not be used since threads
-    // library is not initialized.
-
-//    IDmutex.lock();
-    int pid=0;
-#ifdef _WIN32
-    pid = _getpid();
-#else
-    pid = getpid();
-#endif
-    
-
-    sprintf(res,"id%lX%d",IDc.get(),pid);
-//    IDc++;
-//    IDmutex.unlock();
-    
-    return res;
-}
-
 void  FWObjectDatabase::setDirty(bool f)
 {
     dirty=f;
@@ -336,22 +394,22 @@ void FWObjectDatabase::addToIndex(FWObject* o)
     if (o)
     {
         o->setRoot( this );
-        if (!o->getId().empty() ) obj_index[o->getId()] = o;
+        if (o->getId() > -1 ) obj_index[o->getId()] = o;
     }
 }
 
-void FWObjectDatabase::removeFromIndex(const string &id)
+void FWObjectDatabase::removeFromIndex(int id)
 {
     obj_index.erase(id);
 }
 
-FWObject* FWObjectDatabase::checkIndex(const std::string &id)
+FWObject* FWObjectDatabase::checkIndex(int id)
 {
-    FWObject *o = obj_index[id];
-    return o;
+    if (obj_index.count(id)==0) return NULL;
+    return obj_index[id];
 }
 
-FWObject* FWObjectDatabase::findInIndex(const std::string &id)
+FWObject* FWObjectDatabase::findInIndex(int id)
 {
     FWObject *o = checkIndex(id);
     if (o!=NULL) index_hits++;
@@ -409,7 +467,9 @@ void FWObjectDatabase::clearIndex()
     obj_index.clear();
 }
 
-void FWObjectDatabase::getIndexStats(int &index_size, int &hit_counter, int &miss_counter)
+void FWObjectDatabase::getIndexStats(int &index_size,
+                                     int &hit_counter,
+                                     int &miss_counter)
 {
     index_size = obj_index.size();
     hit_counter = index_hits;
@@ -429,7 +489,8 @@ void FWObjectDatabase::addToIndexRecursive(FWObject *o)
 FWObject *FWObjectDatabase::createFromXML(xmlNodePtr data)
 {
     const char *n;
-    string typen, id;
+    string typen;
+    int id = -1;
 
     n=FROMXMLCAST(data->name);
     if(!n)
@@ -439,13 +500,13 @@ FWObject *FWObjectDatabase::createFromXML(xmlNodePtr data)
     n=FROMXMLCAST(xmlGetProp(data,TOXMLCAST("id")));
     if(n)
     {
-        id = n;
-        FREEXMLBUFF(id);
+        id = registerStringId(n);
+        FREEXMLBUFF(n);
     }
 
 // create new object but do not prepopulate objects that
 // automatically create children in constructor
-    return create(typen,id,false);   
+    return create(typen, id, false);
 }
 
 
@@ -453,17 +514,17 @@ FWObject *FWObjectDatabase::createFromXML(xmlNodePtr data)
 if(strcmp(classname::TYPENAME,str)==SAME) \
 { \
   nobj=new classname(this,prepopulate); \
-  if (!id.empty()) nobj->setId(id); \
+  if (id > -1) nobj->setId(id); \
   addToIndex(nobj); \
   return nobj; \
 }
 
 
 FWObject *FWObjectDatabase::create(const string &type_name,
-                                   const std::string &id,
+                                   int id,
                                    bool prepopulate)
 {
-    const char *n=type_name.c_str();
+    const char *n = type_name.c_str();
     FWObject   *nobj;
 
     if (strcmp("comment",n)==SAME) return NULL;
@@ -471,7 +532,7 @@ FWObject *FWObjectDatabase::create(const string &type_name,
     if(strcmp("AnyNetwork"  ,n)==SAME)
     {
         nobj = new Network(this,prepopulate);
-        if (!id.empty()) nobj->setId(id);
+        if (id > -1) nobj->setId(id);
         nobj->setXMLName("AnyNetwork");
         addToIndex(nobj);
         return nobj;
@@ -480,7 +541,7 @@ FWObject *FWObjectDatabase::create(const string &type_name,
     if(strcmp("AnyIPService",n)==SAME)
     {
         nobj = new IPService(this,prepopulate);
-        if (!id.empty()) nobj->setId(id);
+        if (id > -1) nobj->setId(id);
         nobj->setXMLName("AnyIPService");
         addToIndex(nobj);
         return nobj;
@@ -489,7 +550,7 @@ FWObject *FWObjectDatabase::create(const string &type_name,
     if(strcmp("AnyInterval" ,n)==SAME)
     {
         nobj = new Interval(this,prepopulate);
-        if (!id.empty()) nobj->setId(id);
+        if (id > -1) nobj->setId(id);
         nobj->setXMLName("AnyInterval");
         addToIndex(nobj);
         return nobj;
@@ -531,8 +592,10 @@ FWObject *FWObjectDatabase::create(const string &type_name,
     CREATE_OBJ(FWBDManagement      ,n, id, prepopulate);
     CREATE_OBJ(Firewall            ,n, id, prepopulate);
     CREATE_OBJ(Network             ,n, id, prepopulate);
+    CREATE_OBJ(NetworkIPv6         ,n, id, prepopulate);
     CREATE_OBJ(Interface           ,n, id, prepopulate);
     CREATE_OBJ(IPv4                ,n, id, prepopulate);
+    CREATE_OBJ(IPv6                ,n, id, prepopulate);
     CREATE_OBJ(physAddress         ,n, id, prepopulate);
     CREATE_OBJ(DNSName             ,n, id, prepopulate);
     CREATE_OBJ(AddressTable        ,n, id, prepopulate);
@@ -543,8 +606,10 @@ FWObject *FWObjectDatabase::create(const string &type_name,
     CREATE_OBJ(UDPService          ,n, id, prepopulate);
     CREATE_OBJ(TagService          ,n, id, prepopulate);
     CREATE_OBJ(ICMPService         ,n, id, prepopulate);
+    CREATE_OBJ(ICMP6Service        ,n, id, prepopulate);
     CREATE_OBJ(IPService           ,n, id, prepopulate);
     CREATE_OBJ(CustomService       ,n, id, prepopulate);
+    CREATE_OBJ(UserService         ,n, id, prepopulate);
     CREATE_OBJ(Interval            ,n, id, prepopulate);
     CREATE_OBJ(ObjectGroup         ,n, id, prepopulate);
     CREATE_OBJ(ServiceGroup        ,n, id, prepopulate);
@@ -564,7 +629,7 @@ Firewall* FWObjectDatabase::_findFirewallByNameRecursive(FWObject* db,
 {
     if (Firewall::isA(db) &&
         db->getName()==name &&
-        db->getParent()->getId()!=getDeletedObjectsId())
+        db->getParent()->getId()!=FWObjectDatabase::DELETED_OBJECTS_ID)
         return Firewall::cast(db);
 
     list<FWObject*>::iterator j;
@@ -619,13 +684,13 @@ void FWObjectDatabase::recursivelyRemoveObjFromTree(FWObject* obj,
 
 class FWObjectTreeScanner {
 
-    FWObjectDatabase            *treeRoot;
-    map<string,FWObject*>        srcMap;
-    map<string,FWObject*>        dstMap;
+    FWObjectDatabase         *treeRoot;
+    map<int,FWObject*>        srcMap;
+    map<int,FWObject*>        dstMap;
     FWObjectDatabase::ConflictResolutionPredicate *crp;
     bool                         defaultCrp;
-
-    void walkTree(map<string,FWObject*> &m,FWObject *root);
+    int                          reference_object_id_offset;
+    void walkTree(map<int,FWObject*> &m,FWObject *root);
     void addRecursively(FWObject *src);
 
     public:
@@ -633,6 +698,7 @@ class FWObjectTreeScanner {
     FWObjectTreeScanner(FWObject *r,
                         FWObjectDatabase::ConflictResolutionPredicate *_crp=NULL)
     {
+        reference_object_id_offset = 1000000;
         treeRoot=FWObjectDatabase::cast(r);
         defaultCrp=false;
         if (_crp==NULL)
@@ -655,14 +721,20 @@ class FWObjectTreeScanner {
  *
  * Here, in effect, I am artifically adding IDs to references.
  */
-void FWObjectTreeScanner::walkTree(map<string,FWObject*> &m,FWObject *root)
+void FWObjectTreeScanner::walkTree(map<int,FWObject*> &m,
+                                   FWObject *root)
 {
-    if (root->exists("id"))  m[root->getId()]=root;
+    if (root->haveId())  m[root->getId()]=root;
 
     if (FWReference::cast(root)!=NULL)
     {
         FWReference *r=FWReference::cast(root);
-        m[string("ref_")+r->getPointerId()]=root;
+        // need to add reference to the map, but references do not have
+        // their own Id. Create new id using id of the object reference
+        // points to, plus some offset.
+        // I can not just generate new uniq id because I need to be able
+        // to find this object later, and for that its id must be predictable.
+        m[reference_object_id_offset+r->getPointerId()]=root;
     }
 
     for (FWObject::iterator i=root->begin(); i!=root->end(); i++)
@@ -679,7 +751,7 @@ void FWObjectTreeScanner::addRecursively(FWObject *src)
 
     if (dstMap[src->getId()]==NULL)
     {
-        FWObject *o1=treeRoot->create(src->getTypeName(),"",false);
+        FWObject *o1 = treeRoot->create(src->getTypeName(), -1, false);
         FWObject *pdst = dstMap[src->getParent()->getId()];
         assert(pdst!=NULL);
 
@@ -739,22 +811,34 @@ void FWObjectTreeScanner::scanAndAdd(FWObject *dst,FWObject *source)
         FWObject *o1=*i;
         if (FWReference::cast(o1)!=NULL)
         {
-            string   pid   = FWReference::cast(o1)->getPointerId();
+            int pid   = FWReference::cast(o1)->getPointerId();
             FWObject *o2   = dstMap[pid];
 
             if (o2==NULL)
             {
                 FWObject *osrc = srcMap[ pid ];
-                addRecursively( osrc);
+                if (osrc==NULL)
+                    cerr << "Object with ID=" << pid
+                         << " (" << FWObjectDatabase::getStringId(pid) << ") "
+                         << " disappeared" << endl;
+                else
+                    addRecursively( osrc);
             }
         } else
             scanAndAdd( o1 , source );
     }
+    // TODO: do the same for the objects referenced by
+    // rule actions Branch and Tag - find those objects and add.
+    // Wrap operations with network_zone in methods of class Interface,
+    // setNetworkZone(FWObject*) getNetworkZone()
+    // (Just like Rule::getBranch Rule::setBranch)
+    //
     if (Interface::isA(dst))
     {
-        string   pid = dst->getStr("network_zone");
-        if ( !pid.empty() )
+        string sid = dst->getStr("network_zone");
+        if ( !sid.empty() )
         {
+            int pid = FWObjectDatabase::getIntId(sid);
             FWObject *o2 = dstMap[pid];
             if (o2==NULL)
             {
@@ -769,7 +853,7 @@ void FWObjectTreeScanner::scanAndAdd(FWObject *dst,FWObject *source)
 
 void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
 {
-    string dobjId             = FWObjectDatabase::getDeletedObjectsId();
+    int dobjId = FWObjectDatabase::DELETED_OBJECTS_ID;
 
     if (dst==NULL)
     {
@@ -783,27 +867,6 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
          */
         FWObjectDatabase *dstroot = dst->getRoot();
 
-        /* commented out 11/12/04
-         * If src has some deleted objects, we should just ignore them
-         */
-
-#if 0
-        FWObjectDatabase *srcroot = src->getRoot();
-        FWObject *srcdobj = srcroot->getById( dobjId );
-        if (srcdobj)
-        {
-            for (FWObject::iterator i=srcdobj->begin(); i!=srcdobj->end(); i++)
-            {
-                FWObject *dobj = dstMap[ (*i)->getId() ];
-                if(dobj!=NULL && dobj->getParent()->getId()!=dobjId)
-                {
-                    crp->askUser( dobj, *i );
-                    dstroot->recursivelyRemoveObjFromTree(dobj);
-                }
-            }
-        }
-#endif
-
         /*
          * find deleted objects library in dst and delete objects from
          * it if they are present and not deleted in src
@@ -845,7 +908,7 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
         if (FWReference::cast( *i ))
         {
             FWReference *r=FWReference::cast(*i);
-            dobj= dstMap[ string("ref_")+r->getPointerId() ];
+            dobj= dstMap[reference_object_id_offset + r->getPointerId()];
         } else dobj= dstMap[ (*i)->getId() ];
 
         if (dobj==NULL)
@@ -904,25 +967,36 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
 
             if (Group::cast(dobj)!=NULL)
             {
-                FWObject *firstChild=NULL;
-                if (dobj->size()>0)         firstChild= dobj->front();
+                // at one point I've got bunch of data files where
+                // DeletedObjects library contained references for
+                // some reason. This should not happen, but at the
+                // same time this is valid file structure so the code
+                // should be able to handle it.
+                if (dobj->getId()==FWObjectDatabase::DELETED_OBJECTS_ID)
+                    merge( dobj , *i );
                 else
                 {
-                    if ( (*i)->size()>0 )   firstChild= (*i)->front();
-                }
-                if (firstChild==NULL || FWReference::cast(firstChild)!=NULL)
-                {
-                    if (crp!=NULL && crp->askUser( dobj, *i ))
+
+                    FWObject *firstChild=NULL;
+                    if (dobj->size()>0)         firstChild= dobj->front();
+                    else
                     {
-#ifdef DEBUG_MERGE
-                        cerr << "--------------------------------" << endl;
-                        cerr << "merge: duplicate #2 " << endl;
-                        dobj->dump(true,true);
-                        cerr << endl;
-#endif
-                        dobj->duplicate( (*i), false );
+                        if ( (*i)->size()>0 )   firstChild= (*i)->front();
                     }
-                } else merge( dobj , *i );
+                    if (firstChild==NULL || FWReference::cast(firstChild)!=NULL)
+                    {
+                        if (crp!=NULL && crp->askUser( dobj, *i ))
+                        {
+#ifdef DEBUG_MERGE
+                            cerr << "--------------------------------" << endl;
+                            cerr << "merge: duplicate #2 " << endl;
+                            dobj->dump(true,true);
+                            cerr << endl;
+#endif
+                            dobj->duplicate( (*i), false );
+                        }
+                    } else merge( dobj , *i );
+                }
             }
             else
             {
@@ -944,9 +1018,9 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
 
 FWObjectDatabase* FWObjectDatabase::exportSubtree( const list<FWObject*> &libs )
 {
-    FWObjectDatabase *ndb  = new FWObjectDatabase();
+    FWObjectDatabase *ndb = new FWObjectDatabase();
 
-    ndb->init=true;
+    ndb->init = true;
 
     for (list<FWObject*>::const_iterator i=libs.begin(); i!=libs.end(); i++)
     {
@@ -957,27 +1031,27 @@ FWObjectDatabase* FWObjectDatabase::exportSubtree( const list<FWObject*> &libs )
     }
 
     FWObjectTreeScanner scanner(ndb);
-    scanner.scanAndAdd(NULL,this);
+    scanner.scanAndAdd(NULL, this);
 
-    ndb->init=false;
+    ndb->init = false;
 
     return ndb;
 }
 
 FWObjectDatabase* FWObjectDatabase::exportSubtree( FWObject *lib )
 {
-    FWObjectDatabase *ndb  = new FWObjectDatabase();
+    FWObjectDatabase *ndb = new FWObjectDatabase();
 
-    ndb->init=true;
+    ndb->init = true;
 
     FWObject *nlib = ndb->create(lib->getTypeName());
     ndb->add(nlib);
     *nlib = *lib;
 
     FWObjectTreeScanner scanner(ndb);
-    scanner.scanAndAdd(NULL,this);
+    scanner.scanAndAdd(NULL, this);
 
-    ndb->init=false;
+    ndb->init = false;
 
     return ndb;
 }
@@ -985,12 +1059,12 @@ FWObjectDatabase* FWObjectDatabase::exportSubtree( FWObject *lib )
 void FWObjectDatabase::merge( FWObjectDatabase *ndb,
                               ConflictResolutionPredicate *crp)
 {
-    init=true;
+    init = true;
 
-    FWObjectTreeScanner scanner(this,crp);
-    scanner.merge(NULL,ndb);
+    FWObjectTreeScanner scanner(this, crp);
+    scanner.merge(NULL, ndb);
 
-    init=false;
+    init = false;
 }
 
 /*
@@ -1001,7 +1075,7 @@ void FWObjectDatabase::merge( FWObjectDatabase *ndb,
 void FWObjectDatabase::findObjectsInGroup(Group *g,set<FWObject *> &res)
 {
     searchId++;
-    _findObjectsInGroup(g,res);
+    _findObjectsInGroup(g, res);
 }
 
 void FWObjectDatabase::_findObjectsInGroup(Group *g,set<FWObject *> &res)
@@ -1048,18 +1122,55 @@ bool FWObjectDatabase::_isInIgnoreList(FWObject *o)
  *  references using temporary flags set in objects this method
  *  visits.
  */
-void FWObjectDatabase::findWhereUsed(FWObject *o,FWObject *p,set<FWObject *> &resset)
+void FWObjectDatabase::findWhereUsed(FWObject *o,
+                                     FWObject *p,
+                                     set<FWObject *> &resset)
 {
+    set<FWObject*> results;
     searchId++;
-    _findWhereUsed(o,p,resset);
+    _findWhereUsed(o, p, results);
+    for (set<FWObject*>::iterator it=results.begin(); it!=results.end(); ++it)
+    {
+        FWObject *obj = *it;
+        if (RuleSet::cast(obj))
+        {
+            resset.insert(obj->getParent());
+            continue;
+        }
+        resset.insert(obj);
+    }
 }
 
-bool FWObjectDatabase::_findWhereUsed(FWObject *o,FWObject *p,set<FWObject *> &resset)
+//#define DEBUG_WHERE_USED 1
+
+bool FWObjectDatabase::_findWhereUsed(FWObject *o,
+                                      FWObject *p,
+                                      set<FWObject *> &resset)
 {
     bool res=false;
-        
-    if ( _isInIgnoreList(p) || p->size()==0) return res;
-    
+    if ( _isInIgnoreList(p)) return res;
+
+#if DEBUG_WHERE_USED
+    cerr << "_findWhereUsed"
+         << "  o=" << o
+         << "  " << o->getName()
+         << "  p=" << p
+         << "  " << p->getName()
+         << " (" << p->getTypeName() << ")"
+         << "  resset.size()=" << resset.size()
+         << "  p->getInt(\".searchId\")=" << p->getInt(".searchId")
+         << "  searchId=" << searchId
+         << endl;
+#endif        
+
+    if (o==p)
+    {
+        resset.insert(p);
+        res = true;
+    }
+
+    if (p->size()==0) return res;
+
     if (p->getInt(".searchId")==searchId)
     {
         res=p->getBool(".searchResult");
@@ -1067,31 +1178,69 @@ bool FWObjectDatabase::_findWhereUsed(FWObject *o,FWObject *p,set<FWObject *> &r
     }
 
 // set flags to break indefinite recursion in case we encounter circular groups
-    p->setInt(".searchId",searchId);
-    p->setBool(".searchResult",res);   // res==false at this time
-   
-    FWObject::iterator i1=p->begin();
-    for (;i1!=p->end(); ++i1)
-    {
-        if ((*i1)->getId()==FWObjectDatabase::getDeletedObjectsId()) continue;
+    p->setInt(".searchId", searchId);
+    p->setBool(".searchResult", res);   // res==false at this time
 
-        FWReference  *ref=FWReference::cast(*i1);
+    if (PolicyRule::cast(p))
+    {
+        PolicyRule *rule = PolicyRule::cast(p);
+        switch (rule->getAction()) {
+        case PolicyRule::Tag:
+        {
+            FWObject *tagobj = rule->getTagObject();
+            if (o==tagobj)
+            {
+                resset.insert(p);
+                res = true;
+                break;
+            }
+            break;
+        }
+        case PolicyRule::Branch:
+        {
+            FWObject *ruleset = rule->getBranch();
+            if (o==ruleset)
+            {
+                resset.insert(p);
+                res = true;
+                break;
+            }
+            break;
+        }
+        default: ;
+        }
+    }
+   
+    FWObject::iterator i1 = p->begin();
+    for ( ; i1!=p->end(); ++i1)
+    {
+        if ((*i1)->getId()==FWObjectDatabase::DELETED_OBJECTS_ID) continue;
+
+        FWReference  *ref = FWReference::cast(*i1);
         if (ref!=NULL)
         {  // child is a reference
-            FWObject *g=ref->getPointer();
-            if (o==g) res= true;
-            else      res |= _findWhereUsed(o,g,resset);
+            FWObject *g = ref->getPointer();
+            if (_findWhereUsed(o, g, resset))
+            {
+                resset.insert(p);
+                res = true;
+            }
         }
         else    // child is a regular object, not a reference
         {
-            if (o==(*i1))  res=true;
-            else           res |= _findWhereUsed(o,*i1,resset);
+            if (_findWhereUsed(o, *i1, resset))
+            {
+                // do not add rule to the results if rule element
+                // matches because rule element should already be there.
+                if (Rule::cast(p) && RuleElement::cast(*i1)==NULL)
+                    resset.insert(p);
+                res = true;
+            }
         }
     }
 
     p->setBool(".searchResult",res);
-    if (res) resset.insert(p);
+    //if (res) resset.insert(p);
     return res;
 }
 
-
