@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWOptions.cpp,v 1.6 2006/09/10 22:40:37 vkurland Exp $
+  $Id: FWOptions.cpp 177 2008-08-23 23:48:45Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -60,7 +60,6 @@ void FWOptions::fromXML(xmlNodePtr root) throw(FWException)
     {
         if(cur && !xmlIsBlankNode(cur))    
         {
-
             n=FROMXMLCAST(xmlGetProp(cur,TOXMLCAST("name")));
             assert(n!=NULL);
             cont=FROMXMLCAST( xmlNodeGetContent(cur) );
@@ -78,10 +77,10 @@ xmlNodePtr FWOptions::toXML(xmlNodePtr root) throw(FWException)
 {
     xmlNodePtr opt;
 
-    xmlNodePtr me = xmlNewChild(root, NULL, 
-		xml_name.empty()?STRTOXMLCAST(getTypeName()):STRTOXMLCAST(xml_name), 
-				NULL);
-
+    xmlNodePtr me = xmlNewChild(
+        root, NULL, 
+        xml_name.empty() ? STRTOXMLCAST(getTypeName()) : STRTOXMLCAST(xml_name),
+        NULL);
 
     map<string, string>::iterator i;
     for(i=data.begin(); i!=data.end(); ++i)  
