@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Firewall.cpp,v 1.11 2007/07/07 05:36:22 vkurland Exp $
+  $Id: Firewall.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -36,6 +36,8 @@
 #include <fwbuilder/FWOptions.h>
 #include <fwbuilder/Interface.h>
 #include <fwbuilder/Management.h>
+#include <fwbuilder/IPv4.h>
+#include <fwbuilder/IPv6.h>
 
 #include <fwbuilder/Policy.h>
 #include <fwbuilder/InterfacePolicy.h>
@@ -62,7 +64,7 @@ Firewall::Firewall()
     setInt("lastModified" ,0);
     setInt("lastInstalled" ,0);
     setInt("lastCompiled" ,0);
-    
+   
 }
 
 Firewall::Firewall(const FWObject *root,bool prepopulate) : Host(root,prepopulate)
@@ -76,9 +78,16 @@ Firewall::Firewall(const FWObject *root,bool prepopulate) : Host(root,prepopulat
     if (prepopulate)
     {
         add( getRoot()->create(FirewallOptions::TYPENAME) );
-        add( getRoot()->create(Policy::TYPENAME) );
-        add( getRoot()->create(NAT::TYPENAME) );
-        add( getRoot()->create(Routing::TYPENAME) );
+        RuleSet *p;
+        p = RuleSet::cast(getRoot()->create(Policy::TYPENAME));
+        p->setTop(true);
+        add(p);
+        p = RuleSet::cast(getRoot()->create(NAT::TYPENAME));
+        p->setTop(true);
+        add(p);
+        p = RuleSet::cast(getRoot()->create(Routing::TYPENAME));
+        p->setTop(true);
+        add(p);
     }
 }
 
@@ -139,19 +148,36 @@ void Firewall::fromXML(xmlNodePtr root) throw(FWException)
 xmlNodePtr Firewall::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+
     FWObject *o;
-    o=getFirstByType( NAT::TYPENAME );
-    if (o) o->toXML(me);
+    for (FWObjectTypedChildIterator it = findByType(NAT::TYPENAME);
+         it != it.end(); ++it)
+    {
+        o = *it;
+        if (o) o->toXML(me);
+    }
+    for (FWObjectTypedChildIterator it = findByType(Policy::TYPENAME);
+         it != it.end(); ++it)
+    {
+        o = *it;
+        if (o) o->toXML(me);
+    }
+    for (FWObjectTypedChildIterator it = findByType(Routing::TYPENAME);
+         it != it.end(); ++it)
+    {
+        o = *it;
+        if (o) o->toXML(me);
+    }
 
-    o=getFirstByType( Policy::TYPENAME );
-    if (o) o->toXML(me);
-
-    o=getFirstByType( Routing::TYPENAME );
-    if (o) o->toXML(me);
-
-    for(FWObjectTypedChildIterator j=findByType(Interface::TYPENAME); j!=j.end(); ++j)
- 	if((o=(*j))!=NULL )
- 	    o->toXML(me);
+    for(FWObjectTypedChildIterator it = findByType(Interface::TYPENAME);
+        it != it.end(); ++it)
+    {
+ 	o = *it;
+        if (o) o->toXML(me);
+    }
 
     o=getFirstByType( Management::TYPENAME );
     if(o) o->toXML(me);
@@ -170,24 +196,30 @@ FWOptions* Firewall::getOptionsObject()
 
 Policy* Firewall::getPolicy() 
 { 
-    return(Policy::cast(getFirstByType(Policy::TYPENAME)));
+    return(Policy::cast(findObjectByName(Policy::TYPENAME, "Policy")));
 }
 
 NAT* Firewall::getNAT()    
 { 
-    return(NAT::cast(getFirstByType(NAT::TYPENAME)));
+    return(NAT::cast(findObjectByName(NAT::TYPENAME, "NAT")));
 }
 
 
 Routing* Firewall::getRouting()    
 {
-    return(Routing::cast(getFirstByType(Routing::TYPENAME)));
+    return(Routing::cast(findObjectByName(Routing::TYPENAME, "Routing")));
 }
 
 
 bool  Firewall::validateChild(FWObject *o)
 { 
     string otype=o->getTypeName();
+    if (Routing::isA(o))
+    {
+        // there can be only one
+        list<FWObject*> routing_objects = getByType(Routing::TYPENAME);
+        if (routing_objects.size() > 0) return false;
+    }
     return (FWObject::validateChild(o) && 
 	    (otype==Interface::TYPENAME  ||
 	     otype==RuleSet::TYPENAME    ||
@@ -198,92 +230,77 @@ bool  Firewall::validateChild(FWObject *o)
 	     otype==FirewallOptions::TYPENAME ));
 }
 
-void Firewall::replaceRef(FWObject *rs,
-                          const string &old_id,const std::string &new_id)
-{
-    FWReference *ref=FWObjectReference::cast(rs);
-    if (ref==NULL)
-    {
-        for (FWObject::iterator j1=rs->begin(); j1!=rs->end(); ++j1)
-            replaceRef( *j1, old_id, new_id);
-    } else
-    { 
-        if (ref->getPointerId()==old_id)
-            ref->setPointerId(new_id);
-    }
-}
-
-FWObject& Firewall::duplicate(const FWObject *obj, bool preserve_id) throw(FWException)
+FWObject& Firewall::duplicate(const FWObject *obj,
+                              bool preserve_id) throw(FWException)
 {
     string err="Error creating object with type: ";
 
     checkReadOnly();
     bool xro=obj->getBool("ro"); 
 
-    shallowDuplicate(obj,preserve_id);
+    //shallowDuplicate(obj, preserve_id);
+    FWObject::shallowDuplicate(obj, preserve_id);
 
     setReadOnly(false);
 
     destroyChildren();
 
-    Policy   *op = Policy::cast(obj->getFirstByType(Policy::TYPENAME));
-    FWObject *pol = addCopyOf(op,preserve_id);
-    replaceRef(pol,obj->getId(), getId() );
+    for (FWObjectTypedChildIterator it = obj->findByType(Policy::TYPENAME);
+         it != it.end(); ++it)
+    {
+        addCopyOf(*it, preserve_id);
+    }
+    for (FWObjectTypedChildIterator it = obj->findByType(NAT::TYPENAME);
+         it != it.end(); ++it)
+    {
+        addCopyOf(*it, preserve_id);
+    }
+    for (FWObjectTypedChildIterator it = obj->findByType(Routing::TYPENAME);
+         it != it.end(); ++it)
+    {
+        addCopyOf(*it, preserve_id);
+    }
 
-    NAT      *on = NAT::cast(obj->getFirstByType(NAT::TYPENAME));
-    FWObject *nat = addCopyOf(on,preserve_id);
-    replaceRef(nat,obj->getId(), getId() );
+    replaceRef(obj->getId(), getId() );
 
-    Routing  *oroute = Routing::cast(obj->getFirstByType(Routing::TYPENAME));
-    FWObject *r = addCopyOf(oroute,preserve_id);
-    replaceRef(r,obj->getId(), getId() );  
-    
-    FWObjectTypedChildIterator m=obj->findByType(Interface::TYPENAME);
-    for ( ; m!=m.end(); ++m ) 
+    for (FWObjectTypedChildIterator m = obj->findByType(Interface::TYPENAME);
+         m!=m.end(); ++m ) 
     {
         FWObject *o   = *m;
         FWObject *o1  = addCopyOf(o,preserve_id);
-        replaceRef(pol, o->getId(),   o1->getId()   );
-        replaceRef(nat, o->getId(),   o1->getId()   );
+
+        replaceRef(o->getId(),   o1->getId()   );
+
         o1->destroyChildren();
 
-#ifdef USING_INTERFACE_POLICY
-	FWObject  *ipolicy  = o->getFirstByType(InterfacePolicy::TYPENAME);
-        FWObject  *ipolicy1 = NULL;
-        if (ipolicy)
-        {
-            ipolicy1=o1->addCopyOf(ipolicy,preserve_id);
-            replaceRef(ipolicy1, obj->getId(), getId()       );
-            replaceRef(ipolicy1, o->getId(),   o1->getId()   );
-        }
-#endif
-
-        FWObjectTypedChildIterator k=o->findByType(IPv4::TYPENAME);
-        for ( ; k!=k.end(); ++k ) 
+        for (FWObjectTypedChildIterator k=o->findByType(IPv4::TYPENAME);
+             k!=k.end(); ++k ) 
         {
             FWObject *oa = *k;
             FWObject *oa1= o1->addCopyOf(oa,preserve_id);
 
             if (oa!=NULL && oa1!=NULL)
-            {       
-                replaceRef(pol, oa->getId(),  oa1->getId() );
-                replaceRef(nat, oa->getId(),  oa1->getId() );
-//                if (ipolicy1) replaceRef(ipolicy1, oa->getId(),  oa1->getId() );
-            }
+                replaceRef(oa->getId(),  oa1->getId() );
         }
 
-        k=o->findByType(physAddress::TYPENAME);
-        for ( ; k!=k.end(); ++k ) 
+        for (FWObjectTypedChildIterator k=o->findByType(IPv6::TYPENAME);
+             k!=k.end(); ++k ) 
+        {
+            FWObject *oa = *k;
+            FWObject *oa1= o1->addCopyOf(oa,preserve_id);
+
+            if (oa!=NULL && oa1!=NULL)
+                replaceRef(oa->getId(),  oa1->getId() );
+        }
+
+        for (FWObjectTypedChildIterator k = o->findByType(physAddress::TYPENAME);
+             k!=k.end(); ++k ) 
         {
             FWObject *opa = *k;
             FWObject *opa1= o1->addCopyOf(opa,preserve_id);
 
             if (opa!=NULL && opa1!=NULL)
-            {       
-                replaceRef(pol, opa->getId(),  opa1->getId() );
-                replaceRef(nat, opa->getId(),  opa1->getId() );
-//                if (ipolicy1) replaceRef(ipolicy1, opa->getId(),  opa1->getId() );
-            }
+                replaceRef(opa->getId(),  opa1->getId() );
         }
     }
 
@@ -303,29 +320,35 @@ void   Firewall::updateLastInstalledTimestamp()
 {
     setInt("lastInstalled",time(NULL));
 }
+
 void Firewall::updateLastModifiedTimestamp()
 {
     setInt("lastModified",time(NULL));
 }
+
 bool Firewall::needsInstall()
 {
     if (getLastInstalled()==0 || getLastCompiled()==0) return true;
     return !(getLastModified()<=getLastCompiled() && 
             getLastCompiled()<=getLastInstalled());
 }
+
 bool Firewall::needsCompile()
 {
     return getLastModified()>getLastCompiled() || getLastCompiled()==0;
 }
+
 time_t Firewall::getLastModified()
 {
     return getInt("lastModified");
     
 }
+
 time_t Firewall::getLastInstalled()
 {
     return getInt("lastInstalled");
 }
+
 time_t Firewall::getLastCompiled()
 {
     return getInt("lastCompiled");
@@ -335,11 +358,14 @@ void   Firewall::updateLastCompiledTimestamp()
 {
     setInt("lastCompiled",time(NULL));
 }
+
 bool   Firewall::getInactive()
 {
     return getBool("inactive");
 }
+
 void   Firewall::setInactive(bool b)
 {
     setBool("inactive",b);
 }
+
