@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Host.cpp,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: Host.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -25,6 +25,7 @@
 */
 
 #include <assert.h>
+#include <iostream>
 
 #include <fwbuilder/libfwbuilder-config.h>
 
@@ -55,18 +56,16 @@ Host::~Host()  {}
 
 void Host::fromXML(xmlNodePtr root) throw(FWException)
 {
-//    const char *n;
-//    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("address")));
-//    assert(n!=NULL);
-//    setStr("address", n);
-//    FREEXMLBUFF(n);
-
     FWObject::fromXML(root);
 }
 
 xmlNodePtr Host::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+
     FWObject *o;
 
     for(FWObjectTypedChildIterator j=findByType(Interface::TYPENAME); j!=j.end(); ++j)
@@ -97,7 +96,7 @@ bool  Host::validateChild(FWObject *o)
  *  TODO:    Implement syntax checks for interfaces
  *
  */
-bool Host::Appropriate(Interface *i)
+bool Host::Appropriate(Interface*)
 {
   return true;
 }
@@ -117,73 +116,42 @@ FWOptions* Host::getOptionsObject()
     return FWOptions::cast( getFirstByType(HostOptions::TYPENAME) );
 }
 
-IPAddress Host::getAddress() const
-{
-    Interface *iface=NULL;
-    for(FWObjectTypedChildIterator j=findByType(Interface::TYPENAME); j!=j.end(); ++j)
-    {
-        iface=Interface::cast(*j);
-        if (iface->isLoopback()) continue;
-        if (iface->isManagement()) return iface->getAddress();
-    }
-    if (iface!=NULL) return iface->getAddress();
-    return IPAddress("0.0.0.0"); 
-}
-
-Netmask   Host::getNetmask() const
-{
-    Interface *iface=Interface::cast( getFirstByType(Interface::TYPENAME));
-    if (iface!=NULL) return iface->getNetmask();
-    return Netmask("0.0.0.0");
-}
-
-void Host::setAddress(const IPAddress &a)  { setAddress(a.toString()); }
-void Host::setNetmask(const Netmask   &nm) {}
-
-void Host::setAddress(const std::string &a)  
-{ 
-    Interface *iface=Interface::cast( getFirstByType(Interface::TYPENAME));
-    if (iface!=NULL) iface->setAddress(a); 
-}
-
-void Host::setNetmask(const std::string &nm)
-{ 
-    Interface *iface=Interface::cast( getFirstByType(Interface::TYPENAME));
-    if (iface!=NULL) iface->setNetmask(nm); 
-}
-
-guint32   Host::dimension()  const
-{
-    return 1;
-}
-
 Management *Host::getManagementObject()
 {
-    Management *res = dynamic_cast<Management *>(getFirstByType(Management::TYPENAME));
+    Management *res = dynamic_cast<Management *>(
+        getFirstByType(Management::TYPENAME));
     if(!res)
         add( res = Management::cast(getRoot()->create(Management::TYPENAME)) );
 //        add(res = new Management());
     return res;
 }
 
-/*
- * takes address from management interface and copies it into
- * Management object. If there is no management interface or no
- * address to be found, returns "0.0.0.0". May throw exception if
- * interface has invalid address.
+/**
+ * returns address from management interface. If there is no
+ * management interface or no address to be found, returns NULL.
+ * May throw exception if interface has invalid address.
  */
-IPAddress Host::getManagementAddress() throw(FWException)
+const InetAddr* Host::getManagementAddress() throw(FWException)
 {
-    Management *mgmt=getManagementObject();
-
-    for(FWObjectTypedChildIterator j=findByType(Interface::TYPENAME); j!=j.end(); ++j)
+    FWObjectTypedChildIterator j = findByType(Interface::TYPENAME);
+    for( ; j!=j.end(); ++j)
     {
-        Interface *iface=Interface::cast(*j);
+        Interface *iface = Interface::cast(*j);
         if (iface->isManagement())
         {
-            mgmt->setAddress( iface->getAddress() );
-            return iface->getAddress();
+            FWObjectTypedChildIterator k = iface->findByType(IPv4::TYPENAME);
+            if (k != k.end())
+                return Address::cast(*k)->getAddressPtr();
+
         }
     }    
-    return IPAddress("0.0.0.0");
+    return NULL;
 }
+
+const Address* Host::getAddressObject() const
+{
+    FWObjectTypedChildIterator j = findByType(Interface::TYPENAME);
+    if (j == j.end()) return NULL;
+    return Interface::cast(*j)->getAddressObject();
+}
+
