@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ICMPService.cpp,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: ICMPService.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -70,10 +70,14 @@ void ICMPService::fromXML(xmlNodePtr root) throw(FWException)
     }
 }
 
+xmlNodePtr ICMPService::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
 
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
 
-
-
-
-
+    return me;
+}
 
