@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: IPService.cpp,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: IPService.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -113,6 +113,51 @@ void IPService::fromXML(xmlNodePtr root) throw(FWException)
         setStr("ts", n);
         FREEXMLBUFF(n);
     }
+
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("tos")));
+    if(n!=NULL)
+    {
+        setStr("tos", n);
+        FREEXMLBUFF(n);
+    }
+
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("dscp")));
+    if(n!=NULL)
+    {
+        setStr("dscp", n);
+        FREEXMLBUFF(n);
+    }
+}
+
+xmlNodePtr IPService::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+
+    return me;
+}
+
+string IPService::getTOSCode() const
+{
+    return getStr("tos");
+}
+
+void IPService::setTOSCode(const string &c)
+{
+    setStr("tos", c);
+}
+
+string IPService::getDSCPCode() const
+{
+    return getStr("dscp");
+}
+
+void IPService::setDSCPCode(const string &c)
+{
+    setStr("dscp", c);
 }
 
 
