@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: IPService.h,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: IPService.h 177 2008-08-23 23:48:45Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -45,11 +45,18 @@ class IPService : public Service
     virtual ~IPService();
 
     virtual void fromXML(xmlNodePtr parent) throw(FWException);
+    virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
 
     DECLARE_FWOBJECT_SUBTYPE(IPService);
 
     virtual std::string getProtocolName();
     virtual int    getProtocolNumber();
+
+    std::string getTOSCode() const;
+    void setTOSCode(const std::string &c);
+
+    std::string getDSCPCode() const;
+    void setDSCPCode(const std::string &c);
 };
 
 }
