@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: IPv4.h,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: IPv4.h 166 2008-08-03 18:18:06Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -29,6 +29,7 @@
 
 #include <string>
 
+#include <fwbuilder/InetAddrMask.h>
 #include <fwbuilder/Address.h>
 #include <fwbuilder/FWException.h>
 
@@ -37,31 +38,29 @@ namespace libfwbuilder
 
 class IPv4 : public Address
 {
-    public:
+public:
     
     IPv4();
-    IPv4(const FWObject *root,bool prepopulate);
-    IPv4(const IPv4 &i);
-    IPv4(const std::string& addr,const std::string& mask);
+    IPv4(const FWObject *root, bool prepopulate);
 
     virtual ~IPv4();
 
     virtual void fromXML(xmlNodePtr parent) throw(FWException);
+    virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
 
+    virtual unsigned int dimension()  const { return 1; }
+    
     DECLARE_FWOBJECT_SUBTYPE(IPv4);
 
-    IPNetwork getIPNetwork() const throw(FWException);
+    virtual bool hasInetAddress() const { return true; }
 
-    void    setAddress(const std::string &a);
-    void    setNetmask(const std::string &nm);
+    virtual const Address* getAddressObject() const { return this; }
 
-    virtual IPAddress getAddress() const;
-    virtual Netmask   getNetmask() const;
-    virtual guint32   dimension()  const;
-
-    virtual void setAddress(const IPAddress &a);
-    virtual void setNetmask(const Netmask   &nm);
+    virtual void setAddress(const InetAddr &a);
+    virtual void setNetmask(const InetAddr &nm);
+    virtual void setAddressNetmask(const std::string& s);
 
+    virtual void dump(std::ostream &f,bool recursive,bool brief,int offset=0);
 };
 
 }
