@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Interface.cpp,v 1.6 2007/05/10 00:53:45 vkurland Exp $
+  $Id: Interface.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -25,6 +25,7 @@
 */
 
 #include <assert.h>
+#include <iostream>
 
 #include <fwbuilder/libfwbuilder-config.h>
 
@@ -32,6 +33,7 @@
 #include <fwbuilder/InterfacePolicy.h>
 #include <fwbuilder/XMLTools.h>
 #include <fwbuilder/IPv4.h>
+#include <fwbuilder/IPv6.h>
 #include <fwbuilder/FWObjectDatabase.h>
 
 using namespace std;
@@ -62,7 +64,8 @@ Interface::Interface():Address()
     snmp_type        = -1   ;     
 }
 
-Interface::Interface(const FWObject *root,bool prepopulate) : Address(root,prepopulate)
+Interface::Interface(const FWObject *root,bool prepopulate) :
+    Address(root,prepopulate)
 {
     setName("unknown");
     setBool("dyn",false);
@@ -78,7 +81,8 @@ Interface::Interface(const FWObject *root,bool prepopulate) : Address(root,prepo
 
 Interface::~Interface() {}
 
-FWObject& Interface::shallowDuplicate(const FWObject *o, bool preserve_id) throw(FWException)
+FWObject& Interface::shallowDuplicate(const FWObject *o, bool preserve_id)
+    throw(FWException)
 {
     FWObject::shallowDuplicate(o,preserve_id);
 
@@ -91,11 +95,12 @@ FWObject& Interface::shallowDuplicate(const FWObject *o, bool preserve_id) throw
     return *this;
 }
 
-FWObject& Interface::duplicate(const FWObject *x, bool preserve_id) throw(FWException)
+FWObject& Interface::duplicate(const FWObject *x, bool preserve_id)
+    throw(FWException)
 {
-    FWObject::duplicate(x,preserve_id);
+    FWObject::duplicate(x, preserve_id);
 
-    const Interface *rx=Interface::constcast(x);
+    const Interface *rx = Interface::constcast(x);
     if (rx!=NULL)
     {
         bcast_bits = rx->bcast_bits;
@@ -165,7 +170,7 @@ void Interface::fromXML(xmlNodePtr root) throw(FWException)
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("network_zone")));
     if (n!=NULL)
     {
-        setStr("network_zone",n);
+        setStr("network_zone", n);
         FREEXMLBUFF(n);
     }
 }
@@ -175,14 +180,28 @@ xmlNodePtr Interface::toXML(xmlNodePtr parent) throw(FWException)
     xmlNodePtr me = FWObject::toXML(parent, false);
     FWObject *o;
 
-    for(FWObjectTypedChildIterator j1=findByType(IPv4::TYPENAME); j1!=j1.end(); ++j1)
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+
+    for(FWObjectTypedChildIterator j1=findByType(IPv4::TYPENAME);
+        j1!=j1.end(); ++j1)
+    {
         if((o=(*j1))!=NULL )
             o->toXML(me);
-    
-    for(FWObjectTypedChildIterator j2=findByType(physAddress::TYPENAME); j2!=j2.end(); ++j2)
+    }
+    for(FWObjectTypedChildIterator j1=findByType(IPv6::TYPENAME);
+        j1!=j1.end(); ++j1)
+    {
+        if((o=(*j1))!=NULL )
+            o->toXML(me);
+    }
+    for(FWObjectTypedChildIterator j2=findByType(physAddress::TYPENAME);
+        j2!=j2.end(); ++j2)
+    {
         if((o=(*j2))!=NULL )
             o->toXML(me);
-    
+    }
     o=getFirstByType( InterfacePolicy::TYPENAME );
     if (o) o->toXML(me);
 
@@ -240,12 +259,16 @@ void Interface::setBroadcastBits(int _val) { bcast_bits=_val; }
 bool  Interface::validateChild(FWObject *o)
 {
     string otype=o->getTypeName();
-    return (otype==IPv4::TYPENAME || otype==physAddress::TYPENAME || otype==InterfacePolicy::TYPENAME );
+    return (otype==IPv4::TYPENAME ||
+            otype==IPv6::TYPENAME ||
+            otype==physAddress::TYPENAME ||
+            otype==InterfacePolicy::TYPENAME );
 }
 
 bool Interface::isLoopback() const
 {
-    return (getAddress()==IPAddress("127.0.0.1"));
+    const Address *iaddr = getAddressObject();
+    return (iaddr && *(iaddr->getAddressPtr()) == InetAddr::getLoopbackAddr());
 }
 
 physAddress*  Interface::getPhysicalAddress () const
@@ -270,79 +293,35 @@ void  Interface::setPhysicalAddress(const std::string &paddr)
     }
 }
 
-const string &Interface::getLabel() const
+const string& Interface::getLabel() const
 {
     return getStr("label");
 }
 
-void          Interface::setLabel(const string& n)
+void Interface::setLabel(const string& n)
 {
     setStr("label",n);
 }
-    
-const IPAddress Interface::getIPAddress() const throw(FWException)
-{
-    IPv4 *ipv4=IPv4::cast( getFirstByType( IPv4::TYPENAME ) );
-    if (ipv4!=NULL) return ipv4->getAddress();
-    else return IPAddress();
-}
 
-IPNetwork Interface::getIPNetwork() const throw(FWException)
+const Address* Interface::getAddressObject() const
 {
-    IPv4 *ipv4=IPv4::cast( getFirstByType(IPv4::TYPENAME) );
-    if (ipv4!=NULL) return ipv4->getIPNetwork();
-    else  return IPNetwork( IPAddress() , Netmask() );
-}
-
-IPAddress Interface::getAddress() const
-{
-    IPv4 *ipv4=IPv4::cast( getFirstByType( IPv4::TYPENAME ) );
-    if (ipv4!=NULL) return ipv4->getAddress();
-    else return IPAddress();
-}
-
-Netmask   Interface::getNetmask() const
-{
-    IPv4 *ipv4=IPv4::cast( getFirstByType( IPv4::TYPENAME ) );
-    if (ipv4!=NULL) return ipv4->getNetmask();
-    else return Netmask();
+    Address *res = Address::cast(getFirstByType(IPv4::TYPENAME));
+    if (res==NULL)
+        res = Address::cast(getFirstByType(IPv6::TYPENAME));
+    return res;
 }
 
 IPv4*  Interface::addIPv4()
 {
-    IPv4* ipv4=IPv4::cast( getRoot()->create(IPv4::TYPENAME) );
+    IPv4* ipv4 = IPv4::cast( getRoot()->create(IPv4::TYPENAME) );
     add(ipv4);
     return ipv4;
 }
 
-void Interface::setAddress(const IPAddress &a)  
-{ 
-    IPv4 *ipv4=IPv4::cast( getFirstByType( IPv4::TYPENAME ) );
-    if (ipv4==NULL)   ipv4=addIPv4();
-    ipv4->setAddress(a);
-}
-
-void Interface::setNetmask(const Netmask   &nm) 
-{ 
-    IPv4 *ipv4=IPv4::cast( getFirstByType( IPv4::TYPENAME ) );
-    if (ipv4==NULL)   ipv4=addIPv4();
-    ipv4->setNetmask(nm);
-}
-
-void Interface::setAddress(const std::string &a)  
+IPv6*  Interface::addIPv6()
 {
-    setAddress(IPAddress(a));
+    IPv6* ipv6 = IPv6::cast( getRoot()->create(IPv6::TYPENAME) );
+    add(ipv6);
+    return ipv6;
 }
 
-void Interface::setNetmask(const std::string &nm) 
-{
-    setNetmask(Netmask(nm));
-}
-
-guint32   Interface::dimension()  const
-{
-    return 1;
-}
-
-
-
