@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Library.cpp,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: Library.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -31,6 +31,7 @@
 #include <fwbuilder/XMLTools.h>
 
 using namespace libfwbuilder;
+using namespace std;
 
 const char *Library::TYPENAME={"Library"};
 
@@ -42,7 +43,7 @@ Library::~Library()
 {
 }
 
-bool  Library::validateChild(FWObject *o)
+bool  Library::validateChild(FWObject*)
 { 
     return true;   // anything goes
 }
@@ -58,6 +59,19 @@ void Library::fromXML(xmlNodePtr root) throw(FWException)
     FWObject::fromXML(root);
 }
 
+xmlNodePtr Library::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j)
+        (*j)->toXML(me);
+
+    return me;
+}
+
 
 
 
