@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Rule.cpp,v 1.15 2007/07/19 01:44:21 vkurland Exp $
+  $Id: Rule.cpp 177 2008-08-23 23:48:45Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -31,6 +31,7 @@
 #include <fwbuilder/libfwbuilder-config.h>
 
 #include <fwbuilder/FWObjectDatabase.h>
+#include <fwbuilder/Firewall.h>
 #include <fwbuilder/Rule.h>
 #include <fwbuilder/RuleElement.h>
 #include <fwbuilder/Policy.h>
@@ -39,6 +40,7 @@
 #include <fwbuilder/Policy.h>
 #include <fwbuilder/NAT.h>
 #include <fwbuilder/Routing.h>
+#include <fwbuilder/TagService.h>
 
 using namespace std;
 using namespace libfwbuilder;
@@ -47,16 +49,15 @@ const char *Rule::TYPENAME={"Rule"};
 
 Rule::Rule()
 {
-    remStr("name");
     setInt("position",0);
     enable();
     setFallback(false);
     setHidden(false);
+    setRuleGroupName("");
 }
 
 Rule::Rule(const FWObject *root,bool prepopulate) : Group(root,prepopulate)
 {
-    remStr("name");
     setInt("position",0);
     enable();
     setFallback(false);
@@ -70,7 +71,18 @@ int        Rule::getPosition() const {    return getInt("position");}
 void       Rule::disable()           {    setBool("disabled",true); }
 void       Rule::enable()            {    setBool("disabled",false);}
 bool       Rule::isDisabled() const  {    return( getBool("disabled") );}
-bool       Rule::isEmpty() const     {    return false;             }
+bool       Rule::isEmpty()           {    return false;             }
+
+void Rule::setBranch(RuleSet*) {};
+
+string Rule::getRuleGroupName() const { return getStr("group"); }
+
+void Rule::setRuleGroupName(const std::string &group_name)
+{
+    setStr("group", group_name);
+}
+
+
 
 FWObject& Rule::shallowDuplicate(const FWObject *x, bool preserve_id) throw(FWException)
 {
@@ -92,23 +104,44 @@ const char *PolicyRule::TYPENAME={"PolicyRule"};
 PolicyRule::PolicyRule()
 {
     setStr("action","Deny");
+
+    src_re = NULL;
+    dst_re = NULL;
+    srv_re = NULL;
+    itf_re = NULL;
+    when_re = NULL;
 }
 
 PolicyRule::PolicyRule(const FWObject *root,bool prepopulate) : Rule(root,prepopulate)
 {
     setStr("action","Deny");
 
+    src_re = NULL;
+    dst_re = NULL;
+    srv_re = NULL;
+    itf_re = NULL;
+    when_re = NULL;
+
     if (prepopulate)
     {
         FWObject         *re;
         FWObjectDatabase *db=(FWObjectDatabase*)root;
         assert(db);
 
-        re=db->create("Src");  assert(re!=NULL); add(re);
-        re=db->create("Dst");  assert(re!=NULL); add(re);
-        re=db->create("Srv");  assert(re!=NULL); add(re);
-        re=db->create("When"); assert(re!=NULL); add(re);
-        re=db->create("Itf");  assert(re!=NULL); add(re);
+        re=db->create("Src");  assert(re!=NULL);
+        add(re); src_re = RuleElementSrc::cast(re);
+
+        re=db->create("Dst");  assert(re!=NULL);
+        add(re); dst_re = RuleElementDst::cast(re);
+
+        re=db->create("Srv");  assert(re!=NULL);
+        add(re); srv_re = RuleElementSrv::cast(re);
+
+        re=db->create("When"); assert(re!=NULL);
+        add(re); when_re = RuleElementInterval::cast(re);
+
+        re=db->create("Itf");  assert(re!=NULL);
+        add(re); itf_re = RuleElementItf::cast(re);
 
         add( db->create(PolicyRuleOptions::TYPENAME) );
     }
@@ -120,37 +153,54 @@ FWObject& PolicyRule::shallowDuplicate(const FWObject *x, bool preserve_id) thro
     setDirection(rx->getDirection());
     setAction(rx->getAction());
     setLogging(rx->getLogging());
+
+    src_re = NULL;
+    dst_re = NULL;
+    srv_re = NULL;
+    itf_re = NULL;
+    when_re = NULL;
+
     return  Rule::shallowDuplicate(x,preserve_id);
 }
 
 
-RuleElementSrc*  PolicyRule::getSrc()  const
+RuleElementSrc*  PolicyRule::getSrc()
 {
-    return RuleElementSrc::cast(getFirstByType(RuleElementSrc::TYPENAME));
+    if (src_re) return src_re;
+    src_re = RuleElementSrc::cast(getFirstByType(RuleElementSrc::TYPENAME));
+    return src_re;
 }
 
-RuleElementDst*  PolicyRule::getDst()  const
+RuleElementDst*  PolicyRule::getDst()
 {
-    return RuleElementDst::cast(getFirstByType(RuleElementDst::TYPENAME));
+    if (dst_re) return dst_re;
+    dst_re = RuleElementDst::cast(getFirstByType(RuleElementDst::TYPENAME));
+    return dst_re;
 }
 
-RuleElementSrv*  PolicyRule::getSrv()  const
+RuleElementSrv*  PolicyRule::getSrv()
 {
-    return RuleElementSrv::cast(getFirstByType(RuleElementSrv::TYPENAME));
+    if (srv_re) return srv_re;
+    srv_re = RuleElementSrv::cast(getFirstByType(RuleElementSrv::TYPENAME));
+    return srv_re;
 }
 
-RuleElementItf*  PolicyRule::getItf()  const
+RuleElementItf*  PolicyRule::getItf()
 {
-    return RuleElementItf::cast(getFirstByType(RuleElementItf::TYPENAME));
+    if (itf_re) return itf_re;
+    itf_re = RuleElementItf::cast(getFirstByType(RuleElementItf::TYPENAME));
+    return itf_re;
 }
 
-RuleElementInterval* PolicyRule::getWhen()  const
+RuleElementInterval* PolicyRule::getWhen()
 {
-    return RuleElementInterval::cast(getFirstByType(RuleElementInterval::TYPENAME));
+    if (when_re) return when_re;
+    when_re = RuleElementInterval::cast(getFirstByType(RuleElementInterval::TYPENAME));
+    return when_re;
 }
 
 
-bool PolicyRule::isEmpty() const
+bool PolicyRule::isEmpty()
 {
   return (getSrc()->isAny() && 
           getDst()->isAny() && 
@@ -174,7 +224,7 @@ PolicyRule::Action PolicyRule::getAction()  const
     if (a=="Classify")   return Classify;
     if (a=="Custom")     return Custom;
     if (a=="Branch")     return Branch;
-    if (a=="Route")     return Route;
+    if (a=="Route")      return Route;
     return Deny;
 }
 
@@ -293,11 +343,21 @@ void PolicyRule::fromXML(xmlNodePtr root) throw(FWException)
         FREEXMLBUFF(n);
     }
 
+    n= FROMXMLCAST(xmlGetProp(root,TOXMLCAST("group")));
+    if(n)
+    {
+        setStr("group",n);
+        FREEXMLBUFF(n);
+    }
+
 }
 
 xmlNodePtr PolicyRule::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+//    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+
     FWObject *o;
     
     if ( (o=getFirstByType( RuleElementSrc::TYPENAME ))!=NULL )
@@ -331,7 +391,92 @@ FWOptions* PolicyRule::getOptionsObject()
 
 RuleSet*   PolicyRule::getBranch()
 {
-    return RuleSet::cast( getFirstByType(Policy::TYPENAME) );
+    FWObject *fw = this;
+    while (fw && !Firewall::isA(fw)) fw = fw->getParent();
+    assert(fw!=NULL);
+    string branch_id = getOptionsObject()->getStr("branch_id");
+    if (!branch_id.empty())
+    {
+        return RuleSet::cast(getRoot()->findInIndex(
+                                 FWObjectDatabase::getIntId(branch_id)));
+    } else
+    {
+        string branch_name = getOptionsObject()->getStr("branch_name");
+        if (!branch_name.empty())
+        {
+            return RuleSet::cast(
+                fw->findObjectByName(Policy::TYPENAME, branch_name));
+        } else
+            return NULL;
+    }
+}
+
+void PolicyRule::setBranch(RuleSet* ruleset)
+{
+    string branch_id = 
+        (ruleset) ? FWObjectDatabase::getStringId(ruleset->getId()) : "";
+    getOptionsObject()->setStr("branch_id", branch_id);
+}
+
+void PolicyRule::setTagObject(FWObject *tag_object)
+{
+    string tag_id =
+        (tag_object) ? FWObjectDatabase::getStringId(tag_object->getId()) : "";
+    getOptionsObject()->setStr("tagobject_id", tag_id);
+}
+
+FWObject* PolicyRule::getTagObject()
+{
+    if (getAction() == Tag)
+    {
+        string tagobj_id = getOptionsObject()->getStr("tagobject_id");
+        if (!tagobj_id.empty())
+        {
+            return getRoot()->findInIndex(
+                FWObjectDatabase::getIntId(tagobj_id));
+        }
+    }
+    return NULL;
+}
+
+string PolicyRule::getTagValue()
+{
+    if (getAction() == Tag)
+    {
+        TagService *tagobj = TagService::cast(getTagObject());
+        if (tagobj)  return tagobj->getCode();
+        else         return getOptionsObject()->getStr("tagvalue");
+    }
+    return "";
+}
+
+/**
+ * Removes reference to given object among children of 'this'. In case
+ * of PolicyRule we should also clear reference to it if action is
+ * Branch. Caveat: clear reference to it even if action is not branch
+ * right now but was in the past and reference got stuck in options.
+ *
+ * Do the same for the TagService
+ */
+void PolicyRule::removeRef(FWObject *obj)
+{
+    if (RuleSet::cast(obj))
+    {
+        string branch_id = FWObjectDatabase::getStringId(obj->getId());
+        string rule_branch_id = getOptionsObject()->getStr("branch_id");
+        if (branch_id == rule_branch_id)
+            getOptionsObject()->setStr("branch_id", "");
+    }
+
+    if (TagService::cast(obj))
+    {
+        string tag_id = FWObjectDatabase::getStringId(obj->getId());
+        string rule_tag_id = getOptionsObject()->getStr("tagobject_id");
+        if (tag_id == rule_tag_id)
+            getOptionsObject()->setStr("tagobject_id", "");
+    }
+
+    FWObject::removeRef(obj);
 }
 
 /***************************************************************************/
@@ -341,12 +486,28 @@ const char *NATRule::TYPENAME={"NATRule"};
 NATRule::NATRule() : Rule()
 {
     rule_type=Unknown;
+
+    osrc_re = NULL;
+    odst_re = NULL;
+    osrv_re = NULL;
+    tsrc_re = NULL;
+    tdst_re = NULL;
+    tsrv_re = NULL;
+    when_re = NULL;
 }
 
 NATRule::NATRule(const FWObject *root,bool prepopulate) : Rule(root,prepopulate)
 {
     rule_type=Unknown;
 
+    osrc_re = NULL;
+    odst_re = NULL;
+    osrv_re = NULL;
+    tsrc_re = NULL;
+    tdst_re = NULL;
+    tsrv_re = NULL;
+    when_re = NULL;
+
     if (prepopulate)
     {
         FWObject         *re;
@@ -365,42 +526,56 @@ NATRule::NATRule(const FWObject *root,bool prepopulate) : Rule(root,prepopulate)
     }
 }
 
-RuleElementOSrc*  NATRule::getOSrc()  const
+RuleElementOSrc*  NATRule::getOSrc()
 {
-    return RuleElementOSrc::cast(getFirstByType(RuleElementOSrc::TYPENAME));
+    if (osrc_re) return osrc_re;
+    osrc_re = RuleElementOSrc::cast(getFirstByType(RuleElementOSrc::TYPENAME));
+    return osrc_re;
 }
 
-RuleElementODst*  NATRule::getODst()  const
+RuleElementODst*  NATRule::getODst()
 {
-    return RuleElementODst::cast(getFirstByType(RuleElementODst::TYPENAME));
+    if (odst_re) return odst_re;
+    odst_re = RuleElementODst::cast(getFirstByType(RuleElementODst::TYPENAME));
+    return odst_re;
 }
 
-RuleElementOSrv*  NATRule::getOSrv()  const
+RuleElementOSrv*  NATRule::getOSrv()
 {
-    return RuleElementOSrv::cast(getFirstByType(RuleElementOSrv::TYPENAME));
+    if (osrv_re) return osrv_re;
+    osrv_re = RuleElementOSrv::cast(getFirstByType(RuleElementOSrv::TYPENAME));
+    return osrv_re;
 }
 
-RuleElementTSrc*  NATRule::getTSrc()  const
+RuleElementTSrc*  NATRule::getTSrc()
 {
-    return RuleElementTSrc::cast(getFirstByType(RuleElementTSrc::TYPENAME));
+    if (tsrc_re) return tsrc_re;
+    tsrc_re = RuleElementTSrc::cast(getFirstByType(RuleElementTSrc::TYPENAME));
+    return tsrc_re;
 }
 
-RuleElementTDst*  NATRule::getTDst()  const
+RuleElementTDst*  NATRule::getTDst()
 {
-    return RuleElementTDst::cast(getFirstByType(RuleElementTDst::TYPENAME));
+    if (tdst_re) return tdst_re;
+    tdst_re = RuleElementTDst::cast(getFirstByType(RuleElementTDst::TYPENAME));
+    return tdst_re;
 }
 
-RuleElementTSrv*  NATRule::getTSrv()  const
+RuleElementTSrv*  NATRule::getTSrv()
 {
-    return RuleElementTSrv::cast(getFirstByType(RuleElementTSrv::TYPENAME));
+    if (tsrv_re) return tsrv_re;
+    tsrv_re = RuleElementTSrv::cast(getFirstByType(RuleElementTSrv::TYPENAME));
+    return tsrv_re;
 }
 
-RuleElementInterval* NATRule::getWhen()  const
+RuleElementInterval* NATRule::getWhen()
 {
-    return RuleElementInterval::cast(getFirstByType(RuleElementInterval::TYPENAME));
+    if (when_re) return when_re;
+    when_re = RuleElementInterval::cast(getFirstByType(RuleElementInterval::TYPENAME));
+    return when_re;
 }
 
-bool NATRule::isEmpty() const
+bool NATRule::isEmpty()
 {
     RuleElement *osrc=getOSrc();
     RuleElement *odst=getODst();
@@ -420,15 +595,34 @@ void NATRule::fromXML(xmlNodePtr root) throw(FWException)
     FWObject::fromXML(root);
 
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("disabled")));
-    if(n)  setStr("disabled",n);
+    if(n)
+    {
+        setStr("disabled",n);
+        FREEXMLBUFF(n);
+    }
 
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("position")));
-    if(n)  setStr("position",n);
+    if(n)
+    {
+        setStr("position",n);
+        FREEXMLBUFF(n);
+    }
+
+    n= FROMXMLCAST(xmlGetProp(root,TOXMLCAST("group")));
+    if(n)
+    {
+        setStr("group",n);
+        FREEXMLBUFF(n);
+    }
+
 }
 
 xmlNodePtr NATRule::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+//    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+
     FWObject *o;
 
     if ( (o=getFirstByType( RuleElementOSrc::TYPENAME ))!=NULL )
@@ -468,7 +662,29 @@ FWOptions* NATRule::getOptionsObject()
 
 RuleSet*   NATRule::getBranch()
 {
-    return RuleSet::cast( getFirstByType(NAT::TYPENAME) );
+    FWObject *fw = getParent()->getParent();
+    assert(fw!=NULL);
+    string branch_id = getOptionsObject()->getStr("branch_id");
+    if (!branch_id.empty())
+    {
+        return RuleSet::cast(getRoot()->findInIndex(
+                                 FWObjectDatabase::getIntId(branch_id)));
+    } else
+    {
+        string branch_name = getOptionsObject()->getStr("branch_name");
+        if (!branch_name.empty())
+            return RuleSet::cast(fw->findObjectByName(NAT::TYPENAME,
+                                                  branch_name));
+        else
+            return NULL;
+    }
+}
+
+void NATRule::setBranch(RuleSet* ruleset)
+{
+    string branch_id = 
+        (ruleset) ? FWObjectDatabase::getStringId(ruleset->getId()) : "";
+    getOptionsObject()->setStr("branch_id", branch_id);
 }
 
 NATRule::NATRuleTypes NATRule::getRuleType() const
@@ -504,6 +720,15 @@ FWObject& NATRule::shallowDuplicate(const FWObject *x, bool preserve_id) throw(F
 {
     const NATRule *rx=NATRule::constcast(x);
     if (rx!=NULL) rule_type=rx->rule_type;
+
+    osrc_re = NULL;
+    odst_re = NULL;
+    osrv_re = NULL;
+    tsrc_re = NULL;
+    tdst_re = NULL;
+    tsrv_re = NULL;
+    when_re = NULL;
+
     return  Rule::shallowDuplicate(x,preserve_id);
 }
 
@@ -592,18 +817,41 @@ void RoutingRule::fromXML(xmlNodePtr root) throw(FWException)
     FWObject::fromXML(root);
 
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("disabled")));
-    if(n)  setStr("disabled",n);
-    
+    if(n)
+    {
+        setStr("disabled",n);
+        FREEXMLBUFF(n);
+    }
+
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("metric")));
-    if(n)  setStr("metric",n);
+    if(n)
+    {
+        setStr("metric",n);
+        FREEXMLBUFF(n);
+    }
 
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("position")));
-    if(n)  setStr("position",n);
+    if(n)
+    {
+        setStr("position",n);
+        FREEXMLBUFF(n);
+    }
+
+    n= FROMXMLCAST(xmlGetProp(root,TOXMLCAST("group")));
+    if(n)
+    {
+        setStr("group",n);
+        FREEXMLBUFF(n);
+    }
+
 }
 
 xmlNodePtr RoutingRule::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+//    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+
     FWObject *o;
 
     if ( (o=getFirstByType( RuleElementRDst::TYPENAME ))!=NULL )
@@ -631,7 +879,22 @@ FWOptions* RoutingRule::getOptionsObject()
 
 RuleSet*   RoutingRule::getBranch()
 {
-    return RuleSet::cast( getFirstByType(Routing::TYPENAME) );
+    FWObject *fw = getParent()->getParent();
+    assert(fw!=NULL);
+    string branch_id = getOptionsObject()->getStr("branch_id");
+    if (!branch_id.empty())
+    {
+        return RuleSet::cast(getRoot()->findInIndex(
+                                 FWObjectDatabase::getIntId(branch_id)));
+    } else
+    {
+        string branch_name = getOptionsObject()->getStr("branch_name");
+        if (!branch_name.empty())
+            return RuleSet::cast(fw->findObjectByName(Routing::TYPENAME,
+                                                      branch_name));
+        else
+            return NULL;
+    }
 }
 
 RoutingRule::RoutingRuleTypes RoutingRule::getRuleType() const
