@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: RuleElement.cpp,v 1.9 2007/05/22 20:27:42 vkurland Exp $
+  $Id: RuleElement.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -53,20 +53,14 @@ const char *RuleElement::TYPENAME={"RuleElement"};
 
 RuleElement::RuleElement() 
 {
-    remStr("comment");
-    remStr("name");
-    remStr("id");
-
     setBool("neg",false);
 }
 
-RuleElement::RuleElement(const FWObject *root,bool prepopulate) : FWObject(root,prepopulate)
+RuleElement::RuleElement(const FWObject *root, bool prepopulate) :
+    FWObject(root,prepopulate)
 {
-    remStr("comment");
-    remStr("name");
-    remStr("id");
-
-    setBool("neg",false);
+    setBool("neg", false);
+    setId(-1);
 }
 
 void RuleElement::fromXML(xmlNodePtr root) throw(FWException)
@@ -76,7 +70,7 @@ void RuleElement::fromXML(xmlNodePtr root) throw(FWException)
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("neg")));
     if(n)
     {
-        setStr("neg",n);
+        setStr("neg", n);
         FREEXMLBUFF(n);
     }
 
@@ -126,7 +120,7 @@ bool RuleElement::isAny()
 
 void RuleElement::setAnyElement()
 {
-    string any_id=getAnyElementId();
+    int any_id = getAnyElementId();
     
     FWObject *any=getRoot()->findInIndex( any_id );
     if (any)  addRef( any );
@@ -158,15 +152,15 @@ void RuleElement::_initialize(const FWObject *root)
    it is read from the file anyway
 */
 
-    string any_id=getAnyElementId();
+    int any_id = getAnyElementId();
     FWObject *any_obj=((FWObjectDatabase*)root)->checkIndex(any_id);
     if (any_obj)
         FWObject::addRef( any_obj );
 }
 
-string RuleElement::getAnyElementId()
+int RuleElement::getAnyElementId()
 {
-    return "";
+    return -1;
 }
 
 /*
@@ -200,8 +194,18 @@ RuleElementSrc::RuleElementSrc(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementSrc::getAnyElementId() {
-    return FWObjectDatabase::getAnyNetworkId();
+int RuleElementSrc::getAnyElementId() {
+    return FWObjectDatabase::ANY_ADDRESS_ID;
+}
+
+xmlNodePtr RuleElementSrc::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
 }
 
 
@@ -215,8 +219,18 @@ RuleElementDst::RuleElementDst(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementDst::getAnyElementId() {
-    return FWObjectDatabase::getAnyNetworkId();
+int RuleElementDst::getAnyElementId() {
+    return FWObjectDatabase::ANY_ADDRESS_ID;
+}
+
+xmlNodePtr RuleElementDst::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
 }
 
 
@@ -231,10 +245,21 @@ RuleElementSrv::RuleElementSrv(const FWObject *root,bool prepopulate) :
 }
 
 
-string RuleElementSrv::getAnyElementId() {
-    return FWObjectDatabase::getAnyIPServiceId();
+int RuleElementSrv::getAnyElementId() {
+    return FWObjectDatabase::ANY_SERVICE_ID;
 }
 
+xmlNodePtr RuleElementSrv::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
+}
+
+
 const char *RuleElementItf::TYPENAME={"Itf"};
 RuleElementItf::RuleElementItf() {}
 RuleElementItf::RuleElementItf(const FWObject *root,bool prepopulate) :
@@ -244,11 +269,22 @@ RuleElementItf::RuleElementItf(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementItf::getAnyElementId()
+int RuleElementItf::getAnyElementId()
 {
-    return FWObjectDatabase::getAnyNetworkId();
+    return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
+xmlNodePtr RuleElementItf::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
+}
+
+
 bool RuleElementItf::validateChild(FWObject *o)
 {
   if (FWObjectReference::cast(o)!=NULL) return true;  //I dont like this
@@ -286,30 +322,29 @@ bool RuleElementItf::validateChild(FWObject *o)
  */
 bool RuleElementItf::checkItfChildOfThisFw(FWObject *o)
 {
-    // Check if o is child of a firewall
     FWObject* o_tmp = getRoot()->findInIndex(o->getId());
-    bool child_of_firewall = false;
-    while( o_tmp && 
-           o_tmp->getRoot() != o_tmp && 
-           Firewall::cast(o_tmp) == NULL) o_tmp = o_tmp->getParent();
-    if( Firewall::cast(o_tmp)!=NULL) child_of_firewall = true;
-    
-    // Check if the Firewall the Interface belongs to is the same this RuleElement belongs to
-    bool same_firewall = false;
-    if( child_of_firewall)
+    FWObject* o_tmp2 = this;
+
+    while (o_tmp)
     {
-        FWObject* o_tmp2 = this;
-        while( Firewall::cast(o_tmp2) == NULL) o_tmp2 = o_tmp2->getParent();
-      
-        if( Firewall::cast(o_tmp) == Firewall::cast(o_tmp2)) same_firewall = true;
+        if (Firewall::cast(o_tmp))
+        {
+            // Check if the Firewall the Interface belongs to is the same this
+            // RuleElement belongs to
+            while (o_tmp2)
+            {
+                if (Firewall::cast(o_tmp2) && o_tmp->getId()==o_tmp2->getId())
+                    return true;
+                o_tmp2 = o_tmp2->getParent();
+            }
+            return false;
+        }
+        o_tmp = o_tmp->getParent();
     }
-    
-    return (child_of_firewall && same_firewall);
+    return false;
 }
 
 
-
-
 const char *RuleElementOSrc::TYPENAME={"OSrc"};
 RuleElementOSrc::RuleElementOSrc() {}
 RuleElementOSrc::RuleElementOSrc(const FWObject *root,bool prepopulate) :
@@ -319,8 +354,19 @@ RuleElementOSrc::RuleElementOSrc(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementOSrc::getAnyElementId() {
-    return FWObjectDatabase::getAnyNetworkId();
+int RuleElementOSrc::getAnyElementId() {
+    return FWObjectDatabase::ANY_ADDRESS_ID;
+}
+
+
+xmlNodePtr RuleElementOSrc::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
 }
 
 
@@ -333,8 +379,19 @@ RuleElementODst::RuleElementODst(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementODst::getAnyElementId() {
-    return FWObjectDatabase::getAnyNetworkId();
+int RuleElementODst::getAnyElementId() {
+    return FWObjectDatabase::ANY_ADDRESS_ID;
+}
+
+
+xmlNodePtr RuleElementODst::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
 }
 
 
@@ -347,8 +404,19 @@ RuleElementOSrv::RuleElementOSrv(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementOSrv::getAnyElementId() {
-    return FWObjectDatabase::getAnyIPServiceId();
+int RuleElementOSrv::getAnyElementId() {
+    return FWObjectDatabase::ANY_SERVICE_ID;
+}
+
+
+xmlNodePtr RuleElementOSrv::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
 }
 
 
@@ -363,8 +431,19 @@ RuleElementTSrc::RuleElementTSrc(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementTSrc::getAnyElementId() {
-    return FWObjectDatabase::getAnyNetworkId();
+int RuleElementTSrc::getAnyElementId() {
+    return FWObjectDatabase::ANY_ADDRESS_ID;
+}
+
+
+xmlNodePtr RuleElementTSrc::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
 }
 
 
@@ -377,8 +456,19 @@ RuleElementTDst::RuleElementTDst(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementTDst::getAnyElementId() {
-    return FWObjectDatabase::getAnyNetworkId();
+int RuleElementTDst::getAnyElementId() {
+    return FWObjectDatabase::ANY_ADDRESS_ID;
+}
+
+
+xmlNodePtr RuleElementTDst::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
 }
 
 
@@ -391,8 +481,19 @@ RuleElementTSrv::RuleElementTSrv(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementTSrv::getAnyElementId() {
-    return FWObjectDatabase::getAnyIPServiceId();
+int RuleElementTSrv::getAnyElementId() {
+    return FWObjectDatabase::ANY_SERVICE_ID;
+}
+
+
+xmlNodePtr RuleElementTSrv::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
 }
 
 
@@ -407,8 +508,19 @@ RuleElementInterval::RuleElementInterval(const FWObject *root,bool prepopulate)
         _initialize(root);
 }
 
-string RuleElementInterval::getAnyElementId() {
-    return FWObjectDatabase::getAnyIntervalId();
+int RuleElementInterval::getAnyElementId() {
+    return FWObjectDatabase::ANY_INTERVAL_ID;
+}
+
+
+xmlNodePtr RuleElementInterval::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
 }
 
 
@@ -423,10 +535,21 @@ RuleElementRDst::RuleElementRDst(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementRDst::getAnyElementId() {
-    return FWObjectDatabase::getAnyNetworkId();
+int RuleElementRDst::getAnyElementId() {
+    return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
+xmlNodePtr RuleElementRDst::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
+}
+
+
 const char *RuleElementRGtw::TYPENAME={"RGtw"};
 RuleElementRGtw::RuleElementRGtw() {}
 RuleElementRGtw::RuleElementRGtw(const FWObject *root,bool prepopulate) :
@@ -436,10 +559,21 @@ RuleElementRGtw::RuleElementRGtw(const FWObject *root,bool prepopulate) :
         _initialize(root);
 }
 
-string RuleElementRGtw::getAnyElementId() {
-    return FWObjectDatabase::getAnyNetworkId();
+int RuleElementRGtw::getAnyElementId() {
+    return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
+xmlNodePtr RuleElementRGtw::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
+}
+
+
 bool RuleElementRGtw::validateChild(FWObject *o) {
 
     if( getChildrenCount() > 0 && !isAny()) return false;
@@ -482,84 +616,6 @@ bool RuleElementRGtw::checkSingleIPAdress(FWObject *o) {
             (IPv4::cast(o)!=NULL || FWObjectReference::cast(o)!=NULL)));
 }
 
-// the IP address of the gateway has to be in a local network of the firewall
-bool RuleElementRGtw::checkReachableIPAdress(FWObject *o) {
-
-    FWObject* o_tmp = this;
-    while( Firewall::cast(o_tmp) == NULL) o_tmp = o_tmp->getParent();
-    // let's walk over all interfaces of this firewall
-    FWObjectTypedChildIterator j=o_tmp->findByType(Interface::TYPENAME);
-
-
-    if( Host::cast(o) != NULL) {
-
-        Host *host=Host::cast(o);
-        IPAddress ip_host = host->getAddress();
-
-        for ( ; j!=j.end(); ++j ) {
-            Interface *i_firewall=Interface::cast(*j);
-            for(FWObjectTypedChildIterator fw_ips=i_firewall->findByType(IPv4::TYPENAME); fw_ips!=fw_ips.end(); ++fw_ips) {
-                IPv4 *ipv4_obj_firewall = IPv4::cast(*fw_ips);
-
-                Netmask n_firewall = ipv4_obj_firewall->getNetmask();
-                IPAddress ip_firewall = ipv4_obj_firewall->getAddress();
-
-                if ((n_firewall.to32BitInt() & ip_firewall.to32BitInt()) == (n_firewall.to32BitInt() & ip_host.to32BitInt())) {
-                    return true;
-                }
-            }
-        }
-        return false;
-
-    } else if( Interface::cast(o) != NULL) {
-
-        Interface *gw_interface=Interface::cast(o);
-        IPAddress ip_gateway = gw_interface->getAddress();
-
-        // walk over all interfaces of this firewall
-        for ( ; j!=j.end(); ++j ) {
-            Interface *if_firewall=Interface::cast(*j);
-            FWObjectTypedChildIterator addresses=if_firewall->findByType(IPv4::TYPENAME);
-
-            // check all IPv4 addresses of this firewall interface
-            for ( ; addresses!=addresses.end(); ++addresses ) {
-                IPv4 *ipv4_obj_firewall = IPv4::cast(*addresses);
-                Netmask n_firewall = ipv4_obj_firewall->getNetmask();
-                IPAddress ip_firewall = ipv4_obj_firewall->getAddress();
-
-                if ((n_firewall.to32BitInt() & ip_firewall.to32BitInt()) == (n_firewall.to32BitInt() & ip_gateway.to32BitInt())) {
-                    return true;
-                }
-            }
-        }
-        return false;
-
-    } else if( IPv4::cast(o) != NULL) {
-
-        IPv4 *ipv4=IPv4::cast(o);
-        IPAddress ip_ipv4 = ipv4->getAddress();
-
-        for ( ; j!=j.end(); ++j ) {
-            Interface *if_firewall=Interface::cast(*j);
-            FWObjectTypedChildIterator addresses=if_firewall->findByType(IPv4::TYPENAME);
-
-            // check all IPv4 addresses of this firewall interface
-            for ( ; addresses!=addresses.end(); ++addresses ) {
-                IPv4 *ipv4_obj_firewall = IPv4::cast(*addresses);
-                
-                Netmask n_firewall = ipv4_obj_firewall->getNetmask();
-                IPAddress ip_firewall = ipv4_obj_firewall->getAddress();
-                
-                if ((n_firewall.to32BitInt() & ip_firewall.to32BitInt()) == (n_firewall.to32BitInt() & ip_ipv4.to32BitInt())) {
-                    return true;
-                } 
-            }
-        } return false;
-
-    } else return true;
-
-}
-
 const char *RuleElementRItf::TYPENAME={"RItf"};
 RuleElementRItf::RuleElementRItf()  {}
 RuleElementRItf::RuleElementRItf(const FWObject *root,bool prepopulate) :
