@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: RuleElement.h,v 1.6 2006/09/10 22:40:37 vkurland Exp $
+  $Id: RuleElement.h 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -62,7 +62,7 @@ public:
 
     void      setAnyElement();
     void      reset();
-    virtual   std::string  getAnyElementId();
+    virtual   int getAnyElementId();
     bool      isAny();
 
     /**
@@ -89,7 +89,8 @@ class RuleElementSrc : public ObjectGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementSrc);
     RuleElementSrc();
     RuleElementSrc(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementDst : public ObjectGroup, public RuleElement {
@@ -97,7 +98,8 @@ class RuleElementDst : public ObjectGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementDst);
     RuleElementDst();
     RuleElementDst(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementSrv : public ServiceGroup, public RuleElement {
@@ -105,7 +107,8 @@ class RuleElementSrv : public ServiceGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementSrv);
     RuleElementSrv();
     RuleElementSrv(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementItf : public ObjectGroup, public RuleElement {
@@ -113,9 +116,10 @@ class RuleElementItf : public ObjectGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementItf);
     RuleElementItf();
     RuleElementItf(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
-    virtual   bool         validateChild(FWObject *o);
+    virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     bool checkItfChildOfThisFw(FWObject *o);
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementInterval : public IntervalGroup, public RuleElement {
@@ -123,7 +127,8 @@ class RuleElementInterval : public IntervalGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementInterval);
     RuleElementInterval();
     RuleElementInterval(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementOSrc : public ObjectGroup, public RuleElement {
@@ -131,7 +136,8 @@ class RuleElementOSrc : public ObjectGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementOSrc);
     RuleElementOSrc();
     RuleElementOSrc(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementODst : public ObjectGroup, public RuleElement {
@@ -139,7 +145,8 @@ class RuleElementODst : public ObjectGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementODst);
     RuleElementODst();
     RuleElementODst(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementOSrv : public ServiceGroup, public RuleElement {
@@ -147,7 +154,8 @@ class RuleElementOSrv : public ServiceGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementOSrv);
     RuleElementOSrv();
     RuleElementOSrv(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementTSrc : public ObjectGroup, public RuleElement {
@@ -155,7 +163,8 @@ class RuleElementTSrc : public ObjectGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementTSrc);
     RuleElementTSrc();
     RuleElementTSrc(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementTDst : public ObjectGroup, public RuleElement {
@@ -163,7 +172,8 @@ class RuleElementTDst : public ObjectGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementTDst);
     RuleElementTDst();
     RuleElementTDst(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementTSrv : public ServiceGroup, public RuleElement {
@@ -171,7 +181,8 @@ class RuleElementTSrv : public ServiceGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementTSrv);
     RuleElementTSrv();
     RuleElementTSrv(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementRDst : public ObjectGroup, public RuleElement {
@@ -179,7 +190,8 @@ class RuleElementRDst : public ObjectGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementRDst);
     RuleElementRDst();
     RuleElementRDst(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
+    virtual int getAnyElementId();
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementRGtw : public ObjectGroup, public RuleElement {
@@ -187,10 +199,11 @@ class RuleElementRGtw : public ObjectGroup, public RuleElement {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementRGtw);
     RuleElementRGtw();
     RuleElementRGtw(const FWObject *root,bool prepopulate);
-    virtual   std::string  getAnyElementId();
-    virtual   bool         validateChild(FWObject *o);
+    virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     bool checkSingleIPAdress(FWObject *o);
     bool checkReachableIPAdress(FWObject *o);
+    virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
 };
 
 class RuleElementRItf : public RuleElementItf {
@@ -198,7 +211,7 @@ class RuleElementRItf : public RuleElementItf {
     DECLARE_FWOBJECT_SUBTYPE(RuleElementRItf);
     RuleElementRItf();
     RuleElementRItf(const FWObject *root,bool prepopulate);
-    virtual   bool         validateChild(FWObject *o);
+    virtual bool validateChild(FWObject *o);
 };
 
 }
