@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: RuleSet.cpp,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: RuleSet.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -32,6 +32,8 @@
 #include <fwbuilder/FWOptions.h>
 #include <fwbuilder/XMLTools.h>
 
+#include <iostream>
+
 using namespace std;
 using namespace libfwbuilder;
  
@@ -42,13 +44,75 @@ RuleSet::RuleSet()
     setName("RuleSet");
 }
 
-RuleSet::RuleSet(const FWObject *root,bool prepopulate)
+RuleSet::RuleSet(const FWObject*,bool)
 {
     setName("RuleSet");
+    ipv6 = false;
+    top = false;
 }
 
 RuleSet::~RuleSet() {}
 
+void RuleSet::fromXML(xmlNodePtr root) throw(FWException)
+{
+    FWObject::fromXML(root);
+
+    const char *n;
+
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("ipv6_rule_set")));
+    if (n!=NULL)
+    {
+        ipv6 = (string(n)=="True" || string(n)=="true");
+        FREEXMLBUFF(n);
+    }
+
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("top_rule_set")));
+    if (n!=NULL)
+    {
+        top = (string(n)=="True" || string(n)=="true");
+        FREEXMLBUFF(n);
+    }
+}
+
+xmlNodePtr RuleSet::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+
+    string ipv6_s = (ipv6)?"True":"False";
+    string top_s = (top)?"True":"False";
+    xmlNewProp(me, 
+               TOXMLCAST("ipv6_rule_set"),
+               STRTOXMLCAST(ipv6_s));
+    xmlNewProp(me, 
+               TOXMLCAST("top_rule_set"),
+               STRTOXMLCAST(top_s));
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
+        (*j)->toXML(me);
+
+    return me;
+}
+
+FWObject& RuleSet::shallowDuplicate(const FWObject *o, bool preserve_id)
+    throw(FWException)
+{
+    const RuleSet *other = RuleSet::constcast(o);
+
+    FWObject::shallowDuplicate(o,preserve_id);
+
+    ipv6 = other->ipv6;
+    top = other->top;
+
+    return *this;
+}
+
+
+
+
+
 Rule* RuleSet::insertRuleAtTop() 
 {
     Rule *r=createRule();
