@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ServiceGroup.cpp,v 1.6 2006/09/10 22:40:37 vkurland Exp $
+  $Id: ServiceGroup.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -63,4 +63,17 @@ FWReference* ServiceGroup::createRef()
     return ref;
 }
 
+xmlNodePtr ServiceGroup::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+
+    for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j)
+        (*j)->toXML(me);
+
+    return me;
+}
+
 
