@@ -7,7 +7,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: TagService.cpp,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: TagService.cpp 192 2008-09-21 09:39:24Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -52,12 +52,12 @@ TagService::TagService(const FWObject *root,bool prepopulate) : Service(root,pre
 
 TagService::~TagService() {}
 
-void TagService::setCode(string p)
+void TagService::setCode(const string &p)
 {
     setStr("tagcode", p);
 }
 
-string TagService::getCode()
+string TagService::getCode() const
 {
     return getStr("tagcode");
 }
@@ -74,6 +74,16 @@ void TagService::fromXML(xmlNodePtr root) throw(FWException)
 
 }
 
+xmlNodePtr TagService::toXML(xmlNodePtr parent) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+
+    return me;
+}
+
 
 
 
