@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: XMLTools.h,v 1.5 2006/08/18 03:59:32 vkurland Exp $
+  $Id: XMLTools.h 35 2008-04-26 19:13:33Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -88,7 +88,7 @@ class XMLTools
 
         virtual ~UpgradePredicate() {}
 
-        virtual bool operator()(const std::string &string) const 
+        virtual bool operator()(const std::string&) const 
         { 
             return true;
         }
