@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Compiler.cpp,v 1.29 2008/03/10 03:41:57 vkurland Exp $
+  $Id: Compiler.cpp 176 2008-08-19 04:29:54Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,8 +32,10 @@
 #include "fwbuilder/RuleElement.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Network.h"
+#include "fwbuilder/NetworkIPv6.h"
 #include "fwbuilder/IPService.h"
 #include "fwbuilder/ICMPService.h"
+#include "fwbuilder/ICMP6Service.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
 #include "fwbuilder/CustomService.h"
@@ -41,6 +43,7 @@
 #include "fwbuilder/Rule.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
+#include "fwbuilder/IPv6.h"
 #include "fwbuilder/InterfacePolicy.h"
 #include "fwbuilder/DNSName.h"
 #include "fwbuilder/MultiAddress.h"
@@ -72,7 +75,7 @@ Compiler::~Compiler() {}
 
 int Compiler::cache_objects(FWObject *o)
 {
-    if ( !o->getId().empty() )  cacheObj(o);
+    if ( o->getId() > -1 )  cacheObj(o);
 
     int n=0;
     for (FWObject::iterator i=o->begin(); i!=o->end(); ++i) {
@@ -140,6 +143,25 @@ string Compiler::getCompiledScript()
     return res;
 }
 
+bool Compiler::haveErrorsAndWarnings()
+{
+    return (int(output.tellp()) > 0);
+}
+
+string Compiler::getErrors(const string &comment_sep)
+{
+    ostringstream ostr;
+    istringstream istr(errors_buffer.str());
+    while (!istr.eof())
+    {
+        string tmpstr;
+        getline(istr, tmpstr);
+        ostr << comment_sep << tmpstr << endl;
+    }
+    errors_buffer.str("");
+    return ostr.str();
+}
+
 void Compiler::_init(FWObjectDatabase *_db, const string &fwobjectname)
 { 
     initialized=false;
@@ -164,33 +186,39 @@ void Compiler::_init(FWObjectDatabase *_db, const string &fwobjectname)
 }
 
 Compiler::Compiler(FWObjectDatabase *_db, 
-		   const string &fwobjectname)
+		   const string &fwobjectname, bool ipv6_policy)
 {
     source_ruleset = NULL;
     ruleSetName = "";
     test_mode = false;
     osconfigurator = NULL;
+    countIPv6Rules = 0;
+    ipv6 = ipv6_policy;
     _init(_db,fwobjectname);
 }
 
 Compiler::Compiler(FWObjectDatabase *_db, 
-		   const string &fwobjectname,
+		   const string &fwobjectname, bool ipv6_policy,
 		   OSConfigurator *_oscnf)
 {
     source_ruleset = NULL;
     ruleSetName = "";
     test_mode = false;
     osconfigurator = _oscnf;
+    countIPv6Rules = 0;
+    ipv6 = ipv6_policy;
     _init(_db,fwobjectname);
 }
 
 // this constructor is used by class Preprocessor, it does not call _init
-Compiler::Compiler(FWObjectDatabase *_db)
+Compiler::Compiler(FWObjectDatabase*, bool ipv6_policy)
 {
     source_ruleset = NULL;
     ruleSetName = "";
     test_mode = false;
     osconfigurator = NULL;
+    countIPv6Rules = 0;
+    ipv6 = ipv6_policy;
     initialized = false;
     _cntr_ = 1; 
     fw = NULL; 
@@ -201,28 +229,14 @@ Compiler::Compiler(FWObjectDatabase *_db)
     verbose = true;
 }
 
-string Compiler::createRuleLabel(const string &txt,
-                       int rule_num)
-{
-    return createRuleLabel(txt,NULL,rule_num);
-}
-
-string Compiler::createRuleLabel(Interface *iface,
-                       int rule_num)
-{
-    return createRuleLabel("",iface,rule_num);
-}
-
-
-string Compiler::createRuleLabel(const string &txt,
-                                 Interface *iface,
-                                 int rule_num)
+string Compiler::createRuleLabel(const std::string &prefix,
+                                 const string &txt, int rule_num)
 {
     ostringstream  str;
-    
+
+    if (!prefix.empty()) str << prefix << " ";
     str << rule_num << " ";
-    if (iface!=NULL) str << "(" << iface->getName() << ")";
-    else             str << "(" << txt << ")";
+    str << "(" << txt << ")";
     return str.str();
 }
 
@@ -239,6 +253,8 @@ void Compiler::error(const string &errstr)
     cout << flush;
     cerr << "Error (" << myPlatformName() << "): ";
     cerr << errstr << endl;
+    errors_buffer << "Error (" << myPlatformName() << "): ";
+    errors_buffer << errstr << endl;
 }
 
 void Compiler::warning(const string &warnstr)
@@ -246,21 +262,18 @@ void Compiler::warning(const string &warnstr)
     cout << flush;
     cerr << "Warning (" << myPlatformName() << "): ";
     cerr << warnstr << endl;
+    errors_buffer << "Warning (" << myPlatformName() << "): ";
+    errors_buffer << warnstr << endl;
 }
 
-
-
-
 string Compiler::getUniqueRuleLabel()
 {
-    char str[64];
-    sprintf(str,"R_%d",_cntr_);
+    ostringstream str;
+    str << "R_" << _cntr_;
     _cntr_++;
-    return str;
+    return str.str();
 }
 
-
-
 void Compiler::compile()
 {
     assert(fw);
@@ -268,9 +281,34 @@ void Compiler::compile()
 
 }
 
+bool Compiler::MatchesAddressFamily(FWObject *o)
+{
+    if (Address::cast(o))
+    {
+        const  InetAddr *inet_addr = Address::cast(o)->getAddressPtr();
+        if (inet_addr)
+        {
+            if (ipv6)
+            {
+                if (inet_addr->isV6()) return true;
+            } else
+            {
+                if (inet_addr->isV4()) return true;
+            }
+        } else
+        {
+            // Address object with no ip address (e.g. dynamic interface
+            // or run-time address table)
+            return true;
+        }
+        return false;
+    }
+    // not an address object at all
+    return true;
+}
 
 
-void Compiler::_expand_group_recursive(FWObject *o,list<FWObject*> &ol)
+void Compiler::_expand_group_recursive(FWObject *o, list<FWObject*> &ol)
 {
 /* special case: MultiAddress. This class inherits ObjectGroup, but
  * should not be expanded if it is expanded at run time
@@ -279,20 +317,40 @@ void Compiler::_expand_group_recursive(FWObject *o,list<FWObject*> &ol)
  * run-time address tables
  */
     MultiAddress *adt = MultiAddress::cast(o);
-    if ((Group::cast(o)!=NULL && adt==NULL) || (adt!=NULL && adt->isCompileTime()))
+    if ((Group::cast(o)!=NULL && adt==NULL) ||
+        (adt!=NULL && adt->isCompileTime()))
     {
 	for (FWObject::iterator i2=o->begin(); i2!=o->end(); ++i2)
         {
-	    FWObject *o1= *i2;
-	    if (FWReference::cast(o1)!=NULL) o1=FWReference::cast(o1)->getPointer();
+	    FWObject *o1 = *i2;
+	    if (FWReference::cast(o1)!=NULL)
+                o1=FWReference::cast(o1)->getPointer();
 	    assert(o1);
 
-	    _expand_group_recursive(o1,ol);
+	    _expand_group_recursive(o1, ol);
 	}
     } else
     {
-	o->ref();
-	ol.push_back( o );
+        if (o->getId() == FWObjectDatabase::ANY_ADDRESS_ID)
+        {
+            o->ref();
+            ol.push_back( o );
+        } else
+        {
+            if (Address::cast(o) && Address::cast(o)->hasInetAddress())
+            {
+                if (MatchesAddressFamily(o))
+                {
+                    o->ref();
+                    ol.push_back( o );
+                }
+            } else
+            {
+                // not an address object at all
+                o->ref();
+                ol.push_back( o );
+            }
+        }
     }
 }
 
@@ -308,7 +366,7 @@ void Compiler::expandGroupsInRuleElement(RuleElement *s)
         FWObject *o= *i1;
         if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         assert(o);
-        _expand_group_recursive(o,cl);
+        _expand_group_recursive(o, cl);
     }
 
     s->clearChildren();
@@ -320,13 +378,12 @@ void Compiler::expandGroupsInRuleElement(RuleElement *s)
     }
 }
 
-void Compiler::_expand_addr_recursive(Rule *rule,FWObject *s,list<FWObject*> &ol)
+void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
+                                      list<FWObject*> &ol)
 {
     Interface *rule_iface = fw_interfaces[rule->getInterfaceId()];
     bool on_loopback= ( rule_iface && rule_iface->isLoopback() );
 
-    //cerr << "Rule " << rule->getLabel() << " object s: " << s->getName() << " " << s->getTypeName() << endl;
-
     list<FWObject*> addrlist;
 
     for (FWObject::iterator i1=s->begin(); i1!=s->end(); ++i1) 
@@ -335,15 +392,41 @@ void Compiler::_expand_addr_recursive(Rule *rule,FWObject *s,list<FWObject*> &ol
 	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	assert(o);
 
-        if (Address::cast(o)!=NULL || MultiAddress::cast(o)!=NULL) addrlist.push_back(o);
+        // this condition includes Host, Firewall and Interface
+        if (Address::cast(o) &&
+            !Address::cast(o)->hasInetAddress())
+        {
+            addrlist.push_back(o);
+            continue;
+        }
+
+        // IPv4, IPv6, Network, NetworkIPv6
+        if (Address::cast(o) &&
+            Address::cast(o)->hasInetAddress() &&
+            MatchesAddressFamily(o))
+        {            
+            addrlist.push_back(o);
+            continue;
+        }
+
+        if (o->getId() == FWObjectDatabase::ANY_ADDRESS_ID ||
+            MultiAddress::cast(o)!=NULL ||
+            Interface::cast(o) ||
+            physAddress::cast(o))
+        {
+            addrlist.push_back(o);
+            continue;
+        }
     }
 
-    //cerr << "Rule " << rule->getLabel() << "  addrlist.size()=" << addrlist.size() << endl;
-
-    if (addrlist.empty()) ol.push_back(s);
+    if (addrlist.empty())
+    {
+        if (RuleElement::cast(s)==NULL) ol.push_back(s);
+    }
     else
     {
-        for (list<FWObject*>::iterator i2=addrlist.begin(); i2!=addrlist.end(); ++i2)
+        for (list<FWObject*>::iterator i2=addrlist.begin();
+             i2!=addrlist.end(); ++i2)
         {
             if (Interface::cast(*i2)!=NULL)
             {
@@ -358,7 +441,7 @@ void Compiler::_expand_addr_recursive(Rule *rule,FWObject *s,list<FWObject*> &ol
 
                 continue;
             }
-            _expand_addr_recursive(rule,*i2,ol);
+            _expand_addr_recursive(rule, *i2, ol);
         }
     }
 }
@@ -403,7 +486,7 @@ void Compiler::_expandInterface(Interface *iface,  std::list<FWObject*> &ol)
             continue;
         }
 
-        if (Address::cast(o)!=NULL) ol.push_back(o);
+        if (Address::cast(o)!=NULL && MatchesAddressFamily(o)) ol.push_back(o);
     }
 
 //    FWObjectTypedChildIterator j=iface->findByType(IPv4::TYPENAME);
@@ -418,66 +501,76 @@ void Compiler::_expandInterface(Interface *iface,  std::list<FWObject*> &ol)
  * a pointer at either src or dst in the rule
  *
  */
-void Compiler::_expandAddr(Rule *rule,FWObject *s) 
+void Compiler::_expandAddr(Rule *rule, FWObject *s) 
 {
     list<FWObject*> cl;
 
-    _expand_addr_recursive(rule,s,cl);
+    _expand_addr_recursive(rule, s, cl);
 
-    if ( ! cl.empty() )
+    s->clearChildren();
+
+    for (FWObject::iterator i1=cl.begin(); i1!=cl.end(); ++i1) 
     {
-	s->clearChildren();
-	for (FWObject::iterator i1=cl.begin(); i1!=cl.end(); ++i1) 
-        {
-/*
-            cerr << "Compiler::_expandAddr: adding object: (*i1)->name=";
-            cerr << (*i1)->getName();
-            cerr << "  (*i1)->id=" << (*i1)->getId();
-            cerr << endl;
-*/
-	    s->addRef( *i1 );
-	}
+#if 0
+        cerr << "Compiler::_expandAddr: adding object: (*i1)->name=";
+        cerr << (*i1)->getName();
+        cerr << "  (*i1)->id=" << (*i1)->getId();
+        cerr << "  type=" << (*i1)->getTypeName();
+        cerr << endl;
+#endif        
+        s->addRef( *i1 );
     }
 }
 
-void Compiler::_expandAddressRanges(Rule *rule,FWObject *s) 
+/**
+ * replace address range objects in the rule element 're' with series of
+ * regular address obejcts. Drop objects that do not match current
+ * address family.
+ */
+void Compiler::_expandAddressRanges(Rule*, FWObject *re)
 {
     list<FWObject*> cl;
-    for (FWObject::iterator i1=s->begin(); i1!=s->end(); ++i1) 
+    for (FWObject::iterator i1=re->begin(); i1!=re->end(); ++i1) 
     {
 	FWObject *o= *i1;
 	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	assert(o!=NULL);
 
-	if (AddressRange::cast(o)==NULL) cl.push_back(o);
-	else
+        // if this is address range, check if it matches current address
+        // family. If it is not address range, put it back into the rule element
+        // If it is address range but it does not match address family,
+        // throw it away.
+	if (AddressRange::cast(o)!=NULL)
         {
-	    IPAddress a1=AddressRange::cast(o)->getRangeStart();
-	    IPAddress a2=AddressRange::cast(o)->getRangeEnd();
-            vector<IPNetwork> vn=libfwbuilder::convertAddressRange(a1,a2);
-
-            for (vector<IPNetwork>::iterator i=vn.begin(); i!=vn.end(); i++)
+            if (MatchesAddressFamily(o))
             {
-                Network *h;
-                h= Network::cast(dbcopy->create(Network::TYPENAME) );
-                h->setName(string("%n-")+a1.toString()+string("%") );
-                h->setNetmask(i->getNetmask());
-                h->setAddress(i->getAddress());
-                cacheObj(h); // to keep cache consistent
-                dbcopy->add(h,false);
-                cl.push_back(h);
-            }
-	}
-    }
-    if ( ! cl.empty() )
-    {
-	s->clearChildren();
+                InetAddr a1 = AddressRange::cast(o)->getRangeStart();
+                InetAddr a2 = AddressRange::cast(o)->getRangeEnd();
+                vector<InetAddrMask> vn = 
+                    libfwbuilder::convertAddressRange(a1,a2);
 
-	for (FWObject::iterator i1=cl.begin(); i1!=cl.end(); ++i1)
+                for (vector<InetAddrMask>::iterator i=vn.begin();
+                     i!=vn.end(); i++)
+                {
+                    Network *h;
+                    h= Network::cast(dbcopy->create(Network::TYPENAME) );
+                    h->setName(string("%n-")+(*i).toString()+string("%") );
+                    h->setNetmask(*(i->getNetmaskPtr()));
+                    h->setAddress(*(i->getAddressPtr()));
+                    cacheObj(h); // to keep cache consistent
+                    dbcopy->add(h,false);
+                    cl.push_back(h);
+                }
+            }
+	} else
         {
-	    s->addRef( *i1 );
-	}
+            cl.push_back(o);
+        }
     }
+
+    re->clearChildren();
+    for (FWObject::iterator i1=cl.begin(); i1!=cl.end(); ++i1)
+        re->addRef( *i1 );
 }
 
 void Compiler::normalizePortRange(int &rs,int &re)
@@ -488,12 +581,43 @@ void Compiler::normalizePortRange(int &rs,int &re)
 }
 
 
+bool Compiler::_complexMatchWithAddress(const InetAddr *obj1_addr,
+                                        Interface *iface,
+                                        const string &address_type,
+                                        bool recognize_broadcasts)
+{
+    FWObjectTypedChildIterator k = iface->findByType(address_type);
+    for ( ; k!=k.end(); ++k ) 
+    {
+        Address *addr_obj = Address::cast(*k);
+                    
+        if ( *(addr_obj->getAddressPtr())==*(obj1_addr) ) return true;
+        const InetAddr *addr = addr_obj->getAddressPtr();
+        const InetAddr *netm = addr_obj->getNetmaskPtr();
+        if (addr)
+        {
+            InetAddrMask n(*addr, *netm);
+/*
+ * bug #1040773: need to match network address as well as
+ * broadcast. Packets sent to the network address (192.168.1.0 for net
+ * 192.168.1.0/24) go in the broadcast frame and behave just like IP
+ * broadcast packets (sent to 192.168.1.1255 for the same net)
+ *
+ */
+            if (recognize_broadcasts && (
+                    *(n.getNetworkAddressPtr())==*(obj1_addr) ||
+                    *(n.getBroadcastAddressPtr())==*(obj1_addr)
+                )) return true;
+        }
+    }
+    return false;
+}
 
 bool Compiler::_complexMatchWithInterface(Address   *obj1,
                                           Interface *iface,
                                           bool recognize_broadcasts)
 {
-    IPAddress obj1_addr=obj1->getAddress();
+    const InetAddr *obj1_addr = obj1->getAddressPtr();
 
     if ( physAddress::isA(obj1))
     {
@@ -505,28 +629,9 @@ bool Compiler::_complexMatchWithInterface(Address   *obj1,
 
     if ( iface->isRegular() )
     {
-        FWObjectTypedChildIterator k=iface->findByType(IPv4::TYPENAME);
-        for ( ; k!=k.end(); ++k ) 
-        {
-            IPv4 *ipv4=IPv4::cast(*k);
-                    
-            if ( ipv4->getAddress()==obj1_addr ) return true;
-
-            IPNetwork n( ipv4->getAddress() , ipv4->getNetmask() );
-/*
- * bug #1040773: need to match network address as well as
- * broadcast. Packets sent to the network address (192.168.1.0 for net
- * 192.168.1.0/24) go in the broadcast frame and behave just like IP
- * broadcast packets (sent to 192.168.1.1255 for the same net)
- *
- */
-            if (recognize_broadcasts)
-            {
-/* obj1 is an individual address - IPv4 or Address object */
-                if (n.getAddress()==obj1_addr || n.getBroadcastAddress()==obj1_addr)
-                    return true;
-            }
-        }
+        string address_type = (ipv6) ? IPv6::TYPENAME : IPv4::TYPENAME;
+        return _complexMatchWithAddress(
+            obj1_addr, iface, address_type, recognize_broadcasts);
     }
     return false;
 }
@@ -542,28 +647,35 @@ bool Compiler::_complexMatchWithInterface(Address   *obj1,
  */
 bool Compiler::complexMatch(Address *obj1,
                             Address *obj2,
-                          bool recognize_broadcasts,
-                          bool recognize_multicasts)
+                            bool recognize_broadcasts,
+                            bool recognize_multicasts)
 {
-    if (Network::isA(obj1))
+    if (Network::isA(obj1) || NetworkIPv6::isA(obj1))
     {
         /*
          * bug #1055937: "Any->all_multicasts not in INPUT Chain"
          * Need to check for multicast networks. We assume they always
          * match if obj2 is firewall
          */
-        Network *nobj1=Network::cast(obj1);
-        if (nobj1->getAddress().isMulticast() && Firewall::isA(obj2))
-            return true;
-
-        /*
-         * need to check for network object with mask 255.255.255.255
-         * Such objects are created by the method that expands address
-         * ranges, and some often used ranges trigger that (like
-         * "255.255.255.255-255.255.255.255" or "0.0.0.0-0.0.0.0")
-         */
-        if (nobj1->getNetmask().toString()!="255.255.255.255")
-            return false;
+        //Network *nobj1 = Network::cast(obj1);
+        const InetAddr *inet_addr = obj1->getAddressPtr();
+        if (inet_addr)
+        {
+            if (inet_addr->isMulticast() && Firewall::isA(obj2))
+                return true;
+            /*
+             * need to check for network object with mask 255.255.255.255
+             * Such objects are created by the method that expands address
+             * ranges, and some often used ranges trigger that (like
+             * "255.255.255.255-255.255.255.255" or "0.0.0.0-0.0.0.0")
+             */
+            if (!obj1->getNetmaskPtr()->isHostMask())
+                return false;
+        } else
+            warning(string("Network object with no InetAddr: id=") +
+                    FWObjectDatabase::getStringId(obj1->getId()) +
+                    "  type=" + obj1->getTypeName() +
+                    "  name=" + obj1->getName());
     }
 /*
  * if we are matching two firewalls, the positive match is only if
@@ -594,29 +706,38 @@ bool Compiler::complexMatch(Address *obj1,
     while ( (p=p->getParent())!=NULL )  // it is faster to search this way
         if (p->getId()==obj2->getId()) return true;
 
+    // TODO: this can be expensive since we create and return temporary
+    // list object only to get its size. Need method FWObject::countByType
     if ((obj1->getByType(IPv4::TYPENAME)).size()>1) return false;
+    if ((obj1->getByType(IPv6::TYPENAME)).size()>1) return false;
 
-    IPAddress obj1_addr=obj1->getAddress();
-    if (obj1_addr!=IPAddress("0.0.0.0") && 
-        ( (recognize_broadcasts && obj1_addr.isBroadcast()) || 
-          (recognize_multicasts && obj1_addr.isMulticast()) )
-    ) return true;
-
-    if (Interface::cast(obj1)!=NULL && !Interface::cast(obj1)->isRegular())
-        return false;
-
-    if (Interface::cast(obj2)!=NULL)
+    const InetAddr *obj1_addr = obj1->getAddressPtr();
+    // obj1_addr may be NULL if obj1 does not have any real address,
+    // one case when this happens is when obj1 is physAddress
+    if (obj1_addr)
     {
-        return _complexMatchWithInterface(obj1,Interface::cast(obj2));
-    }else
-    {
-        FWObjectTypedChildIterator j=obj2->findByType(Interface::TYPENAME);
-        for ( ; j!=j.end(); ++j ) 
+        if (!obj1_addr->isAny() && 
+            ( (recognize_broadcasts && obj1_addr->isBroadcast()) || 
+              (recognize_multicasts && obj1_addr->isMulticast()) )
+        ) return true;
+
+        if (Interface::cast(obj1)!=NULL && !Interface::cast(obj1)->isRegular())
+            return false;
+
+        if (Interface::cast(obj2)!=NULL)
         {
-            Interface *iface=Interface::cast(*j);
-            if (_complexMatchWithInterface(obj1,iface)) return true;
+            return _complexMatchWithInterface(obj1,Interface::cast(obj2));
+        }else
+        {
+            FWObjectTypedChildIterator j=obj2->findByType(Interface::TYPENAME);
+            for ( ; j!=j.end(); ++j ) 
+            {
+                Interface *iface=Interface::cast(*j);
+                if (_complexMatchWithInterface(obj1,iface)) return true;
+            }
         }
     }
+
     return false;
 }
 
@@ -635,33 +756,34 @@ Interface* Compiler::findInterfaceFor(const Address *obj1, const Address *obj2)
 
         if (iface->getId()      == obj1->getId() )      return iface;
 
-        if ( iface->isRegular() )
+        if ( iface->isRegular() && obj1->getAddressPtr() != NULL)
         {
-            FWObjectTypedChildIterator k=iface->findByType(IPv4::TYPENAME);
-            for ( ; k!=k.end(); ++k ) 
+            if (obj1->getAddressPtr()->isV4())
             {
-                IPv4 *addr=IPv4::cast(*k);
-                assert(addr);
-
-                if (addr->getId() == obj1->getId() ) return iface;
-                if (addr->getAddress() == obj1->getAddress() ) return iface;
-
-                if (Network::constcast(obj1)!=NULL) 
+                FWObjectTypedChildIterator k= iface->findByType(IPv4::TYPENAME);
+                for ( ; k!=k.end(); ++k ) 
                 {
-                    IPNetwork n1( obj1->getAddress() , Network::constcast(obj1)->getNetmask() );
-                    if (n1.belongs( addr->getAddress() ) ) return iface; 
+                    Address *addr = Address::cast(*k);
+                    assert(addr);
+                    if (checkIfAddressesMatch(addr, obj1)) return iface;
+                }
+            }
+            if (obj1->getAddressPtr()->isV6())
+            {
+                FWObjectTypedChildIterator k= iface->findByType(IPv6::TYPENAME);
+                for ( ; k!=k.end(); ++k ) 
+                {
+                    Address *addr = Address::cast(*k);
+                    assert(addr);
+                    if (checkIfAddressesMatch(addr, obj1)) return iface;
                 }
-
-/* n2 is the network interface is sitting on */
-                IPNetwork n2( addr->getAddress() , addr->getNetmask() );
-                if ( n2.belongs( obj1->getAddress() ) )      return iface;
             }
         }
     }
     return NULL;
 }
 
-Address* Compiler::findAddressFor(const Address *obj1,const Address *obj2)
+FWObject* Compiler::findAddressFor(const Address *obj1,const Address *obj2)
 {
     FWObjectTypedChildIterator j=obj2->findByType(Interface::TYPENAME);
     for ( ; j!=j.end(); ++j ) 
@@ -669,34 +791,49 @@ Address* Compiler::findAddressFor(const Address *obj1,const Address *obj2)
 	Interface *iface=Interface::cast(*j);
 	assert(iface);
 
-        if (iface->getId()      == obj1->getId() )      return iface;
+        if (iface->getId() == obj1->getId() )      return iface;
 
         if ( iface->isRegular() )
         {
-            FWObjectTypedChildIterator k=iface->findByType(IPv4::TYPENAME);
-            for ( ; k!=k.end(); ++k ) 
+            if (obj1->getAddressPtr()->isV4())
             {
-                IPv4 *addr=IPv4::cast(*k);
-                assert(addr);
-
-                if (addr->getId() == obj1->getId() ) return addr;
-                if (addr->getAddress() == obj1->getAddress() ) return addr;
-
-                if (Network::constcast(obj1)!=NULL) 
+                FWObjectTypedChildIterator k= iface->findByType(IPv4::TYPENAME);
+                for ( ; k!=k.end(); ++k ) 
                 {
-                    IPNetwork n1( obj1->getAddress() , Network::constcast(obj1)->getNetmask() );
-                    if (n1.belongs( addr->getAddress() ) ) return addr; 
+                    Address *addr = Address::cast(*k);
+                    assert(addr);
+                    if (checkIfAddressesMatch(addr, obj1)) return (*k);
+                }
+            }
+            if (obj1->getAddressPtr()->isV6())
+            {
+                FWObjectTypedChildIterator k= iface->findByType(IPv6::TYPENAME);
+                for ( ; k!=k.end(); ++k ) 
+                {
+                    Address *addr = Address::cast(*k);
+                    assert(addr);
+                    if (checkIfAddressesMatch(addr, obj1)) return (*k);
                 }
-
-/* n2 is the network interface is sitting on */
-                IPNetwork n2( addr->getAddress() , addr->getNetmask() );
-                if ( n2.belongs( obj1->getAddress() ) )      return addr;
             }
         }
     }
     return NULL;
 }
         
+bool Compiler::checkIfAddressesMatch(const Address *a1, const Address *a2)
+{
+    if (a1->getId() == a2->getId()) return true;
+    if (*(a1->getAddressPtr()) == *(a2->getAddressPtr()) ) return true;
+    if ((Network::constcast(a2)!=NULL ||
+         NetworkIPv6::constcast(a2)!=NULL ||
+         Interface::constcast(a2->getParent())) &&
+        a2->belongs(*(a1->getAddressPtr()))) return true;
+    if ((Network::constcast(a1)!=NULL ||
+         NetworkIPv6::constcast(a1)!=NULL ||
+         Interface::constcast(a1->getParent())) &&
+        a1->belongs(*(a2->getAddressPtr()))) return true;
+    return false;
+}
 
 
 void Compiler::debugRule()
@@ -746,7 +883,7 @@ void Compiler::runRuleProcessors()
         j=i;
     }
 
-    while( (*j)->processNext() );
+    while ((*j)->processNext()) ;
 }
 
 void Compiler::deleteRuleProcessors()
@@ -893,7 +1030,7 @@ bool Compiler::splitIfRuleElementMatchesFW::processNext()
         Address *a=Address::cast(obj);
         assert(a!=NULL);
 
-//        IPAddress obj_addr=a->getAddress();
+//        InetAddr obj_addr=a->getAddress();
 
         if (compiler->complexMatch(a,compiler->fw))
         {
@@ -968,7 +1105,7 @@ bool Compiler::eliminateDuplicatesInRE::processNext()
     return true;
 }
 
-void  Compiler::recursiveGroupsInRE::isRecursiveGroup(const string &grid,FWObject *obj)
+void  Compiler::recursiveGroupsInRE::isRecursiveGroup(int grid, FWObject *obj)
 {
     for (FWObject::iterator i=obj->begin(); i!=obj->end(); i++) 
     {
@@ -988,8 +1125,8 @@ void  Compiler::recursiveGroupsInRE::isRecursiveGroup(const string &grid,FWObjec
 
 bool Compiler::recursiveGroupsInRE::processNext()
 {
-    Rule        *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
-    RuleElement *re  =RuleElement::cast(rule->getFirstByType(re_type));
+    Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
+    RuleElement *re = RuleElement::cast(rule->getFirstByType(re_type));
 
     if (re->isAny())
     {
@@ -1022,7 +1159,6 @@ int  Compiler::emptyGroupsInRE::countChildren(FWObject *obj)
     {
 	FWObject *o= *i;
 	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-        
         // Check if this is a group, if yes, then count its children
         // recursively. Group itself does not count since it can be
         // empty, too.  However if this is MultiAddress object with
@@ -1034,16 +1170,18 @@ int  Compiler::emptyGroupsInRE::countChildren(FWObject *obj)
         if (MultiAddress::cast(o)!=NULL && MultiAddress::cast(o)->isRunTime())
             n++;
         else
+        {
             if (Group::cast(o)!=NULL)  n += countChildren(o);
             else n++;   // but if it is not a group, then we count it.
+        }
     }
     return n;
 }
 
 bool Compiler::emptyGroupsInRE::processNext()
 {
-    Rule        *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
-    RuleElement *re  =RuleElement::cast(rule->getFirstByType(re_type));
+    Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
+    RuleElement *re = RuleElement::cast(rule->getFirstByType(re_type));
 
     if (re->isAny())
     {
@@ -1149,9 +1287,17 @@ bool Compiler::swapMultiAddressObjectsInRE::processNext()
         {
             MultiAddress *ma = *i;
 
-            string mart_id = ma->getId()+"_runtime";
-            MultiAddressRunTime *mart = 
-                MultiAddressRunTime::cast(compiler->dbcopy->findInIndex(mart_id));
+            // Need to make sure the ID of the MultiAddressRunTime
+            // object created here is stable and is always the same
+            // for the same MultiAddress object. In particular this
+            // ensures that we reuse tables between policy and NAT rules
+            // in compiler for PF. There could be other similar cases
+            // (object-group in compielr for pix may be ?)
+            string mart_id_str = FWObjectDatabase::getStringId(ma->getId()) +
+                "_runtime";
+            int mart_id = FWObjectDatabase::registerStringId(mart_id_str);
+            MultiAddressRunTime *mart = MultiAddressRunTime::cast(
+                compiler->dbcopy->findInIndex(mart_id));
             if (mart==NULL)
             {
                 mart = new MultiAddressRunTime(ma);
@@ -1165,6 +1311,7 @@ bool Compiler::swapMultiAddressObjectsInRE::processNext()
                 mart->setId( mart_id );
                 compiler->dbcopy->addToIndex(mart);
                 compiler->dbcopy->add(mart);
+                compiler->cacheObj(mart);
             }
             re->removeRef(ma);
             re->addRef(mart);
@@ -1177,6 +1324,90 @@ bool Compiler::swapMultiAddressObjectsInRE::processNext()
     return true;
 }
 
+bool Compiler::FindAddressFamilyInRE(FWObject *parent, bool ipv6)
+{
+    Address *addr = Address::cast(parent);
+    if (addr!=NULL)
+    {
+        const  InetAddr *inet_addr = addr->getAddressPtr();
+        if (ipv6)
+            return (inet_addr && inet_addr->isV6());
+        else
+            return (inet_addr && inet_addr->isV4());
+    }
+
+    for (FWObject::iterator i=parent->begin(); i!=parent->end(); i++) 
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        if (FindAddressFamilyInRE(o, ipv6)) return true;
+    }
+    return false;
+}
+
+
+void Compiler::DropAddressFamilyInRE(RuleElement *rel, bool drop_ipv6)
+{
+    list<FWObject*> objects_to_remove;
+    for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        // skip "Any"
+        if (o->getId() == FWObjectDatabase::ANY_ADDRESS_ID)
+            continue;
+
+        if (drop_ipv6)
+        {
+            if (Address::cast(o) && Address::cast(o)->hasInetAddress())
+            {
+                const  InetAddr *inet_addr = Address::cast(o)->getAddressPtr();
+                if (inet_addr && inet_addr->isV6())
+                    objects_to_remove.push_back(o);
+            }
+        } else
+        {
+            if (Address::cast(o) && Address::cast(o)->hasInetAddress())
+            {
+                const  InetAddr *inet_addr = Address::cast(o)->getAddressPtr();
+                if (inet_addr && inet_addr->isV4())
+                    objects_to_remove.push_back(o);
+            }
+        }
+    }
+
+    for (list<FWObject*>::iterator i = objects_to_remove.begin();
+         i != objects_to_remove.end(); ++i)
+        rel->removeRef(*i);
+}
+
+void Compiler::DropByServiceTypeInRE(RuleElement *rel, bool drop_ipv6)
+{
+    list<FWObject*> objects_to_remove;
+    for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        // skip "Any"
+        if (o->getId() == FWObjectDatabase::ANY_SERVICE_ID)
+            continue;
+
+        // Note that all service objects except for ICMPService can be
+        // used in both ipv4 and ipv6 contexts.
+        if (drop_ipv6)
+        {
+            if (ICMP6Service::isA(o)) objects_to_remove.push_back(o);
+        } else
+        {
+            if (ICMPService::isA(o)) objects_to_remove.push_back(o);
+        }
+    }
+
+    for (list<FWObject*>::iterator i = objects_to_remove.begin();
+         i != objects_to_remove.end(); ++i)
+        rel->removeRef(*i);
+}
+
 
 bool Compiler::catchUnnumberedIfaceInRE(RuleElement *re)
 {
@@ -1189,7 +1420,12 @@ bool Compiler::catchUnnumberedIfaceInRE(RuleElement *re)
         if (o==NULL)
         {
             Rule *rule = Rule::cast(re->getParent());
-            string errmsg = "catchUnnumberedIfaceInRE: Can't find object in cache, ID=" + (*i)->getStr("ref") + "  rule " + rule->getLabel();
+            FWReference *refo = FWReference::cast(*i);
+            string errmsg =
+                string("catchUnnumberedIfaceInRE: Can't find object ") +
+                string("in cache, ID=") +
+                FWObjectDatabase::getStringId(refo->getPointerId()) +
+                "  rule " + rule->getLabel();
             abort(errmsg);
         }
         err |= ((iface=Interface::cast(o))!=NULL &&
@@ -1200,7 +1436,7 @@ bool Compiler::catchUnnumberedIfaceInRE(RuleElement *re)
 }
 
 
-Address* Compiler::getFirstSrc(const PolicyRule *rule)
+Address* Compiler::getFirstSrc(PolicyRule *rule)
 {
     RuleElementSrc *src=rule->getSrc();
 
@@ -1211,7 +1447,7 @@ Address* Compiler::getFirstSrc(const PolicyRule *rule)
     return Address::cast(o);
 }
 
-Address* Compiler::getFirstDst(const PolicyRule *rule)
+Address* Compiler::getFirstDst(PolicyRule *rule)
 {
     RuleElementDst *dst=rule->getDst();
 
@@ -1222,7 +1458,7 @@ Address* Compiler::getFirstDst(const PolicyRule *rule)
     return Address::cast(o);
 }
 
-Service* Compiler::getFirstSrv(const PolicyRule *rule)
+Service* Compiler::getFirstSrv(PolicyRule *rule)
 {
     RuleElementSrv *srv=rule->getSrv();
 
@@ -1233,7 +1469,7 @@ Service* Compiler::getFirstSrv(const PolicyRule *rule)
     return Service::cast(o);
 }
 
-Interval* Compiler::getFirstWhen(const PolicyRule *rule)
+Interval* Compiler::getFirstWhen(PolicyRule *rule)
 {
     RuleElementInterval *when=rule->getWhen();
     if (when==NULL) return NULL;  // when is optional element
@@ -1245,7 +1481,7 @@ Interval* Compiler::getFirstWhen(const PolicyRule *rule)
     return Interval::cast(o);
 }
 
-Interface* Compiler::getFirstItf(const PolicyRule *rule)
+Interface* Compiler::getFirstItf(PolicyRule *rule)
 {
     RuleElementItf *itf=rule->getItf();
     if (itf==NULL) return NULL;  // itf is optional element
@@ -1257,7 +1493,7 @@ Interface* Compiler::getFirstItf(const PolicyRule *rule)
     return Interface::cast(o);
 }
 
-Address* Compiler::getFirstOSrc(const NATRule *rule)
+Address* Compiler::getFirstOSrc(NATRule *rule)
 {
     RuleElementOSrc *osrc=rule->getOSrc();
 
@@ -1268,7 +1504,7 @@ Address* Compiler::getFirstOSrc(const NATRule *rule)
     return Address::cast(o);
 }
 
-Address* Compiler::getFirstODst(const NATRule *rule)
+Address* Compiler::getFirstODst(NATRule *rule)
 {
     RuleElementODst *odst=rule->getODst();
 
@@ -1279,7 +1515,7 @@ Address* Compiler::getFirstODst(const NATRule *rule)
     return Address::cast(o);
 }
 
-Service* Compiler::getFirstOSrv(const NATRule *rule)
+Service* Compiler::getFirstOSrv(NATRule *rule)
 {
     RuleElementOSrv *osrv=rule->getOSrv();
 
@@ -1292,7 +1528,7 @@ Service* Compiler::getFirstOSrv(const NATRule *rule)
 
 
 
-Address* Compiler::getFirstTSrc(const NATRule *rule)
+Address* Compiler::getFirstTSrc(NATRule *rule)
 {
     RuleElementTSrc *tsrc=rule->getTSrc();
 
@@ -1303,7 +1539,7 @@ Address* Compiler::getFirstTSrc(const NATRule *rule)
     return Address::cast(o);
 }
 
-Address* Compiler::getFirstTDst(const NATRule *rule)
+Address* Compiler::getFirstTDst(NATRule *rule)
 {
     RuleElementTDst *tdst=rule->getTDst();
 
@@ -1314,7 +1550,7 @@ Address* Compiler::getFirstTDst(const NATRule *rule)
     return Address::cast(o);
 }
 
-Service* Compiler::getFirstTSrv(const NATRule *rule)
+Service* Compiler::getFirstTSrv(NATRule *rule)
 {
     RuleElementTSrv *tsrv=rule->getTSrv();
 
