@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Compiler.h,v 1.14 2006/08/18 03:59:32 vkurland Exp $
+  $Id: Compiler.h 175 2008-08-19 04:15:54Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -42,7 +42,7 @@ namespace libfwbuilder {
     class FWObject;
     class FWOptions;
     class FWObjectDatabase;
-    class IPAddress;
+    class InetAddr;
     class Address;
     class Service;
     class Interval;
@@ -135,13 +135,29 @@ namespace fwcompiler {
                                         libfwbuilder::Interface *iface,
                                         bool recognize_broadcasts=true);
 
+        bool _complexMatchWithAddress(const libfwbuilder::InetAddr *obj1_addr,
+                                      libfwbuilder::Interface *iface,
+                                      const std::string &address_type,
+                                      bool recognize_broadcasts);
+
+        bool checkIfAddressesMatch(const libfwbuilder::Address *a1,
+                                   const libfwbuilder::Address *a2);
+
 	protected:
 
         int  _cntr_;
         bool initialized;
+        int countIPv6Rules;
+        bool ipv6;
 
         std::list<BasicRuleProcessor*> rule_processors;
 
+        /**
+         * if object <o> is Address, check if it matches address family
+         * (i.e. if it is IPv6 or IPv4). If it is service, always return true.
+         */
+        bool MatchesAddressFamily(libfwbuilder::FWObject *o);
+        
         /**
          * this method finds intersection of two atomic rules. Resulting
          * rule may have multiple objects in src,dst and srv, so
@@ -158,8 +174,8 @@ namespace fwcompiler {
          * completely ignores action and other rule options.
          *
          */
-        void  getIntersection(const libfwbuilder::PolicyRule &r1,
-                              const libfwbuilder::PolicyRule &r2,
+        void  getIntersection(libfwbuilder::PolicyRule &r1,
+                              libfwbuilder::PolicyRule &r2,
                               libfwbuilder::PolicyRule &res);
 
         /**
@@ -168,8 +184,8 @@ namespace fwcompiler {
          * function does not calculate intersection, it just verifies that
          * it does exsit.
          */
-        bool  intersect(const libfwbuilder::PolicyRule &r1,
-                        const libfwbuilder::PolicyRule &r2);
+        bool  intersect(libfwbuilder::PolicyRule &r1,
+                        libfwbuilder::PolicyRule &r2);
 
 
         /**
@@ -191,10 +207,10 @@ namespace fwcompiler {
         * the following variables are simply a cache for frequently used
         * objects
         */
-        std::map<const std::string,libfwbuilder::Interface*> fw_interfaces;
-        std::string                                          fw_id;
-        libfwbuilder::FWOptions                             *fwopt;
-        std::map<const std::string,libfwbuilder::FWObject*>  objcache;
+        std::map<int, libfwbuilder::Interface*> fw_interfaces;
+        int                                     fw_id;
+        libfwbuilder::FWOptions                *fwopt;
+        std::map<int, libfwbuilder::FWObject*>  objcache;
 
         /**
          * stores object o and all its children in the cache, recursively
@@ -204,25 +220,49 @@ namespace fwcompiler {
 
 	public:
 
+	int                                debug;
+	int                                debug_rule;
+	bool                               verbose;
+
+	fwcompiler::OSConfigurator        *osconfigurator;
+	libfwbuilder::FWObjectDatabase    *dbcopy;
+	libfwbuilder::Firewall            *fw;
+
+        std::string                        ruleSetName;;
+        
+	libfwbuilder::RuleSet             *source_ruleset;
+	libfwbuilder::RuleSet             *combined_ruleset;
+	libfwbuilder::RuleSet             *temp_ruleset;
+
+        libfwbuilder::Group               *temp;
+
+	std::stringstream                  output;
+	std::stringstream                  errors_buffer;
+
+        bool                               test_mode;
+
+        void registerIPv6Rule() { countIPv6Rules++; }
+        bool haveIPv6Rules() { return countIPv6Rules > 0; }
+
         /**
          * returns first object referenced by given rule
          * element. Dereferences FWReference if first object is
          * reference. Uses cache, therefore is faster than
          * RuleElement::getFirst(true)
          */
-        libfwbuilder::Address*   getFirstSrc(const libfwbuilder::PolicyRule *rule);
-        libfwbuilder::Address*   getFirstDst(const libfwbuilder::PolicyRule *rule);
-        libfwbuilder::Service*   getFirstSrv(const libfwbuilder::PolicyRule *rule);
-        libfwbuilder::Interval*  getFirstWhen(const libfwbuilder::PolicyRule *rule);
-        libfwbuilder::Interface* getFirstItf(const libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Address*   getFirstSrc(libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Address*   getFirstDst(libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Service*   getFirstSrv(libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Interval*  getFirstWhen(libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Interface* getFirstItf(libfwbuilder::PolicyRule *rule);
         
-        libfwbuilder::Address* getFirstOSrc(const libfwbuilder::NATRule *rule);
-        libfwbuilder::Address* getFirstODst(const libfwbuilder::NATRule *rule);
-        libfwbuilder::Service* getFirstOSrv(const libfwbuilder::NATRule *rule);
+        libfwbuilder::Address* getFirstOSrc(libfwbuilder::NATRule *rule);
+        libfwbuilder::Address* getFirstODst(libfwbuilder::NATRule *rule);
+        libfwbuilder::Service* getFirstOSrv(libfwbuilder::NATRule *rule);
                                                          
-        libfwbuilder::Address* getFirstTSrc(const libfwbuilder::NATRule *rule);
-        libfwbuilder::Address* getFirstTDst(const libfwbuilder::NATRule *rule);
-        libfwbuilder::Service* getFirstTSrv(const libfwbuilder::NATRule *rule);
+        libfwbuilder::Address* getFirstTSrc(libfwbuilder::NATRule *rule);
+        libfwbuilder::Address* getFirstTDst(libfwbuilder::NATRule *rule);
+        libfwbuilder::Service* getFirstTSrv(libfwbuilder::NATRule *rule);
 
 
 	/**
@@ -231,7 +271,26 @@ namespace fwcompiler {
 	 */
         bool catchUnnumberedIfaceInRE(libfwbuilder::RuleElement *re);
 
+        /**
+         * return true if any address object in source or destination is
+         * of given type (can be IPv4 or IPv6).
+         */
+        bool FindAddressFamilyInRE(libfwbuilder::FWObject *re, bool ipv6);
 
+        /**
+         * find ipv6 or ipv4 address objects in the given rule element
+         * and remove reference to them
+         */
+        void DropAddressFamilyInRE(libfwbuilder::RuleElement *rel,
+                                   bool drop_ipv6);
+
+        /**
+         * similarly, remove reference to objects in service rule elements
+         * based on their compatibility with ipv6 context.
+         */
+        void DropByServiceTypeInRE(libfwbuilder::RuleElement *rel,
+                                   bool drop_ipv6);
+        
         /**
          *  rule processor that "injects" rules into the conveyor
          */
@@ -339,7 +398,7 @@ namespace fwcompiler {
         class recursiveGroupsInRE : public BasicRuleProcessor
         {
             std::string re_type;
-            void isRecursiveGroup(const std::string &grid,libfwbuilder::FWObject *gr);
+            void isRecursiveGroup(int grid, libfwbuilder::FWObject *gr);
             public:
             recursiveGroupsInRE(const std::string &n,const std::string &_type) : 
                 BasicRuleProcessor(n) { re_type=_type; }
@@ -412,13 +471,13 @@ namespace fwcompiler {
         /**
          * returns pointer to cached interface
          */
-        libfwbuilder::Interface* getCachedFwInterface(const std::string &id)
+        libfwbuilder::Interface* getCachedFwInterface(int id)
         { return fw_interfaces[id]; }
 
         /**
          * returns cached firewall object ID
          */
-        std::string getFwId() { return fw_id; }
+        int getFwId() { return fw_id; }
 
         /**
          * returns pointer to the cached firewall options object
@@ -433,7 +492,7 @@ namespace fwcompiler {
         /**
          * does cache lookup for object with given ID
          */
-        libfwbuilder::FWObject* getCachedObj(const std::string &id)
+        libfwbuilder::FWObject* getCachedObj(int id)
         {
             return objcache[id];
         }
@@ -507,8 +566,8 @@ namespace fwcompiler {
          * If obj1 is an Interface object, then corresponding Interface
          * object belonging to the firewall is returned (if found).
          */
-        libfwbuilder::Address* findAddressFor(const libfwbuilder::Address *obj1,
-                                              const libfwbuilder::Address *obj2);
+        libfwbuilder::FWObject* findAddressFor(const libfwbuilder::Address *o1,
+                                               const libfwbuilder::Address *o2);
         
 
         /**
@@ -528,51 +587,34 @@ namespace fwcompiler {
          */
 	void warning(const std::string &warnstr);
 
-	int                                debug;
-	int                                debug_rule;
-	bool                               verbose;
-
-	fwcompiler::OSConfigurator        *osconfigurator;
-	libfwbuilder::FWObjectDatabase    *dbcopy;
-	libfwbuilder::Firewall            *fw;
-
-        std::string                        ruleSetName;;
-        
-	libfwbuilder::RuleSet             *source_ruleset;
-	libfwbuilder::RuleSet             *combined_ruleset;
-	libfwbuilder::RuleSet             *temp_ruleset;
-
-        libfwbuilder::Group               *temp;
-
-	std::stringstream                  output;
-
-        bool                               test_mode;
-
 	virtual ~Compiler();
 
 	Compiler(libfwbuilder::FWObjectDatabase *_db,
-		 const std::string &fwname);
+		 const std::string &fwname, bool ipv6_policy);
 
 	Compiler(libfwbuilder::FWObjectDatabase *_db,
-		 const std::string &fwname,
+		 const std::string &fwname, bool ipv6_policy,
 		 fwcompiler::OSConfigurator *_oscnf);
 
-	Compiler(libfwbuilder::FWObjectDatabase *_db);
+	Compiler(libfwbuilder::FWObjectDatabase *_db, bool ipv6_policy);
         
 	void setDebugLevel(int dl) { debug=dl;       }
 	void setDebugRule(int dr)  { debug_rule=dr;  }
 	void setVerbose(bool v)    { verbose=v;      }
         void setTestMode()         { test_mode=true; }
         void setSourceRuleSet(libfwbuilder::RuleSet *rs) { source_ruleset = rs; }
+        libfwbuilder::RuleSet* getSourceRuleSet() { return source_ruleset; }
+
         void setRuleSetName(const std::string &name) { ruleSetName = name; }
         std::string getRuleSetName() { return ruleSetName; }
         
 	std::string getCompiledScript();
-        int         getCompiledScriptLength();
+        int getCompiledScriptLength();
+	std::string getErrors(const std::string &comment_sep);
+	bool haveErrorsAndWarnings();
 
 	void expandGroupsInRuleElement(libfwbuilder::RuleElement *s);
 
-
         /**
 	 * this method should return platform name. It is used 
 	 * to construct proper error and warning messages.
@@ -581,14 +623,8 @@ namespace fwcompiler {
 
 	std::string getUniqueRuleLabel();
 
-	virtual std::string createRuleLabel(const std::string &txt,
-                                            libfwbuilder::Interface *iface,
-                                            int rule_num);
-
-	virtual std::string createRuleLabel(const std::string &txt,
-                                            int rule_num);
-
-	virtual std::string createRuleLabel(libfwbuilder::Interface *iface,
+	virtual std::string createRuleLabel(const std::string &prefix,
+                                            const std::string &txt,
                                             int rule_num);
 
 	/**
