@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Compiler_ops.cpp,v 1.7 2006/03/06 03:00:50 vkurland Exp $
+  $Id: Compiler_ops.cpp 125 2008-07-06 00:58:05Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -35,12 +35,18 @@
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/IPService.h"
 #include "fwbuilder/ICMPService.h"
+#include "fwbuilder/TCPUDPService.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
 #include "fwbuilder/CustomService.h"
+#include "fwbuilder/TagService.h"
+#include "fwbuilder/UserService.h"
+
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Rule.h"
 
+#include <iostream>
+
 using namespace fwcompiler;
 using namespace libfwbuilder;
 using namespace std;
@@ -54,28 +60,37 @@ using namespace std;
  */
 vector<FWObject*> fwcompiler::_find_obj_intersection(Address *op1, Address *op2)
 {
-    const IPNetwork n1( op1->getAddress() , 
-       	(Interface::cast(op1))?Netmask("255.255.255.255"):op1->getNetmask() );
-    const IPNetwork n2( op2->getAddress() , 
-	(Interface::cast(op2))?Netmask("255.255.255.255"):op2->getNetmask() );
-
-    vector<IPNetwork> intersection= libfwbuilder::getOverlap(n1,n2);
-
     vector<FWObject*>  res;
 
-    for (vector<IPNetwork>::iterator i=intersection.begin(); i!=intersection.end(); i++) {
-	IPNetwork *n= &(*i);
-	if (n->getNetmask()==Netmask("255.255.255.255")) {
-            IPv4 *h=new IPv4();
-            h->setAddress(n->getAddress());
-	    h->setName("h-"+n->getAddress().toString());
+    const InetAddr *addr1 = op1->getAddressPtr();
+    const InetAddr *netm1 = op1->getNetmaskPtr();
+    const InetAddr *addr2 = op2->getAddressPtr();
+    const InetAddr *netm2 = op2->getNetmaskPtr();
+
+    if (addr1==NULL || addr2==NULL) return res;
+
+    const InetAddrMask n1( *addr1,
+        (Interface::cast(op1)) ? InetAddr(InetAddr::getAllOnes()) : (*netm1) );
+    const InetAddrMask n2( *addr2,
+        (Interface::cast(op2)) ? InetAddr(InetAddr::getAllOnes()) : (*netm2) );
+
+    vector<InetAddrMask> intersection = libfwbuilder::getOverlap(n1,n2);
+
+    for (vector<InetAddrMask>::iterator i=intersection.begin(); i!=intersection.end(); i++)
+    {
+	InetAddrMask *n= &(*i);
+	if (n->getNetmaskPtr()->isHostMask())
+        {
+            IPv4 *h = new IPv4();
+            h->setAddress(*(n->getAddressPtr()));
+	    h->setName("h-"+n->getAddressPtr()->toString());
             op1->getRoot()->add(h,false);
 	    res.push_back(h);
 	} else {
-	    Network *net=new Network();
-	    net->setAddress(n->getAddress());
-	    net->setNetmask(n->getNetmask());
-	    net->setName("n-"+n->getAddress().toString());
+	    Network *net = new Network();
+	    net->setAddress(*(n->getAddressPtr()));
+	    net->setNetmask(*(n->getNetmaskPtr()));
+	    net->setName("n-"+n->getAddressPtr()->toString());
             op1->getRoot()->add(net,false);
 	    res.push_back(net);
 	}
@@ -125,15 +140,15 @@ vector<FWObject*> fwcompiler::_find_srv_intersection(Service *op1, Service *op2)
 	}
 	if ( TCPService::cast(op1) || UDPService::cast(op1) ) 
         {
-	    int srs1=op1->getInt("src_range_start");
-	    int sre1=op1->getInt("src_range_end");
-	    int drs1=op1->getInt("dst_range_start");
-	    int dre1=op1->getInt("dst_range_end");
+	    int srs1=static_cast<const TCPUDPService*>(op1)->getSrcRangeStart();
+	    int sre1=static_cast<const TCPUDPService*>(op1)->getSrcRangeEnd();
+	    int drs1=static_cast<const TCPUDPService*>(op1)->getDstRangeStart();
+	    int dre1=static_cast<const TCPUDPService*>(op1)->getDstRangeEnd();
 
-	    int srs2=op2->getInt("src_range_start");
-	    int sre2=op2->getInt("src_range_end");
-	    int drs2=op2->getInt("dst_range_start");
-	    int dre2=op2->getInt("dst_range_end");
+	    int srs2=static_cast<const TCPUDPService*>(op2)->getSrcRangeStart();
+	    int sre2=static_cast<const TCPUDPService*>(op2)->getSrcRangeEnd();
+	    int drs2=static_cast<const TCPUDPService*>(op2)->getDstRangeStart();
+	    int dre2=static_cast<const TCPUDPService*>(op2)->getDstRangeEnd();
 
 	    int srsR,sreR,drsR,dreR;
 
@@ -153,10 +168,10 @@ vector<FWObject*> fwcompiler::_find_srv_intersection(Service *op1, Service *op2)
 	    }
 
 	    nserv->setName(op1->getName()+"-"+op2->getName());
-	    nserv->setInt("src_range_start",srsR);
-	    nserv->setInt("src_range_end"  ,sreR);
-	    nserv->setInt("dst_range_start",drsR);
-	    nserv->setInt("dst_range_end"  ,dreR);
+            TCPUDPService::cast(nserv)->setSrcRangeStart(srsR);
+            TCPUDPService::cast(nserv)->setSrcRangeEnd(sreR);
+            TCPUDPService::cast(nserv)->setDstRangeStart(drsR);
+            TCPUDPService::cast(nserv)->setDstRangeEnd(dreR);
 
 	    res.push_back(nserv);
 	    return res;
@@ -172,7 +187,7 @@ vector<FWObject*> fwcompiler::_find_srv_intersection(Service *op1, Service *op2)
 }
 
 
-bool fwcompiler::checkForShadowing(const Service &o1,const Service &o2)
+bool fwcompiler::checkForShadowing(const Service &o1, const Service &o2)
 {
     if (o1.getId()==o2.getId()) return true;
     if (o1.isAny() && o2.isAny()) return false;
@@ -181,8 +196,11 @@ bool fwcompiler::checkForShadowing(const Service &o1,const Service &o2)
 
     if (o1.getTypeName()==o2.getTypeName()) 
     {
-	if ( IPService::constcast(&o1)) 
+        const IPService *ip1;
+        const IPService *ip2;
+	if ((ip1=IPService::constcast(&o1))!=NULL) 
         {
+            ip2 = IPService::constcast(&o2);
 /* 
  * Both objects are IPService
  *
@@ -197,8 +215,15 @@ bool fwcompiler::checkForShadowing(const Service &o1,const Service &o2)
                 o1.getStr("rr")!=o2.getStr("rr") ||
                 o1.getStr("ts")!=o2.getStr("ts") ) return false;
 
-	    if (o1.getInt("protocol_num")==o2.getInt("protocol_num")) return true;
-	    if (o1.getInt("protocol_num")!=0 && o2.getInt("protocol_num")==0) return true;
+            if (ip1->getTOSCode()!=ip2->getTOSCode() ||
+                ip1->getDSCPCode()!=ip2->getDSCPCode()) return false;
+
+	    if (o1.getInt("protocol_num")==o2.getInt("protocol_num"))
+                return true;
+
+	    if (o1.getInt("protocol_num")!=0 && o2.getInt("protocol_num")==0)
+                return true;
+
             return false;
 	}
 	if ( ICMPService::constcast(&o1)) 
@@ -218,18 +243,31 @@ bool fwcompiler::checkForShadowing(const Service &o1,const Service &o2)
         }
 	if ( TCPService::constcast(&o1) || UDPService::constcast(&o1) ) 
         {
-	    int srs1=o1.getInt("src_range_start");
-	    int sre1=o1.getInt("src_range_end");
-	    int drs1=o1.getInt("dst_range_start");
-	    int dre1=o1.getInt("dst_range_end");
+	    int srs1=static_cast<const TCPUDPService*>(&o1)->getSrcRangeStart();
+	    int sre1=static_cast<const TCPUDPService*>(&o1)->getSrcRangeEnd();
+	    int drs1=static_cast<const TCPUDPService*>(&o1)->getDstRangeStart();
+	    int dre1=static_cast<const TCPUDPService*>(&o1)->getDstRangeEnd();
 
-	    int srs2=o2.getInt("src_range_start");
-	    int sre2=o2.getInt("src_range_end");
-	    int drs2=o2.getInt("dst_range_start");
-	    int dre2=o2.getInt("dst_range_end");
+	    int srs2=static_cast<const TCPUDPService*>(&o2)->getSrcRangeStart();
+	    int sre2=static_cast<const TCPUDPService*>(&o2)->getSrcRangeEnd();
+	    int drs2=static_cast<const TCPUDPService*>(&o2)->getDstRangeStart();
+	    int dre2=static_cast<const TCPUDPService*>(&o2)->getDstRangeEnd();
 
 	    return (srs1>=srs2 && sre1<=sre2 && drs1>=drs2 && dre1<=dre2);
 	}
+	if ( TagService::constcast(&o1)) 
+        {
+            string tagvalue1 = TagService::constcast(&o1)->getCode();
+            string tagvalue2 = TagService::constcast(&o2)->getCode();
+            return (tagvalue1 == tagvalue2);
+	}
+	if ( UserService::constcast(&o1)) 
+        {
+            string uid1 = UserService::constcast(&o1)->getUserId();
+            string uid2 = UserService::constcast(&o2)->getUserId();
+            return (uid1 == uid2);
+	}
+
         return false;
     }
 /*
@@ -280,57 +318,71 @@ bool fwcompiler::checkForShadowing(const Address &o1,const Address &o2)
         return o1_pa->getPhysAddress()==o2_pa->getPhysAddress();
     }
     
-    IPAddress o1b;
-    IPAddress o1e;
-    IPAddress o2b;
-    IPAddress o2e;
+    const InetAddr *o1b;
+    const InetAddr *o1e;
+    const InetAddr *o2b;
+    const InetAddr *o2e;
 
     if (AddressRange::isA(&o1))
     {
-        o1b=AddressRange::constcast(&o1)->getRangeStart();  
-        o1e=AddressRange::constcast(&o1)->getRangeEnd();
+        o1b = &(AddressRange::constcast(&o1)->getRangeStart());
+        o1e = &(AddressRange::constcast(&o1)->getRangeEnd());
     } else
     {
         if (Network::isA(&o1))
         {
-            o1b=o1.getAddress();
-            o1e=IPNetwork(o1.getAddress(),o1.getNetmask()).getBroadcastAddress();
+            o1b = o1.getAddressPtr();
+            o1e = o1.getBroadcastAddressPtr();
         } else
         {
-            o1b=o1.getAddress();
-            o1e=o1.getAddress();
+            o1b = o1.getAddressPtr();
+            o1e = o1.getAddressPtr();
         }
     }
 
     if (AddressRange::isA(&o2))
     {
-        o2b=AddressRange::constcast(&o2)->getRangeStart();  
-        o2e=AddressRange::constcast(&o2)->getRangeEnd();
+        o2b = &(AddressRange::constcast(&o2)->getRangeStart());
+        o2e = &(AddressRange::constcast(&o2)->getRangeEnd());
     } else
     {
         if (Network::isA(&o2))
         {
-            o2b=o2.getAddress();
-            o2e=IPNetwork(o2.getAddress(),o2.getNetmask()).getBroadcastAddress();
+            o2b = o2.getAddressPtr();
+            o2e = o2.getBroadcastAddressPtr();
         } else
         {
-            o2b=o2.getAddress();
-            o2e=o2.getAddress();
+            o2b = o2.getAddressPtr();
+            o2e = o2.getAddressPtr();
         }
     }
-/*
-    cerr << "# o1=" << o1.getName() << " "
-         << o1b.toString() << "-" << o1e.toString() << "(" << o1.dimension() << ")"
-         << " o2=" << o2.getName()  << " "
-         << o2b.toString() << "-" << o2e.toString() << "(" << o2.dimension() << ")"
-         << " " << int( (o1b>o2b || o1b==o2b) && (o1e<o2e || o1e==o2e) )
+
+#if 0
+    cerr << "# o1=" << o1.getName() << " [" << o1.toString() << "] "
+         << o1b->toString() << "-" << o1e->toString()
+         << "(" << o1.dimension() << ")"
+         << " o2=" << o2.getName() << " [" << o2.toString() << "] "
+         << o2b->toString() << "-" << o2e->toString()
+         << "(" << o2.dimension() << ")"
+         << " " << int( ((*o1b)>(*o2b) || (*o1b)==(*o2b)) &&
+                        ((*o1e)<(*o2e) || (*o1e)==(*o2e)) )
          << endl;
-*/
+#endif
+
+    // if any of the objects has no ip address, then we can not
+    // check for shadowing and return false. examples of objects with
+    // no real ip address: physAddress, Interface with no child IPv4/IPv6
+    // object. High level compilers should make sure they process rules
+    // to the point where no such objects are left before they call
+    // this method.
+    if (o1b==NULL || o2b==NULL || o1e==NULL || o2e==NULL) return false;
+
     if (o1.isAny() && o2.isAny())  return true;
     if (o1.isAny() && !o2.isAny()) return false;
     if (!o1.isAny() && o2.isAny()) return true;
 
-    return ( (o2b<o1b || o1b==o2b) && (o1e<o2e || o1e==o2e) );
+    return ( ((*o2b) < (*o1b) || (*o1b) == (*o2b)) &&
+             ((*o1e) < (*o2e) || (*o1e) == (*o2e)) );
 }
 
 /*************************************************************************/
@@ -357,30 +409,30 @@ bool fwcompiler::operator==(const Service  &o1,const Service  &o2)
 		(o1.getInt("code")==o2.getInt("code"));
 	}
 	if ( UDPService::constcast(&o1) ) {
-	    int srs1=o1.getInt("src_range_start");
-	    int sre1=o1.getInt("src_range_end");
-	    int drs1=o1.getInt("dst_range_start");
-	    int dre1=o1.getInt("dst_range_end");
+	    int srs1=static_cast<const TCPUDPService*>(&o1)->getSrcRangeStart();
+	    int sre1=static_cast<const TCPUDPService*>(&o1)->getSrcRangeEnd();
+	    int drs1=static_cast<const TCPUDPService*>(&o1)->getDstRangeStart();
+	    int dre1=static_cast<const TCPUDPService*>(&o1)->getDstRangeEnd();
 
-	    int srs2=o2.getInt("src_range_start");
-	    int sre2=o2.getInt("src_range_end");
-	    int drs2=o2.getInt("dst_range_start");
-	    int dre2=o2.getInt("dst_range_end");
+	    int srs2=static_cast<const TCPUDPService*>(&o2)->getSrcRangeStart();
+	    int sre2=static_cast<const TCPUDPService*>(&o2)->getSrcRangeEnd();
+	    int drs2=static_cast<const TCPUDPService*>(&o2)->getDstRangeStart();
+	    int dre2=static_cast<const TCPUDPService*>(&o2)->getDstRangeEnd();
 
             return (srs1==srs2 && sre1==sre2 && drs1==drs2 && dre1==dre2);
 	}
 
 	if ( TCPService::constcast(&o1))
         {
-	    int srs1=o1.getInt("src_range_start");
-	    int sre1=o1.getInt("src_range_end");
-	    int drs1=o1.getInt("dst_range_start");
-	    int dre1=o1.getInt("dst_range_end");
+	    int srs1=static_cast<const TCPUDPService*>(&o1)->getSrcRangeStart();
+	    int sre1=static_cast<const TCPUDPService*>(&o1)->getSrcRangeEnd();
+	    int drs1=static_cast<const TCPUDPService*>(&o1)->getDstRangeStart();
+	    int dre1=static_cast<const TCPUDPService*>(&o1)->getDstRangeEnd();
 
-	    int srs2=o2.getInt("src_range_start");
-	    int sre2=o2.getInt("src_range_end");
-	    int drs2=o2.getInt("dst_range_start");
-	    int dre2=o2.getInt("dst_range_end");
+	    int srs2=static_cast<const TCPUDPService*>(&o2)->getSrcRangeStart();
+	    int sre2=static_cast<const TCPUDPService*>(&o2)->getSrcRangeEnd();
+	    int drs2=static_cast<const TCPUDPService*>(&o2)->getDstRangeStart();
+	    int dre2=static_cast<const TCPUDPService*>(&o2)->getDstRangeEnd();
 
             const TCPService *tcp1=TCPService::constcast(&o1);
             const TCPService *tcp2=TCPService::constcast(&o2);
@@ -398,10 +450,10 @@ bool fwcompiler::operator==(const Address &o1,const Address &o2)
 {
     if (o1.getId()==o2.getId()) return true;
 
-    IPAddress o1b;
-    IPAddress o1e;
-    IPAddress o2b;
-    IPAddress o2e;
+    const InetAddr* o1b;
+    const InetAddr* o1e;
+    const InetAddr* o2b;
+    const InetAddr* o2e;
 
     if (Interface::isA(&o1) && Interface::isA(&o2))
     {
@@ -422,35 +474,36 @@ bool fwcompiler::operator==(const Address &o1,const Address &o2)
     
     if (AddressRange::isA(&o1))
     {
-        o1b=AddressRange::constcast(&o1)->getRangeStart();  
-        o1e=AddressRange::constcast(&o1)->getRangeEnd();
+        o1b = &(AddressRange::constcast(&o1)->getRangeStart());  
+        o1e = &(AddressRange::constcast(&o1)->getRangeEnd());
     } else
         if (Network::isA(&o1))
         {
-            o1b=o1.getAddress();
-            o1e=IPNetwork(o1.getAddress(),o1.getNetmask()).getBroadcastAddress();
+            o1b = o1.getAddressPtr();
+            o1e = o1.getBroadcastAddressPtr();
         } else
         {
-            o1b=o1.getAddress();
-            o1e=o1.getAddress();
+            o1b = o1.getAddressPtr();
+            o1e = o1.getAddressPtr();
         }
 
     if (AddressRange::isA(&o2))
     {
-        o2b=AddressRange::constcast(&o2)->getRangeStart();  
-        o2e=AddressRange::constcast(&o2)->getRangeEnd();
+        o2b = &(AddressRange::constcast(&o2)->getRangeStart());  
+        o2e = &(AddressRange::constcast(&o2)->getRangeEnd());
     } else
         if (Network::isA(&o2))
         {
-            o2b=o2.getAddress();
-            o2e=IPNetwork(o2.getAddress(),o2.getNetmask()).getBroadcastAddress();
+            o2b = o2.getAddressPtr();
+            o2e = o2.getBroadcastAddressPtr();
         } else
         {
-            o2b=o2.getAddress();
-            o2e=o2.getAddress();
+            o2b = o2.getAddressPtr();
+            o2e = o2.getAddressPtr();
         }
 
-    return (o1b==o2b && o1e==o2e);
+    if (o1b==NULL || o2b==NULL || o1e==NULL || o2e==NULL) return false;
+    return ((*o1b) == (*o2b) && (*o1e) == (*o2e));
 }
 
 bool fwcompiler::operator==(const Interval  &o1,const Interval  &o2)
@@ -494,7 +547,7 @@ bool fwcompiler::operator<=(const Service  &o1,const Service  &o2)
 
 /*************************************************************************/
 
-bool Compiler::intersect(const PolicyRule &r1,const PolicyRule &r2)
+bool Compiler::intersect(PolicyRule &r1, PolicyRule &r2)
 {
     string act1=r1.getActionAsString();
     string act2=r2.getActionAsString();
@@ -503,11 +556,11 @@ bool Compiler::intersect(const PolicyRule &r1,const PolicyRule &r2)
     res_act=(act1=="Continue" || act2=="Continue" || act1==act2);
     if (res_act==false) return false;
 
-    string iface1=r1.getInterfaceId();
-    string iface2=r2.getInterfaceId();
+    int iface1 = r1.getInterfaceId();
+    int iface2 = r2.getInterfaceId();
 
     bool res_iface;
-    res_iface=(iface1=="" || iface2=="" || iface1==iface2);
+    res_iface=(iface1==-1 || iface2==-1 || iface1==iface2);
     if (res_iface==false) return false;
 
     vector<FWObject*> v1=_find_obj_intersection( getFirstSrc(&r1) , getFirstSrc(&r2) );
@@ -523,7 +576,7 @@ bool Compiler::intersect(const PolicyRule &r1,const PolicyRule &r2)
 }
 
 
-void Compiler::getIntersection( const PolicyRule &r1, const PolicyRule &r2, PolicyRule &res)
+void Compiler::getIntersection(PolicyRule &r1, PolicyRule &r2, PolicyRule &res)
 {
     string act1=r1.getActionAsString();
     string act2=r2.getActionAsString();
