@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler.cpp,v 1.6 2006/09/09 05:02:58 vkurland Exp $
+  $Id: NATCompiler.cpp 123 2008-07-05 17:38:52Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -33,7 +33,7 @@
 #include "fwbuilder/InterfacePolicy.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/RuleSet.h"
-#include "fwbuilder/IPAddress.h"
+#include "fwbuilder/InetAddr.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/FWObjectDatabase.h"
@@ -76,17 +76,24 @@ int NATCompiler::prolog()
 //    for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) {
 
     FWObject *ruleset = source_ruleset;
-    if (ruleset == NULL) ruleset = nat;
+    if (ruleset == NULL)
+    {
+        source_ruleset = RuleSet::cast(nat);
+        ruleset = nat;
+    }
+
+    string label_prefix = "";
+    if (ruleset->getName() != "NAT") label_prefix = ruleset->getName();
 
     for (FWObject::iterator i=ruleset->begin(); i!=ruleset->end(); i++)
     {
 
 	Rule *r= Rule::cast(*i);
 	if (r->isDisabled()) continue;
-	r->setInterfaceId("");
-	r->setLabel( createRuleLabel("NAT", r->getPosition()) );
+	r->setInterfaceId(-1);
+	r->setLabel( createRuleLabel(label_prefix, "NAT", r->getPosition()) );
 	r->setAbsRuleNumber(global_num); global_num++;
-        r->setUniqueId( r->getId() );
+        r->setUniqueId( FWObjectDatabase::getStringId(r->getId()) );
 	combined_ruleset->add( r );
     }
 
@@ -97,7 +104,7 @@ int NATCompiler::prolog()
 
 
 
-bool NATCompiler::checkForShadowing(const NATRule &r1,const NATRule &r2)
+bool NATCompiler::checkForShadowing(NATRule &r1, NATRule &r2)
 {
     Address  *osrc1=getFirstOSrc(&r1);
     Address  *odst1=getFirstODst(&r1);
@@ -125,7 +132,7 @@ bool NATCompiler::checkForShadowing(const NATRule &r1,const NATRule &r2)
 /*
  * TODO: implement this
  */
-bool NATCompiler::cmpRules(const NATRule &r1,const NATRule &r2)
+bool NATCompiler::cmpRules(NATRule&, NATRule&)
 {
     return false;
 }
@@ -228,23 +235,28 @@ bool NATCompiler::ExpandMultipleAddresses::processNext()
     tmp_queue.push_back(rule);
 
     RuleElement *rel;
-    if (rule->getRuleType()==NATRule::NONAT) {
+    if (rule->getRuleType()==NATRule::NONAT ||
+        rule->getRuleType()==NATRule::Return)
+    {
 	rel=rule->getOSrc();    assert(rel); compiler->_expandAddr(rule,rel);
 	rel=rule->getODst();    assert(rel); compiler->_expandAddr(rule,rel);
     }
-    if (rule->getRuleType()==NATRule::SNAT) {
+    if (rule->getRuleType()==NATRule::SNAT)
+    {
 	rel=rule->getOSrc();    assert(rel); compiler->_expandAddr(rule,rel);
 	rel=rule->getODst();    assert(rel); compiler->_expandAddr(rule,rel);
 	rel=rule->getTSrc();    assert(rel); compiler->_expandAddr(rule,rel);
 	rel=rule->getTDst();    assert(rel); compiler->_expandAddr(rule,rel);
     }
-    if (rule->getRuleType()==NATRule::DNAT) {
+    if (rule->getRuleType()==NATRule::DNAT)
+    {
 	rel=rule->getOSrc();    assert(rel); compiler->_expandAddr(rule,rel);
 	rel=rule->getODst();    assert(rel); compiler->_expandAddr(rule,rel);
 	rel=rule->getTSrc();    assert(rel); compiler->_expandAddr(rule,rel);
 	rel=rule->getTDst();    assert(rel); compiler->_expandAddr(rule,rel);
     }
-    if (rule->getRuleType()==NATRule::Redirect) {
+    if (rule->getRuleType()==NATRule::Redirect)
+    {
 	rel=rule->getOSrc();    assert(rel); compiler->_expandAddr(rule,rel);
 	rel=rule->getODst();    assert(rel); compiler->_expandAddr(rule,rel);
 	rel=rule->getTSrc();    assert(rel); compiler->_expandAddr(rule,rel);
@@ -667,80 +679,137 @@ bool NATCompiler::splitODstForSNAT::processNext()
     return true;
 }
 
+bool NATCompiler::DropRulesByAddressFamilyAndServiceType::processNext()
+{
+    NATRule *rule = getNext(); if (rule==NULL) return false;
+    RuleElementOSrc *osrc = rule->getOSrc();
+    RuleElementODst *odst = rule->getODst();
+    RuleElementOSrv *osrv = rule->getOSrv();
+
+    RuleElementTSrc *tsrc = rule->getTSrc();
+    RuleElementTDst *tdst = rule->getTDst();
+    RuleElementTSrv *tsrv = rule->getTSrv();
+
+    compiler->DropAddressFamilyInRE(osrc, drop_ipv6);
+    compiler->DropAddressFamilyInRE(odst, drop_ipv6);
+    compiler->DropByServiceTypeInRE(osrv, drop_ipv6);
+
+    compiler->DropAddressFamilyInRE(tsrc, drop_ipv6);
+    compiler->DropAddressFamilyInRE(tdst, drop_ipv6);
+    compiler->DropByServiceTypeInRE(tsrv, drop_ipv6);
+
+    tmp_queue.push_back(rule);
+
+    return true;
+}
+
+bool NATCompiler::dropRuleWithEmptyRE::processNext()
+{
+    NATRule *rule = getNext(); if (rule==NULL) return false;
+    RuleElementOSrc *osrcrel = rule->getOSrc();
+    RuleElementODst *odstrel = rule->getODst();
+    RuleElementTSrc *tsrcrel = rule->getTSrc();
+    RuleElementTDst *tdstrel = rule->getTDst();
+    if ((osrcrel->size() == 0) || (odstrel->size() == 0)) return true;
+    if ((tsrcrel->size() == 0) || (tdstrel->size() == 0)) return true;
+//    Address *osrc = compiler->getFirstOSrc(rule);
+//    Address *odst = compiler->getFirstODst(rule);
+//    Address *tsrc = compiler->getFirstTSrc(rule);
+//    Address *tdst = compiler->getFirstTDst(rule);
+//    if (osrc!=NULL && odst!=NULL && tsrc!=NULL && tdst!=NULL)
+//      tmp_queue.push_back(rule);
+
+    tmp_queue.push_back(rule);
+    return true;
+}
 
 string NATCompiler::debugPrintRule(libfwbuilder::Rule *r)
 {
-    NATRule *rule=NATRule::cast(r);
+    NATRule *rule = NATRule::cast(r);
 
-    RuleElementOSrc *osrcrel=rule->getOSrc();
-    RuleElementODst *odstrel=rule->getODst();
-    RuleElementOSrv *osrvrel=rule->getOSrv();
+    RuleElementOSrc *osrcrel = rule->getOSrc();
+    RuleElementODst *odstrel = rule->getODst();
+    RuleElementOSrv *osrvrel = rule->getOSrv();
 
-    RuleElementTSrc *tsrcrel=rule->getTSrc();
-    RuleElementTDst *tdstrel=rule->getTDst();
-    RuleElementTSrv *tsrvrel=rule->getTSrv();
+    RuleElementTSrc *tsrcrel = rule->getTSrc();
+    RuleElementTDst *tdstrel = rule->getTDst();
+    RuleElementTSrv *tsrvrel = rule->getTSrv();
 
 
     ostringstream str;
 
 //    str << setw(70) << setfill('-') << "-";
 
-    int no=0;
-    FWObject::iterator i1=osrcrel->begin();
-    FWObject::iterator i2=odstrel->begin(); 
-    FWObject::iterator i3=osrvrel->begin();
+    int no = 0;
+    FWObject::iterator i1 = osrcrel->begin();
+    FWObject::iterator i2 = odstrel->begin(); 
+    FWObject::iterator i3 = osrvrel->begin();
 
-    FWObject::iterator i4=tsrcrel->begin();
-    FWObject::iterator i5=tdstrel->begin(); 
-    FWObject::iterator i6=tsrvrel->begin();
+    FWObject::iterator i4 = tsrcrel->begin();
+    FWObject::iterator i5 = tdstrel->begin(); 
+    FWObject::iterator i6 = tsrvrel->begin();
 
     while ( i1!=osrcrel->end() || i2!=odstrel->end() || i3!=osrvrel->end() ||
             i4!=tsrcrel->end() || i5!=tdstrel->end() || i6!=tsrvrel->end() ) {
 
         str  << endl;
 
-        string osrc=" ";
-        string odst=" ";
-        string osrv=" ";
+        string osrc = " ";
+        string odst = " ";
+        string osrv = " ";
 
-        string tsrc=" ";
-        string tdst=" ";
-        string tsrv=" ";
+        string tsrc = " ";
+        string tdst = " ";
+        string tsrv = " ";
+
+        int osrc_id = -1;
+        int odst_id = -1;
+        int osrv_id = -1;
+
+        int tsrc_id = -1;
+        int tdst_id = -1;
+        int tsrv_id = -1;
 
         if (i1!=osrcrel->end()) {
             FWObject *o=*i1;
-            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
             osrc=o->getName();
+            osrc_id=o->getId();
         }
 
         if (i2!=odstrel->end()) {
             FWObject *o=*i2;
-            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
             odst=o->getName();
+            odst_id=o->getId();
         }
 
         if (i3!=osrvrel->end()) {
             FWObject *o=*i3;
-            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
             osrv=o->getName();
+            osrv_id=o->getId();
         }
 
         if (i4!=tsrcrel->end()) {
             FWObject *o=*i4;
-            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
             tsrc=o->getName();
+            tsrc_id=o->getId();
         }
 
         if (i5!=tdstrel->end()) {
             FWObject *o=*i5;
-            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
             tdst=o->getName();
+            tdst_id=o->getId();
         }
 
         if (i6!=tsrvrel->end()) {
             FWObject *o=*i6;
-            if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
             tsrv=o->getName();
+            tsrv_id=o->getId();
         }
 
         int w=0;
@@ -749,17 +818,17 @@ string NATCompiler::debugPrintRule(libfwbuilder::Rule *r)
             w=rule->getLabel().length();
         }
         
-        str <<  setw(8-w)  << setfill(' ') << " ";
+        str << setw(8-w)  << setfill(' ') << " ";
 
-        str <<  setw(16) << setfill(' ') << osrc.c_str();
-        str <<  setw(16) << setfill(' ') << odst.c_str();
-        str <<  setw(10) << setfill(' ') << osrv.c_str();
+        str << setw(16) << setfill(' ') << osrc.c_str() << "(" << osrc_id << ")";
+        str << setw(16) << setfill(' ') << odst.c_str() << "(" << odst_id << ")";
+        str << setw(10) << setfill(' ') << osrv.c_str() << "(" << osrv_id << ")";
 //        str <<  endl;
 //
 //        str <<  setw(8)  << setfill(' ') << " ";
-        str <<  setw(16) << setfill(' ') << tsrc.c_str();
-        str <<  setw(16) << setfill(' ') << tdst.c_str();
-        str <<  setw(10) << setfill(' ') << tsrv.c_str();
+        str << setw(16) << setfill(' ') << tsrc.c_str() << "(" << tsrc_id << ")";
+        str << setw(16) << setfill(' ') << tdst.c_str() << "(" << tdst_id << ")";
+        str << setw(10) << setfill(' ') << tsrv.c_str() << "(" << tsrv_id << ")";
 
 
         ++no;
