@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler.cpp,v 1.22 2008/05/08 17:41:21 vkurland Exp $
+  $Id: PolicyCompiler.cpp 123 2008-07-05 17:38:52Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -36,13 +36,14 @@
 #include "fwbuilder/ICMPService.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
+#include "fwbuilder/UserService.h"
 #include "fwbuilder/CustomService.h"
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Rule.h"
 #include "fwbuilder/InterfacePolicy.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/RuleSet.h"
-#include "fwbuilder/IPAddress.h"
+#include "fwbuilder/InetAddr.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/FWObjectDatabase.h"
@@ -75,41 +76,15 @@ int PolicyCompiler::prolog()
 
     int global_num=0;
 
-#ifdef USE_INTERFACE_POLICY
-
-/*
- *  build combined policy by collapsing all the rules together.
- *  store ID of the interface in each rule of interface policy.
- *
- *  also calculate global numbers for all rules and store them, too.
- *  These are used to detect rule shadowing.
- */
-    list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
-    for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) 
-    {
-	Interface *iface=Interface::cast(*i);
-	assert(iface);
-
-	FWObject  *ipolicy=iface->getFirstByType(InterfacePolicy::TYPENAME);
-	if (ipolicy) 
-        { 	
-            list<FWObject*> l3=ipolicy->getByType(PolicyRule::TYPENAME);
-            for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) 
-            {
-                PolicyRule *r= PolicyRule::cast(*j);
-                if (r->isDisabled()) continue;
-                r->setInterfaceId(iface->getId());
-                r->setLabel( createRuleLabel(iface, r->getPosition()) );
-                r->setAbsRuleNumber(global_num); global_num++;
-                r->setUniqueId( r->getId() );
-                combined_ruleset->add( r );
-            }
-	}
-    }
-#endif
-
     FWObject *ruleset = source_ruleset;
-    if (ruleset == NULL) ruleset = policy;
+    if (ruleset == NULL)
+    {
+        source_ruleset = RuleSet::cast(policy);
+        ruleset = policy;
+    }
+
+    string label_prefix = "";
+    if (ruleset->getName() != "Policy") label_prefix = ruleset->getName();
 
     for (FWObject::iterator i=ruleset->begin(); i!=ruleset->end(); i++)
     {
@@ -120,23 +95,24 @@ int PolicyCompiler::prolog()
 
         if (itfre->isAny())
         {
-//          r->setInterfaceId("");
-            r->setLabel( createRuleLabel("global", r->getPosition()) );
+            r->setLabel( createRuleLabel(label_prefix, 
+                                         "global", r->getPosition()) );
         } else
         {
-//          r->setInterfaceId( itf->getId() );
             string interfaces = "";
             for (FWObject::iterator i=itfre->begin(); i!=itfre->end(); ++i)
             {
                 FWObject *o=*i;
-                if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+                if (FWReference::cast(o)!=NULL)
+                    o = FWReference::cast(o)->getPointer();
                 if (interfaces!="") interfaces += ",";
                 interfaces += o->getName();
             }
-            r->setLabel( createRuleLabel(interfaces, r->getPosition()) );
+            r->setLabel( createRuleLabel(label_prefix, 
+                                         interfaces, r->getPosition()) );
         }
 	r->setAbsRuleNumber(global_num); global_num++;
-        r->setUniqueId( r->getId() );
+        r->setUniqueId( FWObjectDatabase::getStringId(r->getId()) );
 	combined_ruleset->add( r );
     }
 
@@ -150,7 +126,8 @@ int PolicyCompiler::prolog()
 /*
  * detects if tule r2 shades rule r1.
  */
-bool PolicyCompiler::checkForShadowing(const PolicyRule &r1,const PolicyRule &r2)
+bool PolicyCompiler::checkForShadowing(PolicyRule &r1,
+                                       PolicyRule &r2)
 {
     if (r1.getSrc()->getNeg()) return false;
     if (r1.getDst()->getNeg()) return false;
@@ -192,13 +169,29 @@ bool PolicyCompiler::checkForShadowing(const PolicyRule &r1,const PolicyRule &r2
         r2.getAction()==PolicyRule::Branch ) return false;
 
 
-    Address  *src1=getFirstSrc(&r1);
-    Address  *dst1=getFirstDst(&r1);
-    Service  *srv1=getFirstSrv(&r1);
+    //Address  *src1=getFirstSrc(&r1);
+    //Address  *dst1=getFirstDst(&r1);
+    //Service  *srv1=getFirstSrv(&r1);
+
+    //Address  *src2=getFirstSrc(&r2);
+    //Address  *dst2=getFirstDst(&r2);
+    //Service  *srv2=getFirstSrv(&r2);
+
+    Address  *src1 = static_cast<Address*>(
+        static_cast<FWReference*>(r1.getFirstByType(RuleElementSrc::TYPENAME)->front())->getPointer());
+    Address  *dst1 = static_cast<Address*>(
+        static_cast<FWReference*>(r1.getFirstByType(RuleElementDst::TYPENAME)->front())->getPointer());
+    Service  *srv1 = static_cast<Service*>(
+        static_cast<FWReference*>(r1.getFirstByType(RuleElementSrv::TYPENAME)->front())->getPointer());
+
+    Address  *src2 = static_cast<Address*>(
+        static_cast<FWReference*>(r2.getFirstByType(RuleElementSrc::TYPENAME)->front())->getPointer());
+    Address  *dst2 = static_cast<Address*>(
+        static_cast<FWReference*>(r2.getFirstByType(RuleElementDst::TYPENAME)->front())->getPointer());
+    Service  *srv2 = static_cast<Service*>(
+        static_cast<FWReference*>(r2.getFirstByType(RuleElementSrv::TYPENAME)->front())->getPointer());
+
 
-    Address  *src2=getFirstSrc(&r2);
-    Address  *dst2=getFirstDst(&r2);
-    Service  *srv2=getFirstSrv(&r2);
 
     if (MultiAddressRunTime::isA(src1) || MultiAddressRunTime::isA(dst1) ||
         MultiAddressRunTime::isA(src2) || MultiAddressRunTime::isA(dst2))
@@ -228,18 +221,19 @@ bool PolicyCompiler::checkForShadowing(const PolicyRule &r1,const PolicyRule &r2
  */
 //    if (srv1->getTypeName()==srv2->getTypeName() || (!srv1->isAny() && srv2->isAny()))
 //    {
-        return ( 
-            fwcompiler::checkForShadowing(*src1, *src2) && 
-            fwcompiler::checkForShadowing(*dst1, *dst2) && 
-            fwcompiler::checkForShadowing(*srv1, *srv2) 
-        );
+    return ( 
+        fwcompiler::checkForShadowing(*src1, *src2) && 
+        fwcompiler::checkForShadowing(*dst1, *dst2) && 
+        fwcompiler::checkForShadowing(*srv1, *srv2) 
+    );
+
 //    }
 
     return false;
 }
 
-bool PolicyCompiler::cmpRules(const PolicyRule &r1,
-                              const PolicyRule &r2)
+bool PolicyCompiler::cmpRules(PolicyRule &r1,
+                              PolicyRule &r2)
 {
     if (r1.getSrc()->getNeg()!=r2.getSrc()->getNeg()) return false;
     if (r1.getDst()->getNeg()!=r2.getDst()->getNeg()) return false;
@@ -286,7 +280,7 @@ bool PolicyCompiler::ItfNegation::processNext()
     if (itfre==NULL)
         compiler->abort("Missing interface RE in rule '" +
                         rule->getLabel() +
-                        "' id=" + rule->getId()
+                        "' id=" + FWObjectDatabase::getStringId(rule->getId())
         );
 
     if (itfre->getNeg())
@@ -316,7 +310,7 @@ bool  PolicyCompiler::InterfacePolicyRules::processNext()
     RuleElementItf *itfre=rule->getItf();   assert(itfre);
     if (itfre->isAny())
     {
-        rule->setInterfaceId("");
+        rule->setInterfaceId(-1);
         tmp_queue.push_back(rule);
         return true;
     }
@@ -383,62 +377,46 @@ bool  PolicyCompiler::ExpandGroups::processNext()
 bool  PolicyCompiler::ExpandMultipleAddressesInSRC::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
-
-    tmp_queue.push_back(rule);
-
     RuleElementSrc *src=rule->getSrc();    assert(src);
-
     compiler->_expandAddr(rule,src);
-
+    tmp_queue.push_back(rule);
     return true;
 }
 
 bool  PolicyCompiler::ExpandMultipleAddressesInDST::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
-
-    tmp_queue.push_back(rule);
-
     RuleElementDst *dst=rule->getDst();    assert(dst);
-
     compiler->_expandAddr(rule,dst);
-
+    tmp_queue.push_back(rule);
     return true;
 }
 
 bool  PolicyCompiler::ExpandMultipleAddresses::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
-
-    tmp_queue.push_back(rule);
-
     RuleElementSrc *src=rule->getSrc();    assert(src);
     RuleElementDst *dst=rule->getDst();    assert(dst);
-
     compiler->_expandAddr(rule,src);
     compiler->_expandAddr(rule,dst);
-
+    tmp_queue.push_back(rule);
     return true;
 }
 
 bool  PolicyCompiler::addressRanges::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
-
-    tmp_queue.push_back(rule);
-
     RuleElementSrc *src=rule->getSrc();    assert(src);
     RuleElementDst *dst=rule->getDst();    assert(dst);
-
     compiler->_expandAddressRanges(rule,src);
     compiler->_expandAddressRanges(rule,dst);
-
+    tmp_queue.push_back(rule);
     return true;
 }
 
 
 
-Rule* PolicyCompiler::getDifference(const PolicyRule &r1, const PolicyRule &r2)
+Rule* PolicyCompiler::getDifference(PolicyRule &r1, PolicyRule &r2)
 {
     PolicyRule *r = new PolicyRule();
     *r = r1;
@@ -540,7 +518,6 @@ bool PolicyCompiler::splitServices::processNext()
 
     map<int, list<Service*> > services;
 
-// loop:
     for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++)
     {
         FWObject *o= *i;
@@ -553,7 +530,8 @@ bool PolicyCompiler::splitServices::processNext()
         services[proto].push_back(s);
     }
 
-    for (map<int, list<Service*> >::iterator i1=services.begin(); i1!=services.end(); i1++)
+    for (map<int, list<Service*> >::iterator i1=services.begin();
+         i1!=services.end(); i1++)
     {
         list<Service*> &sl=(*i1).second;
 
@@ -665,21 +643,30 @@ Address* PolicyCompiler::checkForZeroAddr::findZeroAddress(RuleElement *re)
         MultiAddress *maddr = MultiAddress::cast(o);
         if (maddr && maddr->isRunTime()) continue;
 
-        Address *addr=Address::cast(o);
+        Address *addr = Address::cast(o);
+        
+        if (addr==NULL && o!=NULL)
+            compiler->warning(
+                string("findZeroAddress: Unknown object in rule element: ") +
+                o->getName() +
+                "  type=" + o->getTypeName());
 
-        if (Interface::cast(o)!=NULL && 
-            (Interface::cast(o)->isDyn() ||
-             Interface::cast(o)->isUnnumbered() ||
-             Interface::cast(o)->isBridgePort()))
-            continue;
-
-        if ( ! addr->isAny() 
-             && addr->getAddress()==IPAddress("0.0.0.0")
-             && addr->getNetmask()==Netmask("0.0.0.0")
-        ) 
+        if (addr && addr->hasInetAddress())
         {
-            a=addr;
-            break;
+            if (Interface::cast(o)!=NULL && 
+                (Interface::cast(o)->isDyn() ||
+                 Interface::cast(o)->isUnnumbered() ||
+                 Interface::cast(o)->isBridgePort()))
+                continue;
+
+            if ( ! addr->isAny() 
+                 && addr->getAddressPtr()->isAny()
+                 && addr->getNetmaskPtr()->isAny()
+            ) 
+            {
+                a = addr;
+                break;
+            }
         }
     }
 
@@ -717,22 +704,25 @@ bool PolicyCompiler::checkForZeroAddr::processNext()
 
     Address *a=NULL;
 
-    a=findHostWithNoInterfaces( rule->getSrc() );
-    if (a==NULL) a=findHostWithNoInterfaces( rule->getDst() );
+    a = findHostWithNoInterfaces( rule->getSrc() );
+    if (a==NULL) a = findHostWithNoInterfaces( rule->getDst() );
 
     if (a!=NULL)
-        compiler->abort("Object '"+a->getName()+"' has no interfaces, therefore it does not have an address and can not be used in the rule."+" Rule "+rule->getLabel());
+        compiler->abort("Object '"+a->getName()+
+                        "' has no interfaces, therefore it does not have "
+                        "address and can not be used in the rule."+
+                        " Rule "+rule->getLabel());
 
-    a=findZeroAddress( rule->getSrc() );
-    if (a==NULL) a=findZeroAddress( rule->getDst() );
+    a = findZeroAddress( rule->getSrc() );
+    if (a==NULL) a = findZeroAddress( rule->getDst() );
 
     if (a!=NULL)
     {
         string err="Object '"+a->getName()+"'";
-        if (IPv4::cast(a)!=NULL)
+        if (IPv4::cast(a)!=NULL) // || IPv6::cast(a)!=NULL
         {
             FWObject *p=a->getParent();
-            Interface *iface=Interface::cast(p);
+            Interface *iface = Interface::cast(p);
             if (iface!=NULL) 
             {
                 err+=" (an address of interface ";
@@ -741,7 +731,8 @@ bool PolicyCompiler::checkForZeroAddr::processNext()
                 err+=" )";
             }
         }
-        err+=" has address 0.0.0.0, which is equivalent to 'any'. This is most likely an error. Rule "+rule->getLabel();
+        err += " has address 0.0.0.0, which is equivalent to 'any'. "
+            "This is most likely an error. Rule " + rule->getLabel();
 
         compiler->abort(err);
     }
@@ -865,16 +856,17 @@ bool  PolicyCompiler::ConvertToAtomic::processNext()
 
 
 std::deque<Rule*>::iterator 
-PolicyCompiler::findMoreGeneralRule::find_more_general_rule(PolicyRule *rule,
-                                                            bool check_interface,
-                                                            const std::deque<Rule*>::iterator &start_here,
-                                                            const std::deque<Rule*>::iterator &stop_here,
-                                                            bool reverse)
+PolicyCompiler::findMoreGeneralRule::find_more_general_rule(
+    PolicyRule *rule,
+    bool check_interface,
+    const std::deque<Rule*>::iterator &start_here,
+    const std::deque<Rule*>::iterator &stop_here,
+    bool reverse)
 {
     PolicyCompiler *pcomp=dynamic_cast<PolicyCompiler*>(compiler);
     if (compiler->debug>=9) 
     {
-        cerr << "*********  searching for more general rule: --------------------\n";
+        cerr << "*********  searching for more general rule: -------------\n";
         cerr << compiler->debugPrintRule(rule);
         cerr << endl;
     }
@@ -924,11 +916,12 @@ bool PolicyCompiler::DetectShadowing::processNext()
     if (rule->isFallback()) return true; // do not check fallback  ..
     if (rule->isHidden())   return true; //  ... and hidden rules
 
-    std::deque<Rule*>::iterator i=find_more_general_rule(rule,
-                                                         true,
-                                                         rules_seen_so_far.begin(),
-                                                         rules_seen_so_far.end(),
-                                                         false);
+    std::deque<Rule*>::iterator i =
+        find_more_general_rule(rule,
+                               true,
+                               rules_seen_so_far.begin(),
+                               rules_seen_so_far.end(),
+                               false);
     if (i!=rules_seen_so_far.end()) 
     {
         Rule *r = *i;
@@ -957,11 +950,12 @@ bool PolicyCompiler::DetectShadowingForNonTerminatingRules::processNext()
     if (rule->isFallback()) return true; // do not check fallback  ..
     if (rule->isHidden())   return true; //  ... and hidden rules
 
-    std::deque<Rule*>::iterator i=find_more_general_rule(rule,
-                                                         true,
-                                                         rules_seen_so_far.begin(),
-                                                         rules_seen_so_far.end(),
-                                                         true);   // <<<<<<< NB!
+    std::deque<Rule*>::iterator i = 
+        find_more_general_rule(rule,
+                               true,
+                               rules_seen_so_far.begin(),
+                               rules_seen_so_far.end(),
+                               true);   // <<<<<<< NB!
     if (i!=rules_seen_so_far.end()) 
     {
         Rule *r = *i;
@@ -1058,7 +1052,84 @@ bool PolicyCompiler::CheckForTCPEstablished::processNext()
     return true;
 }
 
+bool PolicyCompiler::CheckForUnsupportedUserService::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
+    RuleElementSrv *srv=rule->getSrv();
+
+    for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+
+        if (UserService::isA(o))
+            compiler->abort(string("UserService object is not supported by ") +
+                            compiler->myPlatformName());
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+/* keep only rules that have ipv4 addresses in src and dst
+ * 
+ * TODO: figure out what to do with rules that have mix of ipv4 and ipv6
+ * addresses in different rule elements (such as ipv4 address in odst
+ * and ipv6 address in tdst or similar)
+ */
+bool PolicyCompiler::DropRulesByAddressFamilyAndServiceType::processNext()
+{
+    PolicyRule *rule = getNext(); if (rule==NULL) return false;
+    RuleElement *src = rule->getSrc();
+    RuleElement *dst = rule->getDst();
+    RuleElement *srv = rule->getSrv();
+
+    bool orig_src_any = src->isAny();
+    bool orig_dst_any = dst->isAny();
+    bool orig_srv_any = srv->isAny();
+    compiler->DropAddressFamilyInRE(src, drop_ipv6);
+    compiler->DropAddressFamilyInRE(dst, drop_ipv6);
+    compiler->DropByServiceTypeInRE(srv, drop_ipv6);
+
+    if (!orig_src_any && src->isAny())
+    {
+        // removing all ipv6 addresses from source makes it 'any', drop
+        // this rule
+        return true;
+    }
+
+    if (!orig_dst_any && dst->isAny())
+    {
+        // removing all ipv6 addresses from destination makes it 'any', drop
+        // this rule
+        return true;
+    }
+
+    if (!orig_srv_any && srv->isAny())
+    {
+        // removing all ipv6 addresses from service makes it 'any', drop
+        // this rule
+        return true;
+    }
+
+    tmp_queue.push_back(rule);
+
+    return true;
+}
+
+bool PolicyCompiler::dropRuleWithEmptyRE::processNext()
+{
+    PolicyRule *rule = getNext(); if (rule==NULL) return false;
+    RuleElementSrc *srcrel=rule->getSrc();
+    RuleElementDst *dstrel=rule->getDst();
+    if ((srcrel->size() == 0) || (dstrel->size() == 0)) return true;
+//    Address     *src = compiler->getFirstSrc(rule);
+//    Address     *dst = compiler->getFirstDst(rule);
+//    if (src!=NULL && dst!=NULL) tmp_queue.push_back(rule);
+    tmp_queue.push_back(rule);
+    return true;
+}
 
 
 string PolicyCompiler::debugPrintRule(Rule *r)
@@ -1071,7 +1142,7 @@ string PolicyCompiler::debugPrintRule(Rule *r)
     RuleElementDst *dstrel=rule->getDst();
     RuleElementSrv *srvrel=rule->getSrv();
 
-    string       iface_id = rule->getInterfaceId();
+    int iface_id = rule->getInterfaceId();
     Interface *rule_iface = fw_interfaces[iface_id];
 
     ostringstream str;
@@ -1090,6 +1161,10 @@ string PolicyCompiler::debugPrintRule(Rule *r)
         string dst=" ";
         string srv=" ";
 
+        int src_id = -1;
+        int dst_id = -1;
+        int srv_id = -1;
+
         if (srcrel->getNeg()) src="!";
         if (dstrel->getNeg()) dst="!";
         if (srvrel->getNeg()) srv="!";
@@ -1097,21 +1172,22 @@ string PolicyCompiler::debugPrintRule(Rule *r)
         if (i1!=srcrel->end()) {
             FWObject *o=*i1;
             if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-            src+=o->getName();
-            src+="("+o->getId()+")";
+            src += o->getName();
+            src_id = o->getId();
         }
 
         if (i2!=dstrel->end()) {
             FWObject *o=*i2;
             if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-            dst+=o->getName();
-            dst+="("+o->getId()+")";
+            dst += o->getName();
+            dst_id = o->getId();
         }
 
         if (i3!=srvrel->end()) {
             FWObject *o=*i3;
             if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-            srv+=o->getName();
+            srv += o->getName();
+            srv_id = o->getId();
         }
 
         int w=0;
@@ -1122,11 +1198,12 @@ string PolicyCompiler::debugPrintRule(Rule *r)
         
         str <<  setw(10-w)  << setfill(' ') << " ";
 
-        str <<  setw(18) << setfill(' ') << src.c_str();
-        str <<  setw(18) << setfill(' ') << dst.c_str();
-        str <<  setw(12) << setfill(' ') << srv.c_str();
+        str <<  setw(18) << setfill(' ') << src.c_str() << "(" << src_id << ")";
+        str <<  setw(18) << setfill(' ') << dst.c_str() << "(" << dst_id << ")";
+        str <<  setw(12) << setfill(' ') << srv.c_str() << "(" << srv_id << ")";
 
-        if (no==0) {
+        if (no==0)
+        {
             str <<  setw(9)  << setfill(' ') << rule->getActionAsString().c_str();
             str <<  setw(9)  << setfill(' ') << rule->getDirectionAsString().c_str();
             if (rule_iface!=NULL) str << " " << rule_iface->getName();
