@@ -41,7 +41,8 @@
 #include "fwbuilder/Rule.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/RuleSet.h"
-#include "fwbuilder/IPAddress.h"
+#include "fwbuilder/InetAddr.h"
+#include "fwbuilder/IPRoute.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/FWObjectDatabase.h"
@@ -73,14 +74,13 @@ int RoutingCompiler::prolog()
     temp_ruleset = new Routing();   // working copy of the routing
     fw->add( temp_ruleset );
 
-    
     list<FWObject*> l=routing->getByType(RoutingRule::TYPENAME);
     for (list<FWObject*>::iterator j=l.begin(); j!=l.end(); ++j) 
     {
 	Rule *r= Rule::cast(*j);
 	if (r->isDisabled()) continue;
-	r->setInterfaceId("");
-	r->setLabel( createRuleLabel("main", r->getPosition()) );
+	r->setInterfaceId(-1);
+	r->setLabel( createRuleLabel("", "main", r->getPosition()) );
 	combined_ruleset->add( r );
     }
 
@@ -105,8 +105,6 @@ string RoutingCompiler::debugPrintRule(Rule *r)
 {
     RoutingRule *rule=RoutingRule::cast(r);
 
-    FWOptions *ruleopt =rule->getOptionsObject();
-
     RuleElementRDst *dstrel=rule->getRDst();
     RuleElementRItf *itfrel=rule->getRItf();
     RuleElementRGtw *gtwrel=rule->getRGtw();
@@ -231,8 +229,8 @@ bool RoutingCompiler::emptyRDstAndRItf::processNext()
     RuleElementRGtw *gtwrel=rule->getRGtw();
     RuleElementRItf *itfrel=rule->getRItf();
 
-    if ( (FWReference::cast(itfrel->front())->getPointer())->getStr("name")=="Any" &&\
-         (FWReference::cast(gtwrel->front())->getPointer())->getStr("name")=="Any") {
+    if ( (FWReference::cast(itfrel->front())->getPointer())->getName()=="Any" &&\
+         (FWReference::cast(gtwrel->front())->getPointer())->getName()=="Any") {
         
         string msg;
         msg = "Gateway and interface are both empty in the rule " + rule->getLabel();
@@ -256,7 +254,7 @@ bool RoutingCompiler::singleAdressInRGtw::processNext()
     if( gtwrel->checkSingleIPAdress(o) == false) {
         
         string msg;
-        msg = "The object \"" + o->getStr("name") + "\" used as gateway in the routing rule "\
+        msg = "The object \"" + o->getName() + "\" used as gateway in the routing rule "\
             + rule->getLabel() + " has more than one interface resp. IP adress!";
         compiler->abort( msg.c_str() );
         
@@ -312,7 +310,7 @@ bool RoutingCompiler::validateNetwork::processNext()
     if( checkValidNetwork(o) == false) {
     
         string msg;
-        msg = "The object \"" + o->getStr("name") + "\" used as destination in the routing rule "\
+        msg = "The object \"" + o->getName() + "\" used as destination in the routing rule "\
             + rule->getLabel() + " has an invalid netmask!";
         compiler->abort( msg.c_str() );
         
@@ -321,24 +319,113 @@ bool RoutingCompiler::validateNetwork::processNext()
     
 }
 
+// the IP address of the gateway has to be in a local network of the firewall
+bool RoutingCompiler::reachableAddressInRGtw::checkReachableIPAddress(FWObject *o)
+{
+    // let's walk over all interfaces of this firewall
+    FWObjectTypedChildIterator j = compiler->fw->findByType(Interface::TYPENAME);
+
+    if( Host::cast(o) != NULL)
+    {
+        Host *host=Host::cast(o);
+        const InetAddr *ip_host = host->getAddressPtr();
+        for ( ; j!=j.end(); ++j )
+        {
+            Interface *i_firewall=Interface::cast(*j);
+            for(FWObjectTypedChildIterator fw_ips = 
+                    i_firewall->findByType(IPv4::TYPENAME);
+                fw_ips!=fw_ips.end(); ++fw_ips)
+            {
+                IPv4 *ipv4_obj_firewall = IPv4::cast(*fw_ips);
+
+                const InetAddr *addr = ipv4_obj_firewall->getAddressPtr();
+                const InetAddr *netm = ipv4_obj_firewall->getNetmaskPtr();
+                if (addr)
+                {
+                    InetAddrMask fw_net(*addr, *netm);
+                    if (fw_net.belongs(*ip_host))
+                        return true;
+                }
+            }
+        }
+        return false;
+
+    } else if( Interface::cast(o) != NULL)
+    {
+        Interface *gw_interface=Interface::cast(o);
+        const InetAddr *ip_gateway = gw_interface->getAddressPtr();
+
+        // walk over all interfaces of this firewall
+        for ( ; j!=j.end(); ++j )
+        {
+            Interface *if_firewall=Interface::cast(*j);
+            FWObjectTypedChildIterator addresses =
+                if_firewall->findByType(IPv4::TYPENAME);
+
+            // check all IPv4 addresses of this firewall interface
+            for ( ; addresses!=addresses.end(); ++addresses )
+            {
+                IPv4 *ipv4_obj_firewall = IPv4::cast(*addresses);
+                const InetAddr *addr = ipv4_obj_firewall->getAddressPtr();
+                const InetAddr *netm = ipv4_obj_firewall->getNetmaskPtr();
+                if (addr)
+                {
+                    InetAddrMask fw_net(*addr, *netm);
+                    if (fw_net.belongs(*ip_gateway))
+                        return true;
+                }
+            }
+        }
+        return false;
+
+    } else if( IPv4::cast(o) != NULL)
+    {
+        IPv4 *ipv4=IPv4::cast(o);
+        const InetAddr *ip_ipv4 = ipv4->getAddressPtr();
+
+        for ( ; j!=j.end(); ++j )
+        {
+            Interface *if_firewall=Interface::cast(*j);
+            FWObjectTypedChildIterator addresses =
+                if_firewall->findByType(IPv4::TYPENAME);
+
+            // check all IPv4 addresses of this firewall interface
+            for ( ; addresses!=addresses.end(); ++addresses ) {
+                IPv4 *ipv4_obj_firewall = IPv4::cast(*addresses);
+                const InetAddr *addr = ipv4_obj_firewall->getAddressPtr();
+                const InetAddr *netm = ipv4_obj_firewall->getNetmaskPtr();
+                if (addr)
+                {
+                    InetAddrMask fw_net(*addr, *netm);
+                    if (fw_net.belongs(*ip_ipv4))
+                        return true;
+                }
+            }
+        } return false;
+
+    } else return true;
+
+    return false;
+}
+
+
 //  the IP address of the gateway has to be in a local network of the firewall
-bool RoutingCompiler::reachableAdressInRGtw::processNext()
+bool RoutingCompiler::reachableAddressInRGtw::processNext()
 {
     RoutingRule *rule=getNext(); if (rule==NULL) return false;
-    
     tmp_queue.push_back(rule);
     
-    
     RuleElementRGtw *gtwrel=rule->getRGtw();
     FWObject *o = FWReference::cast(gtwrel->front())->getPointer();
               
-    if( gtwrel->checkReachableIPAdress(o) == false) {
-        
+    if( checkReachableIPAddress(o) == false)
+    {
         string msg;
-        msg = "The object \"" + o->getStr("name") + "\" used as gateway in the routing rule "\
-            + rule->getLabel() + " is not reachable, since not in any local network of the firewall!";
+        msg = "The object \"" + o->getName() +
+            "\" used as gateway in the routing rule "\
+            + rule->getLabel() +
+            " is not reachable, since not in any local network of the firewall";
         compiler->abort( msg.c_str() );
-        
     }
     return true;
 }
@@ -356,40 +443,45 @@ bool RoutingCompiler::contradictionRGtwAndRItf::processNext()
     FWObject *oRGtw = FWReference::cast(gtwrel->front())->getPointer();
     FWObject *oRItf = FWReference::cast(itfrel->front())->getPointer();
     
-    if (oRItf->getStr("name") == "Any") { return true; }
+    if (oRItf->getName() == "Any") { return true; }
     
     
-    if( Host::cast(oRGtw) != NULL || Interface::cast(oRGtw) != NULL || IPv4::cast(oRGtw) != NULL) {
+    if (Host::cast(oRGtw) != NULL ||
+        Interface::cast(oRGtw) != NULL ||
+        Address::cast(oRGtw)->dimension()==1)
+    {
 
-        IPAddress ip_interface;
+        const InetAddr* ip_interface = NULL;
 
-        if ( Host::cast(oRGtw) != NULL) {
+        if ( Host::cast(oRGtw) != NULL)
+        {
             Host *host=Host::cast(oRGtw);
-            ip_interface= host->getAddress();
-        } else if (Interface::cast(oRGtw) != NULL) {
+            ip_interface = host->getAddressPtr();
+        } else if (Interface::cast(oRGtw) != NULL)
+        {
             Interface *intf=Interface::cast(oRGtw);
-            ip_interface= intf->getAddress();
-        } else if (IPv4::cast(oRGtw) != NULL) {
-            IPv4 *ipv4=IPv4::cast(oRGtw);
-            ip_interface= ipv4->getAddress();
+            ip_interface = intf->getAddressPtr();
+        } else if (Address::cast(oRGtw)->dimension()==1)
+        {
+            Address *ipv4 = Address::cast(oRGtw);
+            ip_interface = ipv4->getAddressPtr();
         }
 
-        
-        list<FWObject*> obj_list = oRItf->getByType("IPv4");
-        for (list<FWObject*>::iterator i=obj_list.begin(); i!=obj_list.end(); ++i) 
+        if (ip_interface)
         {
-            IPv4 *firewall=IPv4::cast(*i); 
-            Netmask n_firewall = firewall->getNetmask();
-            IPAddress ip_firewall = firewall->getAddress();
-             
-            if ((n_firewall.to32BitInt() & ip_firewall.to32BitInt()) == (n_firewall.to32BitInt() & ip_interface.to32BitInt())) {
-                return true;
+            list<FWObject*> obj_list = oRItf->getByType(IPv4::TYPENAME);
+            for (list<FWObject*>::iterator i=obj_list.begin();
+                 i!=obj_list.end(); ++i) 
+            {
+                Address *addr = Address::cast(*i);
+                if (addr->belongs(*ip_interface))
+                    return true;
             }
         }
 
         string msg;
-        msg = "The object \"" + oRGtw->getStr("name") + "\" used as gateway in the routing rule "\
-            + rule->getLabel() + " is not in the same local network as the interface " + oRItf->getStr("name") + "!";
+        msg = "The object \"" + oRGtw->getName() + "\" used as gateway in the routing rule "\
+            + rule->getLabel() + " is not in the same local network as the interface " + oRItf->getName() + "!";
         compiler->abort( msg.c_str() );
 
     }
@@ -401,20 +493,35 @@ bool RoutingCompiler::contradictionRGtwAndRItf::processNext()
 bool RoutingCompiler::rItfChildOfFw::processNext()
 {
     RoutingRule *rule=getNext(); if (rule==NULL) return false;
-    
     tmp_queue.push_back(rule);
     
-    
-    RuleElementRItf *itfrel=rule->getRItf();
+    RuleElementRItf *itfrel = rule->getRItf();
     FWObject *o = FWReference::cast(itfrel->front())->getPointer();
+
+    /*
+     * check if an object 'o' is a child of the firewall being
+     * processed
+     */
+    FWObject* o_tmp = o;
+    bool same_fw = false;
+    while (o_tmp)
+    {
+        if (Firewall::cast(o_tmp) && o_tmp->getId()==compiler->fw->getId())
+        {
+            same_fw = true;
+            break;
+        }
+        o_tmp = o_tmp->getParent();
+    }
               
-    if( itfrel->checkItfChildOfThisFw(o) == false) {
-        
+    if (!same_fw)
+    {
         string msg;
-        msg = "The object \"" + o->getStr("name") + "\" used as interface in the routing rule "\
-            + rule->getLabel() + " is not a child of the firewall the rule belongs to!";
+        msg = "The object \"" + o->getName() + 
+            "\" used as interface in the routing rule " +
+            rule->getLabel() +
+            " is not a child of the firewall the rule belongs to!";
         compiler->abort( msg.c_str() );
-        
     } 
     return true;
 }
@@ -431,11 +538,11 @@ bool RoutingCompiler::competingRules::processNext()
     RuleElementRGtw *gtwrel=rule->getRGtw();
     FWObject *gtw = FWReference::cast(gtwrel->front())->getPointer();
      
-    RuleElementRDst *dstrel=rule->getRDst();
-    
     string metric = rule->getMetricAsString();
     string label  = rule->getSortedDstIds();
-    string combiId = gtw->getStr("id") + itf->getStr("id");
+    ostringstream ostr;
+    ostr << gtw->getId() << "_" << itf->getId();
+    string combiId = ostr.str();
     
     if( label == "") compiler->abort("Place 'createSortedDstIdsLabel()' before 'competingRules()' in the rule processor chain!");
     
@@ -545,11 +652,11 @@ bool RoutingCompiler::classifyRoutingRules::processNext()
         RuleElementRGtw *gtwrel=rule->getRGtw();
         FWObject *gtw = FWReference::cast(gtwrel->front())->getPointer();
         
-        RuleElementRDst *dstrel=rule->getRDst();
-        
         string metric  = rule->getMetricAsString();
         string label   = rule->getSortedDstIds();
-        string combiId = gtw->getStr("id") + itf->getStr("id");
+        ostringstream ostr;
+        ostr << gtw->getId() << "_" << itf->getId();
+        string combiId = ostr.str();
         
         if( label == "") compiler->abort("Place 'createSortedDstIdsLabel()' right before 'classifyRoutingRules()' in the rule processor chain!");
         
@@ -618,7 +725,9 @@ bool RoutingCompiler::createSortedDstIdsLabel::processNext()
     std::list<string> idList;
     for (FWObject::iterator it=dstrel->begin(); it!=dstrel->end(); ++it) {
         
-        idList.insert(idList.end(), (FWReference::cast(*it)->getPointer())->getStr("id"));
+        idList.insert(idList.end(),
+                      FWObjectDatabase::getStringId(
+                          (FWReference::cast(*it)->getPointer())->getId()));
     }
     idList.sort();
     for (std::list<string>::iterator it=idList.begin(); it!=idList.end(); ++it) {
